/*************************************************************************
 *
 *  Demonstrate how zooming can be performed with DAC module.
 *
 *
 *  KEY_COLOR, DCF0 and DCF1 defines must be set properly.
 *
 */

#include <imapi.h>
#include <time.h>

#define KEY_COLOR 8
#define DCF0 "rs170"
#define DCF1 "ccir"

void error(void *Param);


/*---------------------------------------------------------------------------
 *
 * Name:          main()
 *
 * Synopsis:      Main body.
 *
 * Returns:       none
 *
 */

void grabCopyZoom(void);
void grabZoom(void);
 
long dispMode, key = 1;
long display, dispXsize, dispYsize, Xsize, Ysize, index, clear = 0, iter, color = 0;
long dev[3], thr[3];
long osb[2], disp[2], offScreen[2], hostBuf[2], cam[2], dig[2];
long grabControl[2], copyControl[2];
long thr1, thr2, thr3, one = 1, two = 1;
long proc[5];
long con[3];
long winStop[2];
long xsize[2], ysize[2], band[2];
long xzoom[2] = {1, 1}, yzoom[2] = {1, 1};

long dispControl, zoomtype = 0;
int inSingleFrameMode = 1;

void main(int argc, char *argv[])
{
  
   if (argc > 1 && *argv[1] == '?')
   {
      printf("\nUsage: daczoom [options]\n");
      printf("               -nokey      Turn keying off\n");
      printf("               -clear      Clear buffers between grabs\n");
      printf("               -disable n  Disable the desired channel: 0 or 1\n");
      printf("               -xz0 n      Horizontal zooming factor for channel 0 (1, 2, 4)\n");
      printf("               -xz1 n      Horizontal zooming factor for channel 1 (1, 2, 4)\n");
      printf("               -yz0 n      Vertical zooming factor for channel 0 (1, 2, 4)\n");
      printf("               -yz1 n      Vertical zooming factor for channel 1 (1, 2, 4)\n");
      printf("               -zoomtype n 0:Zoom by copy, 1:Zoom while grabbing(horizontal only)\n");
      printf("               -cont       Continuous frame grab (for zoomtype 1 only)\n");
      exit(1);
   }
   else
      printf("Type 'macgrab ?' for help\n");

   for (index = 1; index < argc; index++)
   {
      if (!strcmp(argv[index], "-nokey"))
         key = 0;
      else if (!strcmp(argv[index], "-clear"))
         clear = 1;
      else if (!strcmp(argv[index], "-disable"))
      {
         long dis;

         sscanf(argv[index + 1], "%li", &dis);

         if (dis)
            two = 0;
         else if (!dis)
            one = 0;
      }
      else if (!strcmp(argv[index], "-xz0"))
         sscanf(argv[index + 1], "%li", &xzoom[0]);
      else if (!strcmp(argv[index], "-yz0"))
         sscanf(argv[index + 1], "%li", &yzoom[0]);
      else if (!strcmp(argv[index], "-xz1"))
         sscanf(argv[index + 1], "%li", &xzoom[1]);
      else if (!strcmp(argv[index], "-yz1"))
         sscanf(argv[index + 1], "%li", &yzoom[1]);
      else if (!strcmp(argv[index], "-zoomtype"))
         sscanf(argv[index + 1], "%li", &zoomtype);
      else if (!strcmp(argv[index], "-cont"))
         inSingleFrameMode = 0;
   }

   printf("Loading base board...\n");
   imAppCatchError(IM_DEFAULT, error, NULL);
   imDevAlloc(0, 0, NULL, IM_DEFAULT, &dev[0]);
   imThrAlloc(dev[0], 0, &thr[0]);
   imThrAlloc(dev[0], 0, &thr[1]);
   imThrAlloc(dev[0], 0, &thr[2]);
   imDigAlloc(thr[0], 0, 0, IM_DEFAULT, &dig[0]);
   imDigAlloc(thr[0], 0, 1, IM_DEFAULT, &dig[1]);

   if (zoomtype)
      grabZoom();
   else
      grabCopyZoom();


   imDigFree(thr[0], dig[0]);
   imDigFree(thr[0], dig[1]);
   imThrFree(thr[2]);
   imThrFree(thr[1]);
   imThrFree(thr[0]);
   imDevFree(thr[0]);
   
}



/*---------------------------------------------------------------------------
 *
 * Name:          grabCopyZoom()
 *
 * Synopsis:      Zoom while copying grabbed image
 *
 * Returns:       none
 *
 */
void grabCopyZoom(void)
{
   
   imSyncAlloc(thr[0], &osb[0]);
   imSyncAlloc(thr[0], &osb[1]);

   imBufAllocControl(thr[0], &grabControl[0]);
   imBufAllocControl(thr[0], &grabControl[1]);
   imBufAllocControl(thr[0], &copyControl[0]);
   imBufAllocControl(thr[0], &copyControl[1]);
   imBufAllocControl(thr[0], &dispControl);

   imCamAlloc(thr[0], DCF0, IM_DEFAULT, &cam[0]);
   imCamAlloc(thr[0], DCF1, IM_DEFAULT, &cam[1]);

   imBufPutField(thr[0], copyControl[0], IM_CTL_ZOOM_X, xzoom[0]);
   imBufPutField(thr[0], copyControl[1], IM_CTL_ZOOM_X, xzoom[1]);
   imBufPutField(thr[0], copyControl[0], IM_CTL_ZOOM_Y, yzoom[0]);
   imBufPutField(thr[0], copyControl[1], IM_CTL_ZOOM_Y, yzoom[1]);

   imCamInquire(thr[0], cam[0], IM_DIG_SIZE_X, &xsize[0]);
   imCamInquire(thr[0], cam[0], IM_DIG_SIZE_Y, &ysize[0]);
   imCamInquire(thr[0], cam[1], IM_DIG_SIZE_X, &xsize[1]);
   imCamInquire(thr[0], cam[1], IM_DIG_SIZE_Y, &ysize[1]);
   imCamInquire(thr[0], cam[0], IM_DIG_NUM_BANDS, &band[0]);
   imCamInquire(thr[0], cam[1], IM_DIG_NUM_BANDS, &band[1]);

   /* Adjust keying */
   if (key && ((imDispInquire(thr[0], 0, IM_DISP_VGA_MODE, &dispMode) == IM_SINGLE_SCREEN) || (dispMode == IM_DUAL_HEAD)))
   {
      imBufPutField(thr[0], dispControl, IM_DISP_KEY_MODE, IM_KEY_IN_RANGE);
      imBufPutField(thr[0], dispControl, IM_DISP_KEY_LOW, KEY_COLOR);
      imBufPutField(thr[0], dispControl, IM_DISP_KEY_HIGH, KEY_COLOR);
      imBufPutField(thr[0], dispControl, IM_DISP_MODE, IM_DISP_MONO);
      imDispControl(thr[0], 0, dispControl, IM_NOW);
   }

   /* Clear display */
   imBufChild(thr[0], IM_DISP, 0, 0, IM_ALL, IM_ALL, &display);
   imBufClear(thr[0], display, 0, 0);
   imBufInquire(thr[0], display, IM_BUF_SIZE_X, &dispXsize);
   imBufInquire(thr[0], display, IM_BUF_SIZE_Y, &dispYsize);

   winStop[0] = (dispXsize >> 1) - 1;
   winStop[1] = (dispXsize >> 1) - 1;
   
   if (one)
   {
      imBufPutField(thr[0], grabControl[1], IM_CTL_STOP_X, winStop[1]);
      
      imBufChild(thr[0], IM_DISP_RED, 0, 0, xsize[0], ysize[0], &offScreen[0]);
      imBufClear(thr[0], offScreen[0], 0, 0);
      if (two)
         imBufChild(thr[0], IM_DISP, 0, 0, winStop[0] + 1, ysize[0], &disp[0]);
      else
         imBufChild(thr[0], IM_DISP, 0, 0, xsize[0], ysize[0], &disp[0]);
      imBufAlloc(thr[0], xsize[0], ysize[0], 1, IM_BYTE, IM_HOST, &hostBuf[0]);
   }

   if (two)
   {
      imBufPutField(thr[0], grabControl[0], IM_CTL_STOP_X, winStop[0]);
      
      imBufChild(thr[0], IM_DISP_GREEN, 0, 0, xsize[1], ysize[1], &offScreen[1]);
      imBufClear(thr[0], offScreen[1], 0, 0);
      if (one)
         imBufChild(thr[0], IM_DISP, winStop[0] + 1, 0, winStop[1] + 1, ysize[1], &disp[1]);
      else
         imBufChild(thr[0], IM_DISP, 0, 0, xsize[1], ysize[1], &disp[1]);
      imBufAlloc(thr[0], xsize[1], ysize[1], 1, IM_BYTE, IM_HOST, &hostBuf[1]);
   }   

   imBufClear(thr[0], display, 0, 0);
   imSyncHost(thr[0], 0, IM_COMPLETED);

   iter = 0;
   printf("Grab from two digital sources\n");
   while (!kbhit())
   {
      printf("%d\r", iter++);

      /* Start a grab in each channel, simultaneously */
      if (one) imDigGrab(thr[0], dig[0], cam[0], offScreen[0], 1, grabControl[0], osb[0]);
      if (two) imDigGrab(thr[1], dig[1], cam[1], offScreen[1], 1, grabControl[1], osb[1]);  

      /* Wait for completion */
      if (one) imSyncHost(thr[2], osb[0], IM_COMPLETED);
      if (two) imSyncHost(thr[2], osb[1], IM_COMPLETED);

      if (one)
      {
         imBufCopy(thr[0], offScreen[0], hostBuf[0], 0, osb[0]);   
	      imSyncHost(thr[2], osb[0], IM_COMPLETED);
         imBufCopyPCI(thr[1], hostBuf[0], disp[0], copyControl[0], osb[0]);   
	      imSyncHost(thr[2], osb[0], IM_COMPLETED);
      }

      if (two)
      {
         imBufCopy(thr[0], offScreen[1], hostBuf[1], 0, osb[1]);   
	      imSyncHost(thr[2], osb[1], IM_COMPLETED);
         imBufCopyPCI(thr[1], hostBuf[1], disp[1], copyControl[1], osb[1]);   
	      imSyncHost(thr[2], osb[1], IM_COMPLETED);
      }

      if (clear)
      {
         imBufClear(thr[0], display, 0, 0);
         imSyncHost(thr[0], 0, IM_COMPLETED);
      }
   }
   
   if (one)
   {
      imBufFree(thr[0], offScreen[0]);
      imBufFree(thr[0], disp[0]);
      imBufFree(thr[0], hostBuf[0]);
   }
   if (two)
   {
      imBufFree(thr[1], offScreen[1]);
      imBufFree(thr[1], disp[1]);
      imBufFree(thr[1], hostBuf[1]);
   }

   imBufFree(thr[0], grabControl[0]);
   imBufFree(thr[0], copyControl[0]);
   imBufFree(thr[0], grabControl[1]);
   imBufFree(thr[0], copyControl[1]);
   imBufFree(thr[0], dispControl);
}



/*---------------------------------------------------------------------------
 *
 * Name:          grabZoom()
 *
 * Synopsis:      Zoom while grabbing
 *
 * Returns:       none
 *
 */
void grabZoom(void)
{
   
   imSyncAlloc(thr[0], &osb[0]);
   imSyncAlloc(thr[0], &osb[1]);

   imBufAllocControl(thr[0], &grabControl[0]);
   imBufAllocControl(thr[0], &grabControl[1]);
   imBufAllocControl(thr[0], &dispControl);
   
   if (key && ((imDispInquire(thr[0], 0, IM_DISP_VGA_MODE, &dispMode) == IM_SINGLE_SCREEN) || (dispMode == IM_DUAL_HEAD)))
   {
      imBufPutField(thr[0], dispControl, IM_DISP_KEY_MODE, IM_KEY_IN_RANGE);
      imBufPutField(thr[0], dispControl, IM_DISP_KEY_LOW, KEY_COLOR);
      imBufPutField(thr[0], dispControl, IM_DISP_KEY_HIGH, KEY_COLOR);
      imBufPutField(thr[0], dispControl, IM_DISP_ZOOM_Y, yzoom[0]);
      imDispControl(thr[0], 0, dispControl, IM_NOW);
   }

   imBufPutField(thr[0], grabControl[0], IM_CTL_ZOOM_X, xzoom[0]);
   imBufPutField(thr[1], grabControl[1], IM_CTL_ZOOM_X, xzoom[1]);

   imCamAlloc(thr[0], DCF0, IM_DEFAULT, &cam[0]);
   imCamAlloc(thr[0], DCF1, IM_DEFAULT, &cam[1]);

   imCamInquire(thr[0], cam[0], IM_DIG_SIZE_X, &xsize[0]);
   imCamInquire(thr[0], cam[0], IM_DIG_SIZE_Y, &ysize[0]);
   imCamInquire(thr[0], cam[1], IM_DIG_SIZE_X, &xsize[1]);
   imCamInquire(thr[0], cam[1], IM_DIG_SIZE_Y, &ysize[1]);
   imCamInquire(thr[0], cam[0], IM_DIG_NUM_BANDS, &band[0]);
   imCamInquire(thr[0], cam[1], IM_DIG_NUM_BANDS, &band[1]);


   imBufChild(thr[0], IM_DISP, 0, 0, IM_ALL, IM_ALL, &display);
   imBufClear(thr[0], display, 0, 0);
   imBufInquire(thr[0], display, IM_BUF_SIZE_X, &dispXsize);
   imBufInquire(thr[0], display, IM_BUF_SIZE_Y, &dispYsize);

   winStop[0] = (dispXsize >> 1) - 1;
   winStop[1] = (dispXsize >> 1) - 1;
   
   if (one)
   {
      imBufPutField(thr[0], grabControl[1], IM_CTL_STOP_X, winStop[1]);
      
      if (two)
         imBufChild(thr[0], IM_DISP, 0, 0, winStop[0] + 1, ysize[0] >> (yzoom[0]>>1), &disp[0]);
      else
         imBufChild(thr[0], IM_DISP, 0, 0, xsize[0], ysize[0] >> (yzoom[0]>>1), &disp[0]);
   }

   if (two)
   {
      imBufPutField(thr[0], grabControl[0], IM_CTL_STOP_X, winStop[0]);
      
      if (one)
         imBufChild(thr[0], IM_DISP, winStop[0] + 1, 0, winStop[1] + 1, ysize[1] >> (yzoom[0]>>1), &disp[1]);
      else
         imBufChild(thr[0], IM_DISP, 0, 0, xsize[1], ysize[1] >> (yzoom[0]>>1), &disp[1]);
   }   

   imBufClear(thr[0], display, 0, 0);
   imSyncHost(thr[0], 0, IM_COMPLETED);

   iter = 0;

   if(!inSingleFrameMode)
   {
	   printf("In continuous frame grab mode\n");
	   imDigGrab(thr[0], dig[0], cam[0], disp[0], IM_CONTINUOUS, grabControl[0], osb[0]);
   	imSyncHost(thr[2], osb[0], IM_STARTED);
	   imDigGrab(thr[1], dig[1], cam[1], disp[1], IM_CONTINUOUS, grabControl[1], osb[1]);
   	imSyncHost(thr[2], osb[1], IM_STARTED);
   }
   else
	   printf("In single frame grab mode\n");
   while (!kbhit())
   {
		if(inSingleFrameMode)
			{
				
      	   imDigGrab(thr[0], dig[0], cam[0], disp[0], 1, grabControl[0], osb[0]);
      	   imDigGrab(thr[1], dig[1], cam[1], disp[1], 1, grabControl[1], osb[1]);
         	imSyncHost(thr[2], osb[0], IM_COMPLETED);
         	imSyncHost(thr[2], osb[1], IM_COMPLETED);
				iter++;
			}
   }
   imThrHalt(thr[0], IM_FRAME);
   imThrHalt(thr[1], IM_FRAME);
   
   imSyncHost(thr[0], 0, IM_COMPLETED);
   imSyncHost(thr[1], 0, IM_COMPLETED);

   if (one)
   {
      imBufFree(thr[0], disp[0]);
   }
   if (two)
   {
      imBufFree(thr[1], disp[1]);
   }
   imBufFree(thr[0], grabControl[0]);
   imBufFree(thr[0], grabControl[1]);
   imBufFree(thr[0], dispControl);


}



/*---------------------------------------------------------------------------
 *
 * Name:          error()
 *
 * Synopsis:      Error handler.
 *
 * Returns:       none
 *
 */

void error(void *Param)
{
    char Error[IM_ERR_SIZE];

    /* Just to prevent warnings */
    Param = Param;

    imAppGetError(IM_ERR_MSG_FUNC, Error);
    printf("\n%s\n", Error);

    exit(0);
}



