/********************************************************************
*
* A very simple Genesis program to toggle user bits. 
* this example needs the following to work properly:
* 1) An rs170 camera.
* 2) the 68 pin adaptor board
*
********************************************************************/

#include <stdio.h>

#include "imapi.h"

/* Prototype for error handler function */
void ErrHandler(void *Param);

void main(void)
{
    long Device;        /* Genesis device */
    long Camera;        /* Genesis camera */
    long Thread;        /* Thread to execute all functions */
    long Success = 1;   /* Flag to record success or failure */
    long ScreenBuf, DispBuf;
    long KeyVal =6;
    long digCtrlBuf;
    long dispMode;

    // Enable 1 or any of these 5 options to illustrate how to change
    // the user bits.
    long UseCamContinuous = 0;
    long UseCamSnap = 0;
    long UseDig = 0;
    long UseDigContinuous = 0;
    long UseDigContinuousLoop = 1;

    printf("Allocating the Genesis system...\n");
    
    /* Establish an error handler */
    imAppCatchError(IM_DEFAULT, ErrHandler, (void *) &Success);
    
    /* Allocate the board and a thread */
    imDevAlloc(0, 0, NULL, IM_DEFAULT, &Device);
    imThrAlloc(Device, 0, &Thread);

    imBufChild(Thread, IM_DISP, 0, 0, IM_ALL, IM_ALL, &ScreenBuf);
    imBufClear(Thread, ScreenBuf, 128, 0);
    imBufChild(Thread, IM_DISP, 0, 0, 640, 480, &DispBuf);

    // set the keying 
    if ((imDispInquire(Thread, 0, IM_DISP_VGA_MODE, &dispMode) == IM_SINGLE_SCREEN) || (dispMode == IM_DUAL_HEAD))
    {
        if (imDispInquire(Thread, 0, IM_DISP_KEY_MODE, NULL) == IM_KEY_OFF)
        {
            imBufPutField(Thread, ScreenBuf, IM_DISP_KEY_MODE, IM_KEY_IN_RANGE);
            imBufPutField(Thread, ScreenBuf, IM_DISP_KEY_LOW, KeyVal);
            imBufPutField(Thread, ScreenBuf, IM_DISP_KEY_HIGH, KeyVal);
            imDispControl(Thread, 0, ScreenBuf, IM_FRAME);
        }
    }
    
    if (UseCamContinuous == 1)
    {
        printf("Using imCamControl while grabbing continuously\n");
        
        imCamAlloc(Thread, "rs170.dcf", IM_DEFAULT, &Camera);
        imCamControl(Thread, Camera, IM_DIG_USER_OUT_FORMAT, IM_TTL);
        imDigGrab(Thread, 0, Camera, DispBuf, IM_CONTINUOUS, 0, 0);
        printf("Press enter to set user bits to '1'\n");
        getch();
        
        imCamControl(Thread,  Camera, IM_DIG_USER_OUT + IM_BIT0, 1);
        imCamControl(Thread,  Camera, IM_DIG_USER_OUT + IM_BIT1, 1);
        imThrHalt(Thread, IM_FRAME);
       imDigGrab(Thread, 0, Camera, DispBuf, IM_CONTINUOUS, 0, 0);
             
        printf("Press enter to set user bits to '0'\n");
        getch();
        imCamControl(Thread,  Camera, IM_DIG_USER_OUT + IM_BIT0, 0);
        imCamControl(Thread,  Camera, IM_DIG_USER_OUT + IM_BIT1, 0);
        imThrHalt(Thread, IM_FRAME);
        imDigGrab(Thread, 0, Camera, DispBuf, IM_CONTINUOUS, 0, 0);
             
        printf("Press enter to go to next part.\n");
        getch();
        imThrHalt(Thread, IM_FRAME);
        imCamFree(Thread, Camera);
    }

    if (UseCamSnap == 1)
    {
        printf("Using CamControl and doing snap shot grab\n");
        imCamAlloc(Thread, "rs170.dcf", IM_DEFAULT, &Camera);
        imCamControl(Thread, Camera, IM_DIG_USER_OUT_FORMAT, IM_TTL);

        printf("Press enter to set user bits to '1'\n");
        getch();
        imCamControl(Thread,  Camera, IM_DIG_USER_OUT + IM_BIT0, 1);
        imCamControl(Thread,  Camera, IM_DIG_USER_OUT + IM_BIT1, 1);
        imDigGrab(Thread, 0, Camera, DispBuf, 1, 0, 0);
        
        printf("Press enter to set user bits to '0'\n");
        getch();
        imCamControl(Thread,  Camera, IM_DIG_USER_OUT + IM_BIT0, 0);
        imCamControl(Thread,  Camera, IM_DIG_USER_OUT + IM_BIT1, 0);
        imDigGrab(Thread, 0, Camera, DispBuf, 1, 0, 0);
        
        printf("Press enter to go to next part.\n");
        getch();
        imCamFree(Thread, Camera);
    }

    if (UseDig == 1)
    {
        printf("Using DigControl without allocating Camera\n");
        imDigControl(Thread, 0, IM_DIG_USER_OUT_FORMAT, IM_TTL);
        
        printf("Press enter to set user bits to '1'\n");
        getch();
        imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT0, 1);
        imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT1, 1);
        
        printf("Press enter to set user bits to '0'\n");
        getch();
        imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT0, 0);
        imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT1, 0);
        
        printf("Press enter to end program.\n");
        getch();
    }

    if (UseDigContinuous == 1)
    {
        printf("Using DigControl while grabbing continuously\n");
        imCamAlloc(Thread, "rs170.dcf", IM_DEFAULT, &Camera);
        imBufAllocControl(Thread, &digCtrlBuf);
        imBufPutField(Thread, digCtrlBuf, IM_CTL_GRAB_MODE, IM_ASYNCHRONOUS);

        // 1) Only one of imCamControl or imDigControl to control the output format
        //    can be used.
        // 2) CamControl only works before the DigGrab is called
        //imCamControl(Thread, Camera, IM_DIG_USER_OUT_FORMAT, IM_TTL);

        imDigGrab(Thread, 0, Camera, DispBuf, IM_CONTINUOUS, digCtrlBuf, 0);
        
        printf("Press enter to set user bits to '1'\n");
        getch();

        // 1) Only one of imCamControl or imDigControl to control the output format
        //    can be used.
        // 2) DigControl only works after the DigGrab is called
        imDigControl(Thread, 0, IM_DIG_USER_OUT_FORMAT, IM_TTL);

        // set to '1'
        imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT0, 1);
        imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT1, 1);
        
        printf("Press enter to set user bits to '0'\n");
        getch();
        imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT0, 0);
        imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT1, 0);
        
        printf("Press enter to end program.\n");
        getch();
        imThrHalt(Thread, IM_FRAME);
        imCamFree(Thread, Camera);
    }

    if (UseDigContinuousLoop == 1)
    {
        printf("Using DigControl while grabbing continuously - Toggle speed test\n");
        imCamAlloc(Thread, "rs170.dcf", IM_DEFAULT, &Camera);
        imBufAllocControl(Thread, &digCtrlBuf);
        imBufPutField(Thread, digCtrlBuf, IM_CTL_GRAB_MODE, IM_ASYNCHRONOUS);

        imCamControl(Thread, Camera, IM_DIG_USER_OUT_FORMAT, IM_TTL);
        imDigGrab(Thread, 0, Camera, DispBuf, IM_CONTINUOUS, digCtrlBuf, 0);

        printf("Press enter to end program.\n");
        while (!kbhit())
        {
            imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT0, 1);
            imDigControl(Thread,  0, IM_DIG_USER_OUT + IM_BIT0, 0);
        }
        
        imThrHalt(Thread, IM_FRAME);
        imCamFree(Thread, Camera);
    }



    imThrFree(Thread);
    imDevFree(Device);
}


void ErrHandler(void *Success)
{
    char Error[IM_ERR_SIZE];
    /*
    * Get the error message and print it. Don't reset the error
    * because we only want the first to be printed.
    */
    imAppGetError(IM_ERR_MSG_FUNC, Error);
    printf("%s\n", Error);
    
    /* Record that the error occurred */
    *(long *)Success = 0;
}




