; *******************************************************************
; *
; * COPYRIGHT (c) 1995 Matrox Electronic Systems Ltd.
; * All Rights Reserved
; *
; *******************************************************************
;
; *******************************************************************
; *
; * Synopsis:        This script will execute morphological operation
; *                  on packed binary images
; *
; * Used Functions:  imBinConvert imBinMorphic
; *
; * Designed:        AJONG 
; *
; * Date:            17/05/95
; *
; * Notes:           This Example will demonstrate 
; *                  1) How to convert between integer buffers and
; *                     packed binary buffer,
; *                  2) Apply morphological operation with a
; *                     predefined kernel, 
; *                  3) Create a user defined  kernel to perform  
; *                     binary morphological operations
; *                  4) IM_MATCH operation.
; *
; *******************************************************************


   long    dev
   long    thread
   long    SrcBuf
   long    DispBuf1
   long    DispBuf2
   long    DispBuf3
   long    Buf1
   long    Buf2
   long    BinSrcBuf
   long    BinDstBuf1
   long    BinDstBuf2
   long    BinKerBuf
   long    BinChildBuf
   long    ShortBuf
   long    errorNum
   setstr  errorMsg "                                                             "
   setstr  errorFunc "                                                            "
   

   
   ; Allocate the device and thread
   imDevAlloc 0 0 null IM_DEFAULT &dev
   imThrAlloc dev 0 &thread

   
   ; Alloc the ubyte buffers
   imBufAlloc thread 512 480 1 IM_UBYTE IM_PROC &SrcBuf
   imBufAlloc thread 512 480 1 IM_UBYTE IM_PROC &Buf1
   imBufAlloc thread 512 480 1 IM_UBYTE IM_PROC &Buf2

   ; Alloc the packed binary buffer
   imBufAlloc thread 512 480 1 IM_BINARY IM_PROC &BinSrcBuf
   imBufAlloc thread 512 480 1 IM_BINARY IM_PROC &BinDstBuf1
   imBufAlloc thread 512 480 1 IM_BINARY IM_PROC &BinDstBuf2

   ; Alloc the display child
   imBufChild thread IM_DISP_RED 0   0 512 480 &DispBuf1
   imBufChild thread IM_DISP_RED 512 0 512 480 &DispBuf2
   imBufChild thread IM_DISP_RED 0 480 512 480 &DispBuf3
   

   ; Check and Display any errors which might have occured
   execcl "test_err.cl" thread
   

   
   
   ;;---------------------
   ;; BinConvert example |
   ;;---------------------
   
   ; load the image "im2team.tif"
   imBufLoad thread "im2team.tif" IM_TIFF SrcBuf
   imBufCopy thread SrcBuf DispBuf1 0 0
   pause "Original image... Press any key to continue" 

   
   ; convert to binary image
   imBinConvert thread SrcBuf BinSrcBuf IM_GREATER 0x7F 0 0 
   ; convert back to integer image
   imBinConvert thread BinSrcBuf Buf2 IM_DEFAULT 0x0 0xFF 0 
   imBufCopy thread Buf2 DispBuf2 0 0
   pause "Binarized image... Press any key to continue" 
   

   
   ;;-------------------------------------
   ;; BinMorphic with pre-defined kernel |
   ;;-------------------------------------
   
   ; use source buffer as a control buffer
   imBufPutField thread SrcBuf IM_CTL_OVERSCAN IM_REPLACE
   imBufPutField thread SrcBuf IM_CTL_OVERSCAN_VAL 0x0

   
   ; copy bin image to another bin buffer
   imBufCopy thread BinSrcBuf BinDstBuf1 0 0
   
   
   ; perform a BinMorphic
   imBinMorphic thread BinDstBuf1 BinDstBuf2 IM_3X3_RECT_1 IM_ERODE 1 SrcBuf 0      
   imBinConvert thread BinDstBuf2 Buf2 IM_DEFAULT 0x0 0xFF 0
   imBufCopy thread Buf2 DispBuf2 0 0      
   pause "ERODE with 1 iteration...  Press any key to continue."


   ; perform a BinMorphic
   imBinMorphic thread BinDstBuf1 BinDstBuf2 IM_3X3_RECT_1 IM_DILATE 3 SrcBuf 0      
   imBinConvert thread BinDstBuf2 Buf2 IM_DEFAULT 0x0 0xFF 0
   imBufCopy thread Buf2 DispBuf2 0 0  
   pause "DILATE with 3 iterations...  Press any key to continue."
   
      
   ; perform a BinMorphic
   imBinMorphic thread BinDstBuf1 BinDstBuf2 IM_3X3_RECT_1 IM_THIN IM_IDEMPOTENCE SrcBuf 0      
   imBinConvert thread BinDstBuf2 Buf2 IM_DEFAULT 0x0 0xFF 0
   imBufCopy thread Buf2 DispBuf2 0 0      
   pause "THIN with IM_IDEMPOTENCE...  Press any key to continue."


   
   ;;----------------------------------------
   ;; BinMorphic with a user-defined kernel |
   ;;----------------------------------------

   ;
   ; Let's now create an arbitary kernel 7 by 7 with type SHORT (2bytes)...
   ;

   allocarray bin_ker_array (7*7*2)
   execcl "bin7_7.cl"
   imBufAlloc thread 7 7 1 IM_SHORT IM_PROC &BinKerBuf
   imBufPut thread BinKerBuf [getarray bin_ker_array]
   

   ; perform a BinMorphic
   imBinMorphic thread BinDstBuf1 BinDstBuf2 BinKerBuf IM_THIN 1 SrcBuf 0      
   imBinConvert thread BinDstBuf2 Buf2 IM_DEFAULT 0x0 0xFF 0
   imBufCopy thread Buf2 DispBuf2 0 0      
   pause "THIN with user defined kernel (1 iteration)...  Press any key to continue."

   
   ;;------------------------------------------------
   ;; IM_MATCH operation with a user-defined kernel |
   ;;------------------------------------------------

   ;
   ; Let create a child buffer of a src image and use it as
   ; a kernel buffer for a perfect match.
   ;
   
   ; kernel buffer
   imBufChild thread BinSrcBuf 100 128 128 128 &BinChildBuf
   imBufClear thread Buf2 0 0
   imBinConvert thread BinChildBuf Buf2 IM_DEFAULT 0x0 0xFF 0
   imBufCopy thread Buf2 DispBuf3 0 0  
   pause "Kernel Buffer to MATCH (128x128). Press any key to continue..."
   
   
   ; Alloc result buffer to hold integer match value
   imBufAlloc thread 512 480 1 IM_USHORT IM_PROC &ShortBuf

   echo "Ignore the upcomming timeout message and wait..."
   ; perform a BinMorphic
   imBinMorphic thread BinSrcBuf ShortBuf BinChildBuf IM_MATCH 0 SrcBuf 0      

   
   ; map the max vaule of MATCH to the max of ubyte
   imIntMonadic thread ShortBuf 65.0 Buf2 IM_DIV 0  
   ; enhance the contrast where the max are located
   imIntHistogramEqualize thread Buf2 Buf2 256 IM_UNIFORM 0 200 255 IM_DEFAULT 0  
   imBufCopy thread Buf2 DispBuf2 0 0  
   pause "MATCH found at the brightest area. Press any key to continue..."
      
     
   
   ;
   ; Free up all buffers threads and deallocate the device
   ;
   imBufFree thread ShortBuf
   imBufFree thread BinChildBuf
   imBufFree thread BinKerBuf
   imBufFree thread BinDstBuf2
   imBufFree thread BinDstBuf1
   imBufFree thread BinSrcBuf
   imBufFree thread DispBuf3
   imBufFree thread DispBuf2
   imBufFree thread DispBuf1
   imBufFree thread SrcBuf 
   imBufFree thread Buf1
   imBufFree thread Buf2
   imThrFree thread
   imDevFree dev
   
   
   ; free all used variables
   freearray bin_ker_array
   freedev
   freethread
   freeSrcBuf
   freeDispBuf1
   freeDispBuf2
   freeDispBuf3
   freeBinSrcBuf
   freeBinDstBuf1
   freeBinDstBuf2
   freeBinKerBuf
   freeBinChildBuf
   freeShortBuf
   freeBuf1
   freeBuf2
   freeerrorNum
   clearstr errorMsg 
   clearstr errorFunc
