; *******************************************************************
; *
; * COPYRIGHT (c) 1995 Matrox Electronic Systems Ltd.
; * All Rights Reserved
; *
; *******************************************************************
;
; *******************************************************************
; *
; * Synopsis:        This script will perform a distance operation
; *                  on a child buffer.
; *
; * Used Functions:  imIntDistance
; *
; * Designed:        bfournie
; *
; * Date:            16/05/95
; *
; * Notes:           The test image is a non-null image with few pixels
; *                  with zero value in the middle. The child buffer
; *                  is growing aroung the middle of the test image, 
; *                  and a distance is calculated on it.
; *
; *******************************************************************



      ; Declare internal variables

      long dev
      long thread

      long Src_buf
      long Src_buf_child
      long Dst_buf
      long Dst_tmp
      
      long Disp_Src
      long Disp_Dst
      
      long cnt        

      ; Allocate the device and thread
      imdevalloc 0 0 null im_default &dev
      imthralloc dev 0 &thread



      ; Allocate Source processing buffer
      imbufalloc thread 512 512 1 im_byte im_proc &Src_buf
      imbufchild thread Src_buf 254 254 2 2 &Src_buf_child
      
      
      
      ; ... and generate a Source test image 
      ;     ( non-null image with a hole at 254,254)
      imbufclear thread Src_buf 0x51 0
      imbufclear thread Src_buf_child 0 0
      
      
      
      ; Move Source child buffer for future usage
      imbufchildmove thread Src_buf_child -4 -4 2 2 
      
      
      
      ; Allocate Destination buffer
      ; (in a 16 bits buffer to prevent overflow)
      imbufalloc thread 512 512 1 im_short im_proc &Dst_buf
      imbufclear thread Dst_buf 0 0 
      imbufalloc thread 512 512 1 im_byte im_proc &Dst_tmp
      
      
      
      ; Use the Source buffer to pass control field
      imbufputfield thread Src_buf im_ctl_overscan_val im_replace_min
      
      
      
      ; Allocate Display buffer
      imbufchild thread im_disp_red 0 0 512 512  &Disp_Src
      imbufchild thread im_disp_red 512 0 512 512  &Disp_Dst
      
      
      ; Verify that no error occurs
      execcl "test_err.cl" thread   
      
      
      echo  " Operation to do: imIntDistance of child buffer of various size"
      echo  " Press a key to end "
            
      setcnt 1
      
      while (cnt < 40)
         {   
         
         imbufchildmove thread Src_buf_child -4 -4 ([getcnt] *8) ([getcnt] *8) 
         

         ; Perform the Distance operation 
         imintdistance thread Src_buf_child Dst_buf im_chessboard Src_buf 0
      
      
         ; Convert the 16 bits result in an 8 bits buffer
         ; to display result at the screen
         imintconvert thread  Dst_buf Dst_tmp im_default 0
         
         
         ; Copy the Src and Dst buffer in the Display 
         imbufcopy thread Src_buf_child Disp_Src 0 0
         imbufcopy thread Dst_tmp Disp_Dst 0 0
         
         
         echoln "."
         
         setcnt ([getcnt] + 1)
         
         if( [keyhit] > 0 )
           {
           setcnt 50
           }
         
         }
         
      echo " "
         
         
      ;
      ; Free up all buffers threads and deallocate the device
      ;
      imbuffree thread Src_buf
      imbuffree thread Src_buf_child
      
      imbuffree thread Dst_buf
      imbuffree thread Dst_tmp
       
      imbuffree thread Disp_Src
      imbuffree thread Disp_Dst
       
      imthrfree thread
      imdevfree dev
      
      
      
      ; free all used variables
      freedev
      freethread
      
      freeSrc_buf
      freeSrc_buf_child
      freeDst_buf
      freeDst_tmp
      
      freeDisp_Src
      freeDisp_Dst
