# --------------------------------------------------------------------------
function GetEnviron(var)
{
   if (var in ENVIRON)
      return(ENVIRON[var])

   _var_ = tolower(var)
   for (varndx in ENVIRON)
      {
      if (tolower(varndx) == _var_)
         return(ENVIRON[varndx])
      }

   return("")
}

BEGIN {
   if (ARGC < 1)
      {
      printf "Usage: gawk -f strpdef.awk [for_lib_file]"
      }

   LibName = tolower(ARGV[1])
   ARGV[1] = ""

   gsub(/\..*/, "", LibName)

   DefName = LibName".def"
   DexName = LibName".dex"
   TmpName = GetEnviron("RAMDRIVE")"obj\\"GetEnviron("OBJDIR")"\\tmp.def"
   LibName = LibName".dll"

   print "\nChecking DEF file "DefName > "CON"

   if (ifexist(DefName) == 0)
   {
      printf "File %s is not available\n", DefName
      exit 1
   }

   if (ifexist(DexName) == 0)
   {
      printf "File %s is not available\n", DexName
      exit 1
   }

   DexEntry = 0
   Level = 0

   while (validLine = getline<DexName > 0)
   {
      if (Level == 1 && $3 != "")
      {
         DexTable[DexEntry,0] = $3
         DexTable[DexEntry,1] = 0
         DexEntry++
      }

      else if (Level == 2 && $4 != "")
      {
         DexTable[DexEntry,0] = $4
         DexTable[DexEntry,1] = 0
         DexEntry++
      }

      else if (!Level && match($0, /ordinal.*hint.*RVA/) != 0)    # VC 5.x
         Level = 2

      else if (!Level && match($0, /ordinal.*hint/) != 0)         # VC 4.x
         Level = 1
   }

   close(DexName)


   OldDefEntry = 0
   DefEntry = 0
   Level = 0

   while (validLine = getline<DefName > 0)
   {
      if (Level == 0 && index($0, "EXPORTS") != 0)
         Level = 1

      if (Level == 1 && index($0, "; Obsolete entries") != 0)
         Level = 2

      if (Level < 3 && index($2, "@") != 0)
         Level = 3


      if (Level == 0)
         # Header of File
         printf "%s\n", $0 >TmpName

      if (Level == 1 && index($0, "EXPORTS") != 0)
         # EXPORTS
         printf "%s\n\n", $0 >TmpName

      else if (Level == 2 && $0 != "")
         OldDeftable[OldDefEntry++] = $0

      else if (Level == 3 && index($1, ";") != 0 && index($2, "@") != 0)
      {
         gsub(/^ *;/, "", $0)
         OldDeftable[OldDefEntry++] = ";     "$0
      }

      else if (Level == 3)
      {
         if ($0 != "")
         {
            DefTable[DefEntry,0] = $1
            DefTable[DefEntry,1] = $2
            gsub(/@/, "", DefTable[DefEntry,1])
         }

         else
         {
            DefTable[DefEntry,0] = " "
            DefTable[DefEntry,1] = -1
         }

         DefEntry++
      }
   }

   close(DefName)

   FuncRemoved = 0

   for (ii=0; ii<DefEntry; ii++)
   {
      for (jj=0; jj<DexEntry; jj++)
      {
         if (DexTable[jj,1] == 0 && (DexTable[jj,0] == DefTable[ii,0]))
         {
            DexTable[jj,1] = DefTable[ii,1]
            break
         }
      }

      if (jj == DexEntry && DefTable[ii,0] != " " && DefTable[ii,1] != " ")
      {
         OldDeftable[OldDefEntry++] = sprintf(";     %-69s@%-3d", DefTable[ii,0], DefTable[ii,1])

         if (!FuncRemoved)
         {
            printf "\nOld function(s) removed from %s:\n\n", DefName > "CON"
            FuncRemoved = 1
         }

         printf "    %s\n", DefTable[ii,0] > "CON"

         # Invalidate that entry
         DefTable[ii,0] = 0
         DefTable[ii,1] = 0
      }
   }

   for (ii=0; ii<OldDefEntry; ii++)
   {
      if (ii == 0 && index(OldDeftable[ii], "; Obsolete entries") == 0)
      {
         printf "; Obsolete entries\n" >TmpName
         printf ";\n" >TmpName
      }

      printf "%s\n", OldDeftable[ii] >TmpName
   }

   printf "\n" >TmpName


   for (ii=0; ii<DefEntry; ii++)
   {
      if (DefTable[ii,1] == -1)
         printf "\n" >TmpName

      else if (DefTable[ii,1] != 0)
         printf "      %-69s@%-3d\n", DefTable[ii,0], DefTable[ii,1] >TmpName
   }

   NewDefEntryIndex = 800

   for (ii=0; ii<DexEntry; ii++)
   {
      if (DexTable[ii,1] == 0)
      {
         if (NewDefEntryIndex == 800)
            printf "\nNew function(s) added to %s:\n\n", DefName > "CON"

         printf "    %s\n", DexTable[ii,0] > "CON"

         printf "      %-69s@%-3d\n", DexTable[ii,0], NewDefEntryIndex++ >TmpName
      }
   }

   close(TmpName)



   if (FuncRemoved || NewDefEntryIndex != 800)
      printf "\n" > "CON"

   print "DEF file updated\n" > "CON"

   exit 0
}

