///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// Blob Analysis C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_BLOBANALSIS_CPLUSPLUS)
#define __ACTIVEMIL_BLOBANALSIS_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MILBlobAnalysis.OCX" no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CBlobAnalysis wrapper class

   class CBlobAnalysis : public CWnd, public IBlobAnalysisPtr
   {
      virtual void PreSubclassWindow()
         {
         IBlobAnalysis* pBlob ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(IBlobAnalysis), (void**) &pBlob) ;
         IBlobAnalysisPtr::Attach(pBlob, false) ;
         }
   };

#else // !ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILBlobAnalysisTypes.h"

   // Dispatch interfaces referenced by this interface
   #include "System.h"
   #include "Image.h"
   #include "BlobRun.h"
   #include "BlobRuns.h"
   #include "Pixel.h"
   #include "Pixels.h"
   #include "BlobChain.h"
   #include "BlobChains.h"
   #include "BlobResult.h"
   #include "BlobResults.h"
   #include "FeatureGeneralFeret.h"
   #include "FeatureGeneralMoment.h"
   #include "FeatureBox.h"
   #include "FeatureCenterOfGravity.h"
   #include "FeatureContactPoints.h"
   #include "FeatureList.h"
   #include "Filter.h"
   #include "Filters.h"
   #include "SortingKey.h"
   #include "SortingKeys.h"

   // typedefs to map the old interface names to the new names
   #if !defined(ACTIVEMIL_EXCLUDE_OLD_INTERFACES)
   #define ACTIVEMIL_EXCLUDE_OLD_INTERFACES
      typedef IBlobFeatureBox IFeatureBox;
      typedef IBlobFeatureCenterOfGravity IFeatureCenterOfGravity;
      typedef IBlobFeatureContactPoints IFeatureContactPoints;
      typedef IBlobFeatureGeneralFeret IFeatureGeneralFeret;
      typedef IBlobFeatureGeneralMoment IFeatureGeneralMoment;
      typedef IBlobFeatureList IFeatureList;
      typedef IBlobFilter IFilter;
      typedef IBlobFilters IFilters;
      typedef IBlobPixel IPixel;
      typedef IBlobPixels IPixels;
      typedef IBlobSortingKey ISortingKey;
      typedef IBlobSortingKeys ISortingKeys;
   #endif // ACTIVEMIL_EXCLUDE_OLD_INTERFACES

   /////////////////////////////////////////////////////////////////////////////
   // CBlobAnalysis wrapper class

   class CBlobAnalysis : public CWnd
   {
      CString m_strName;

   protected:
      DECLARE_DYNCREATE(CBlobAnalysis)
   public:
      CLSID const& GetClsid()
      {
         static CLSID const clsid
            = { 0x154919d6, 0xa7da, 0x11d1, { 0x9f, 0xc, 0x0, 0x60, 0x8c, 0xc9, 0xf9, 0xfb } };
         return clsid;
      }
      virtual BOOL Create(LPCTSTR lpszClassName,
         LPCTSTR lpszWindowName, DWORD dwStyle,
         const RECT& rect,
         CWnd* pParentWnd, UINT nID,
         CCreateContext* pContext = NULL)
      { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

      BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
         const RECT& rect, CWnd* pParentWnd, UINT nID,
         CFile* pPersist = NULL, BOOL bStorage = FALSE,
         BSTR bstrLicKey = NULL)
      { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
         pPersist, bStorage, bstrLicKey); }

   // Attributes
   public:
      CString GetName();
      void SetName(LPCTSTR);
      BOOL GetAutomaticAllocation();
      void SetAutomaticAllocation(BOOL);
      BOOL GetPropertyValidation();
      void SetPropertyValidation(BOOL);
      BOOL GetIsAllocated();
      void SetIsAllocated(BOOL);
      long GetForegroundPixelValue();
      void SetForegroundPixelValue(long);
      long GetIdentifierPixelType();
      void SetIdentifierPixelType(long);
      double GetPixelAspectRatio();
      void SetPixelAspectRatio(double);
      long GetLattice();
      void SetLattice(long);
      long GetNumberOfFeretAngles();
      void SetNumberOfFeretAngles(long);
      long GetIdentifierBlobType();
      void SetIdentifierBlobType(long);
      IBlobResults GetResults();
      IBlobResult GetResults(long Index);
      void SetResults(LPDISPATCH);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
      ISystem GetOwnerSystem();
      void SetOwnerSystem(LPCTSTR lpszNewValue);
      IImage GetImage();
      void SetImage(LPCTSTR lpszNewValue);
      IBlobFeatureList GetFeatureList();
      void SetFeatureList(LPCTSTR lpszNewValue);
      IBlobFilters GetFilters();
      IBlobFilter GetFilters(long Index);
      void SetFilters(LPCTSTR lpszNewValue);
      IBlobSortingKeys GetSortingKeys();
      IBlobSortingKey GetSortingKeys(long Index);
      void SetSortingKeys(LPCTSTR lpszNewValue);
      IImage GetGrayscaleImage();
      void SetGrayscaleImage(LPCTSTR lpszNewValue);
      void Allocate();
      void Free();
      VARIANT Inquire(long ParamToInquire);
      void Control(long ControlType, const VARIANT& ControlValue);
      void Modified(long ModificationType = 0);
      void Calculate();
      void LabelImage(LPCTSTR DestinationImage, long Mode);
      void Reconstruct(LPCTSTR SourceImage, LPCTSTR SeedImage, LPCTSTR DestinationImage, long ProcessingMode = blobBinary, long ForegroundPixelValue = blobNonZero, BOOL SeedPixelsAllInBlobs = FALSE);
      void EraseBorderBlobs(LPCTSTR SourceImage, LPCTSTR DestinationImage, long ProcessingMode = blobBinary, long ForegroundPixelValue = blobNonZero);
      void FillHoles(LPCTSTR SourceImage, LPCTSTR DestinationImage, long ForegroundPixelValue = blobNonZero);
      void ExtractHoles(LPCTSTR SourceImage, LPCTSTR DestinationImage, long ForegroundPixelValue = blobNonZero);
      void ApplyFilter(long Index = 1, BOOL Cumulative = TRUE);
      void DrawBlobs(LPCTSTR DestinationImage, long Criterion, long Value, BOOL FillBlobs = TRUE);
      void AboutBox();
   };


   /////////////////////////////////////////////////////////////////////////////
   // IBlobAnalysis wrapper class

   class IBlobAnalysis : public COleDispatchDriver
   {
      CString m_strName;

   public:
	   IBlobAnalysis() {}		// Calls COleDispatchDriver default constructor
	   IBlobAnalysis(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	   IBlobAnalysis(const IBlobAnalysis& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}

     static IBlobAnalysis NewInstance();

   // Attributes
   public:
     CString GetName();
     void SetName(LPCTSTR);
     BOOL GetAutomaticAllocation();
     void SetAutomaticAllocation(BOOL);
     BOOL GetPropertyValidation();
     void SetPropertyValidation(BOOL);
     BOOL GetIsAllocated();
     void SetIsAllocated(BOOL);
     long GetForegroundPixelValue();
     void SetForegroundPixelValue(long);
     long GetIdentifierPixelType();
     void SetIdentifierPixelType(long);
     double GetPixelAspectRatio();
     void SetPixelAspectRatio(double);
     long GetLattice();
     void SetLattice(long);
     long GetNumberOfFeretAngles();
     void SetNumberOfFeretAngles(long);
     long GetIdentifierBlobType();
     void SetIdentifierBlobType(long);
     IBlobResults GetResults();
     IBlobResult GetResults(long Index);
     void SetResults(LPDISPATCH);

     // Default property
     operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
     ISystem GetOwnerSystem();
     void SetOwnerSystem(LPCTSTR lpszNewValue);
     IImage GetImage();
     void SetImage(LPCTSTR lpszNewValue);
     IBlobFeatureList GetFeatureList();
     void SetFeatureList(LPCTSTR lpszNewValue);
     IBlobFilters GetFilters();
     IBlobFilter GetFilters(long Index);
     void SetFilters(LPCTSTR lpszNewValue);
     IBlobSortingKeys GetSortingKeys();
     IBlobSortingKey GetSortingKeys(long Index);
     void SetSortingKeys(LPCTSTR lpszNewValue);
     IImage GetGrayscaleImage();
     void SetGrayscaleImage(LPCTSTR lpszNewValue);
     void Allocate();
     void Free();
     VARIANT Inquire(long ParamToInquire);
     void Control(long ControlType, const VARIANT& ControlValue);
     void Modified(long ModificationType = 0);
     void Calculate();
     void LabelImage(LPCTSTR DestinationImage, long Mode);
     void Reconstruct(LPCTSTR SourceImage, LPCTSTR SeedImage, LPCTSTR DestinationImage, long ProcessingMode = blobBinary, long ForegroundPixelValue = blobNonZero, BOOL SeedPixelsAllInBlobs = FALSE);
     void EraseBorderBlobs(LPCTSTR SourceImage, LPCTSTR DestinationImage, long ProcessingMode = blobBinary, long ForegroundPixelValue = blobNonZero);
     void FillHoles(LPCTSTR SourceImage, LPCTSTR DestinationImage, long ForegroundPixelValue = blobNonZero);
     void ExtractHoles(LPCTSTR SourceImage, LPCTSTR DestinationImage, long ForegroundPixelValue = blobNonZero);
     void ApplyFilter(long Index = 1, BOOL Cumulative = TRUE);
     void DrawBlobs(LPCTSTR DestinationImage, long Criterion, long Value, BOOL FillBlobs = TRUE);
     void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_BLOBANALSIS_CPLUSPLUS
