///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// Calibration C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_CALIBRATION_CPLUSPLUS)
#define __ACTIVEMIL_CALIBRATION_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MILCalibration.OCX" no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CCalibration wrapper class

   class CCalibration : public CWnd, public ICalibrationPtr
   {
      virtual void PreSubclassWindow()
         {
         ICalibration* pCal ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(ICalibration), (void**) &pCal) ;
         ICalibrationPtr::Attach(pCal, false) ;
         }
   };

#else // ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILCalibrationTypes.h"

   // Dispatch interfaces referenced by this interface
   #include "CalibrationResults.h"
   #include "CalibrationGrid.h"
   #include "CalibrationRelativeOrigin.h"
   #include "CalibrationCameraPosition.h"
   #include "CalibrationPoints.h"
   #include "CalibrationPoint.h"

   /////////////////////////////////////////////////////////////////////////////
   // CCalibration wrapper class

   class CCalibration : public CWnd
   {
      CString m_strName;

   protected:
	   DECLARE_DYNCREATE(CCalibration)
   public:
	   CLSID const& GetClsid()
	   {
		   static CLSID const clsid
			   = { 0xa1592493, 0x137d, 0x11d3, { 0xae, 0xc0, 0x0, 0x60, 0x8, 0xab, 0x37, 0x88 } };
		   return clsid;
	   }
	   virtual BOOL Create(LPCTSTR lpszClassName,
		   LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect,
		   CWnd* pParentWnd, UINT nID,
		   CCreateContext* pContext = NULL)
      {  m_strName.LockBuffer();
         return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

       BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect, CWnd* pParentWnd, UINT nID,
		   CFile* pPersist = NULL, BOOL bStorage = FALSE,
		   BSTR bstrLicKey = NULL)
      {  m_strName.LockBuffer();
         return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
            pPersist, bStorage, bstrLicKey); }

   // Attributes
   public:
	   long GetMilID();
	   void SetMilID(long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   BOOL GetIsCalibrated();
	   void SetIsCalibrated(BOOL);
	   long GetCalibrationMode();
	   void SetCalibrationMode(long);
	   BOOL GetTransformCacheEnabled();
	   void SetTransformCacheEnabled(BOOL);
	   long GetOutputCoordinateSystem();
	   void SetOutputCoordinateSystem(long);
	   ICalibrationResults GetResults();
	   void SetResults(LPDISPATCH);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
      ICalibrationGrid GetGrid();
      void SetGrid(LPCTSTR lpszNewValue);
      ICalibrationRelativeOrigin GetRelativeOrigin();
      void SetRelativeOrigin(LPCTSTR lpszNewValue);
      ICalibrationCameraPosition GetCameraPosition();
      void SetCameraPosition(LPCTSTR lpszNewValue);
      ICalibrationPoints GetCalibrationPoints();
      ICalibrationPoint GetCalibrationPoints(long Index);
      void SetCalibrationPoints(LPCTSTR lpszNewValue);
      void Allocate();
      void Free();
      VARIANT Inquire(long ParamToInquire);
      void Control(long ControlType, const VARIANT& ControlValue);
      void Modified(long ModificationType = 0);
      void CalibrateGrid(LPCTSTR SourceImage, BOOL InvertYAxis = FALSE);
      void CalibratePoints();
      void CorrectImage(LPCTSTR SourceImage, LPCTSTR DestinationImage, long InterpolationMode = calNearestNeighbor, long OverscanMode = calOverscanEnable);
      void Load(LPCTSTR FileName);
      void Save(LPCTSTR FileName);
      void ConvertCoordinatePixelToWorld(double X, double Y, double* ResultX, double* ResultY);
      void ConvertCoordinateWorldToPixel(double X, double Y, double* ResultX, double* ResultY);
      void ConvertResultPixelToWorld(long ResultType, double InputResult, double* OutputResult);
      void ConvertResultWorldToPixel(long ResultType, double InputResult, double* OutputResult);
      void AboutBox();
   };


   /////////////////////////////////////////////////////////////////////////////
   // ICalibration wrapper class

   class ICalibration : public COleDispatchDriver
   {
      CString m_strName;

   public:
      ICalibration() { m_strName.LockBuffer(); }		// Calls COleDispatchDriver default constructor
      ICalibration(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) { m_strName.LockBuffer(); }
      ICalibration(const ICalibration& dispatchSrc) : COleDispatchDriver(dispatchSrc) { m_strName.LockBuffer(); }

      static ICalibration NewInstance();

   // Attributes
   public:
      long GetMilID();
      void SetMilID(long);
      CString GetName();
      void SetName(LPCTSTR);
      BOOL GetAutomaticAllocation();
      void SetAutomaticAllocation(BOOL);
      BOOL GetPropertyValidation();
      void SetPropertyValidation(BOOL);
      BOOL GetIsAllocated();
      void SetIsAllocated(BOOL);
      BOOL GetIsCalibrated();
      void SetIsCalibrated(BOOL);
      long GetCalibrationMode();
      void SetCalibrationMode(long);
      BOOL GetTransformCacheEnabled();
      void SetTransformCacheEnabled(BOOL);
      long GetOutputCoordinateSystem();
      void SetOutputCoordinateSystem(long);
      ICalibrationResults GetResults();
      void SetResults(LPDISPATCH);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
      ICalibrationGrid GetGrid();
      void SetGrid(LPCTSTR lpszNewValue);
      ICalibrationRelativeOrigin GetRelativeOrigin();
      void SetRelativeOrigin(LPCTSTR lpszNewValue);
      ICalibrationCameraPosition GetCameraPosition();
      void SetCameraPosition(LPCTSTR lpszNewValue);
      ICalibrationPoints GetCalibrationPoints();
      ICalibrationPoint GetCalibrationPoints(long Index);
      void SetCalibrationPoints(LPCTSTR lpszNewValue);
      void Allocate();
      void Free();
      VARIANT Inquire(long ParamToInquire);
      void Control(long ControlType, const VARIANT& ControlValue);
      void Modified(long ModificationType = 0);
      void CalibrateGrid(LPCTSTR SourceImage, BOOL InvertYAxis = FALSE);
      void CalibratePoints();
      void CorrectImage(LPCTSTR SourceImage, LPCTSTR DestinationImage, long InterpolationMode = calNearestNeighbor, long OverscanMode = calOverscanEnable);
      void Load(LPCTSTR FileName);
      void Save(LPCTSTR FileName);
      void ConvertCoordinatePixelToWorld(double X, double Y, double* ResultX, double* ResultY);
      void ConvertCoordinateWorldToPixel(double X, double Y, double* ResultX, double* ResultY);
      void ConvertResultPixelToWorld(long ResultType, double InputResult, double* OutputResult);
      void ConvertResultWorldToPixel(long ResultType, double InputResult, double* OutputResult);
      void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_CALIBRATION_CPLUSPLUS
