///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// Code C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_CODE_CPLUSPLUS)
#define __ACTIVEMIL_CODE_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MILCode.OCX" no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CCode wrapper class

   class CCode : public CWnd, public ICodePtr
   {
      virtual void PreSubclassWindow()
         {
         ICode* pCode ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(ICode), (void**) &pCode) ;
         ICodePtr::Attach(pCode, false) ;
         }
   };

#else // !ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILCodeTypes.h"

   // Dispatch interfaces referenced by this interface
   #include "System.h"
   #include "Image.h"
   #include "RegionOfInterest.h"
   #include "CodeResults.h"
   #include "CodeTargetGrid.h"

   /////////////////////////////////////////////////////////////////////////////
   // CCode wrapper class

   class CCode : public CWnd
   {
      CString m_strName;

   protected:
	   DECLARE_DYNCREATE(CCode)
   public:
	   CLSID const& GetClsid()
	   {
		   static CLSID const clsid
			   = { 0xdaf83296, 0xa796, 0x11d1, { 0x9f, 0xc, 0x0, 0x60, 0x8c, 0xc9, 0xf9, 0xfb } };
		   return clsid;
	   }
	   virtual BOOL Create(LPCTSTR lpszClassName,
		   LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect,
		   CWnd* pParentWnd, UINT nID,
		   CCreateContext* pContext = NULL)
	   { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

       BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect, CWnd* pParentWnd, UINT nID,
		   CFile* pPersist = NULL, BOOL bStorage = FALSE,
		   BSTR bstrLicKey = NULL)
	   { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
		   pPersist, bStorage, bstrLicKey); }

   // Attributes
   public:
	   long GetMilID();
	   void SetMilID(long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   ICodeResults GetResults();
	   void SetResults(LPDISPATCH);
	   long GetCodeType();
	   void SetCodeType(long);
	   long GetEncodingType();
	   void SetEncodingType(long);
	   long GetErrorCorrectionType();
	   void SetErrorCorrectionType(long);
	   long GetSpeed();
	   void SetSpeed(long);
	   double GetPixelAspectRatio();
	   void SetPixelAspectRatio(double);
	   long GetTargetStringLength();
	   void SetTargetStringLength(long);
	   long GetForegroundPixelValue();
	   void SetForegroundPixelValue(long);
	   long GetImageThreshold();
	   void SetImageThreshold(long);
   
      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   ISystem GetOwnerSystem();
	   void SetOwnerSystem(LPCTSTR lpszNewValue);
	   IImage GetImage();
	   void SetImage(LPCTSTR lpszNewValue);
	   ICodeTargetGrid GetTargetGrid();
	   void SetTargetGrid(LPCTSTR lpszNewValue);
	   IRegionOfInterest GetSearchRegion();
	   void SetSearchRegion(LPCTSTR lpszNewValue);
	   void Allocate();
	   void Free();
	   VARIANT Inquire(long ParamToInquire);
	   void Control(long ControlType, const VARIANT& ControlValue);
	   void Modified(long ModificationType);
	   void ReadCode();
	   void WriteCode(LPCTSTR CodeString);
	   void CalculateCodeSize(LPCTSTR CodeString, long* X, long* Y);
	   void AboutBox();
   };


   /////////////////////////////////////////////////////////////////////////////
   // ICode wrapper class

   class ICode : public COleDispatchDriver
   {
     CString m_strName;

   public:
	   ICode() {}		// Calls COleDispatchDriver default constructor
	   ICode(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	   ICode(const ICode& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}

     static ICode NewInstance();

   // Attributes
   public:
	   long GetMilID();
	   void SetMilID(long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   ICodeResults GetResults();
	   void SetResults(LPDISPATCH);
	   long GetCodeType();
	   void SetCodeType(long);
	   long GetEncodingType();
	   void SetEncodingType(long);
	   long GetErrorCorrectionType();
	   void SetErrorCorrectionType(long);
	   long GetSpeed();
	   void SetSpeed(long);
	   double GetPixelAspectRatio();
	   void SetPixelAspectRatio(double);
	   long GetTargetStringLength();
	   void SetTargetStringLength(long);
	   long GetForegroundPixelValue();
	   void SetForegroundPixelValue(long);
	   long GetImageThreshold();
	   void SetImageThreshold(long);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   ISystem GetOwnerSystem();
	   void SetOwnerSystem(LPCTSTR lpszNewValue);
	   IImage GetImage();
	   void SetImage(LPCTSTR lpszNewValue);
	   ICodeTargetGrid GetTargetGrid();
	   void SetTargetGrid(LPCTSTR lpszNewValue);
	   IRegionOfInterest GetSearchRegion();
	   void SetSearchRegion(LPCTSTR lpszNewValue);
	   void Allocate();
	   void Free();
	   VARIANT Inquire(long ParamToInquire);
	   void Control(long ControlType, const VARIANT& ControlValue);
	   void Modified(long ModificationType);
	   void ReadCode();
	   void WriteCode(LPCTSTR CodeString);
	   void CalculateCodeSize(LPCTSTR CodeString, long* X, long* Y);
	   void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_CODE_CPLUSPLUS
