///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// Graphic Context C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_GRAPHICCONTEXT_CPLUSPLUS)
#define __ACTIVEMIL_GRAPHICCONTEXT_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MIL.OCX" no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CGraphicContext wrapper class

   class CGraphicContext : public CWnd, public IGraphicContextPtr
   {
      virtual void PreSubclassWindow()
         {
         IGraphicContext* pGC ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(IGraphicContext), (void**) &pGC) ;
         IGraphicContextPtr::Attach(pGC, false) ;
         }
   };

#else // !ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILBaseTypes.h"

   // Dispatch interfaces referenced by this interface
   #include "System.h"
   #include "Image.h"
   #include "RegionOfInterest.h"

   /////////////////////////////////////////////////////////////////////////////
   // CGraphicContext wrapper class

   class CGraphicContext : public CWnd
   {
      CString m_strName;

   protected:
	   DECLARE_DYNCREATE(CGraphicContext)
   public:
	   CLSID const& GetClsid()
	   {
		   static CLSID const clsid
			   = { 0xf2e7bde6, 0xb006, 0x11d0, { 0x91, 0x62, 0x0, 0xa0, 0x24, 0xd2, 0x49, 0x92 } };
		   return clsid;
	   }
	   virtual BOOL Create(LPCTSTR lpszClassName,
		   LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect,
		   CWnd* pParentWnd, UINT nID,
		   CCreateContext* pContext = NULL)
	   { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

       BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect, CWnd* pParentWnd, UINT nID,
		   CFile* pPersist = NULL, BOOL bStorage = FALSE,
		   BSTR bstrLicKey = NULL)
	   { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
		   pPersist, bStorage, bstrLicKey); }

   // Attributes
   public:
	   long GetBackgroundMode();
	   void SetBackgroundMode(long);
	   double GetFontScaleX();
	   void SetFontScaleX(double);
	   double GetFontScaleY();
	   void SetFontScaleY(double);
	   long GetMilID();
	   void SetMilID(long);
	   long GetFont();
	   void SetFont(long);
	   unsigned long GetBackgroundColor();
	   void SetBackgroundColor(unsigned long);
	   unsigned long GetForegroundColor();
	   void SetForegroundColor(unsigned long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   double GetForegroundShade();
	   void SetForegroundShade(double);
	   double GetBackgroundShade();
	   void SetBackgroundShade(double);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   IRegionOfInterest GetDrawingRegion();
	   void SetDrawingRegion(LPCTSTR lpszNewValue);
	   ISystem GetOwnerSystem();
	   void SetOwnerSystem(LPCTSTR lpszNewValue);
	   void Rectangle(BOOL FillRegion = FALSE, double Angle = 0.0);
	   void Clear();
	   void Dot();
	   void Fill();
	   void LineSegment();
	   void Text(LPCTSTR OutputString);
	   void Allocate();
	   void Free();
	   void Cross(double Angle = 0.0);
	   VARIANT Inquire(long ParamToInquire);
	   void Control(long ControlType, const VARIANT& ControlValue);
	   void Modified(long ModificationType = 0);
	   void Arc(double StartAngle = 0.0, double EndAngle = 360.0, BOOL FillRegion = FALSE);
	   IImage GetImage();
	   void SetImage(LPCTSTR lpszNewValue);
	   void AboutBox();
   };


   /////////////////////////////////////////////////////////////////////////////
   // IGraphicContext wrapper class

   class IGraphicContext : public COleDispatchDriver
   {
     CString m_strName;

   public:
	   IGraphicContext() {}		// Calls COleDispatchDriver default constructor
	   IGraphicContext(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	   IGraphicContext(const IGraphicContext& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}

     static IGraphicContext NewInstance();

   // Attributes
   public:
	   long GetBackgroundMode();
	   void SetBackgroundMode(long);
	   double GetFontScaleX();
	   void SetFontScaleX(double);
	   double GetFontScaleY();
	   void SetFontScaleY(double);
	   long GetMilID();
	   void SetMilID(long);
	   long GetFont();
	   void SetFont(long);
	   unsigned long GetBackgroundColor();
	   void SetBackgroundColor(unsigned long);
	   unsigned long GetForegroundColor();
	   void SetForegroundColor(unsigned long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   double GetForegroundShade();
	   void SetForegroundShade(double);
	   double GetBackgroundShade();
	   void SetBackgroundShade(double);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   IRegionOfInterest GetDrawingRegion();
	   void SetDrawingRegion(LPCTSTR lpszNewValue);
	   ISystem GetOwnerSystem();
	   void SetOwnerSystem(LPCTSTR lpszNewValue);
	   void Rectangle(BOOL FillRegion = FALSE, double Angle = 0.0);
	   void Clear();
	   void Dot();
	   void Fill();
	   void LineSegment();
	   void Text(LPCTSTR OutputString);
	   void Allocate();
	   void Free();
	   void Cross(double Angle = 0.0);
	   VARIANT Inquire(long ParamToInquire);
	   void Control(long ControlType, const VARIANT& ControlValue);
	   void Modified(long ModificationType = 0);
	   void Arc(double StartAngle = 0.0, double EndAngle = 360.0, BOOL FillRegion = FALSE);
	   IImage GetImage();
	   void SetImage(LPCTSTR lpszNewValue);
	   void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_GRAPHICCONTEXT_CPLUSPLUS
