///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// Image C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_IMAGE_CPLUSPLUS)
#define __ACTIVEMIL_IMAGE_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MIL.OCX" no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CImage wrapper class

   class CImage : public CWnd, public IImagePtr
   {
      virtual void PreSubclassWindow()
         {
         IImage* pImg ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(IImage), (void**) &pImg) ;
         IImagePtr::Attach(pImg, false) ;
         }

   public:
      // this cast operator has been added to facilitate the usage of the 
      // ImageProcessing.SourceX property which take a _variant_t
      operator _variant_t()
         {
         return _variant_t(GetInterfacePtr()) ;
         }

   };

#else // !ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILBaseTypes.h"

   // Dispatch interfaces referenced by this interface
   class IImage;
   #include "System.h"
   #include "Image.h"
   #include "LUT.h"
   #include "RegionOfInterest.h"
   #include "JPEGAlgorithm.h"
   #include "ImageCalibrationInfo.h"

   /////////////////////////////////////////////////////////////////////////////
   // CImage wrapper class

   class CImage : public CWnd
   {
      CString m_strName;

   protected:
	   DECLARE_DYNCREATE(CImage)
   public:
      CLSID const& GetClsid()
      {
         static CLSID const clsid
            = { 0x3985968, 0x6b33, 0x11d0, { 0xab, 0x4a, 0x0, 0x60, 0x8c, 0xc9, 0xca, 0x57 } };
         return clsid;
      }
      virtual BOOL Create(LPCTSTR lpszClassName,
         LPCTSTR lpszWindowName, DWORD dwStyle,
         const RECT& rect,
         CWnd* pParentWnd, UINT nID,
         CCreateContext* pContext = NULL)
      {  return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

      BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
         const RECT& rect, CWnd* pParentWnd, UINT nID,
         CFile* pPersist = NULL, BOOL bStorage = FALSE,
         BSTR bstrLicKey = NULL)
      {  return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
            pPersist, bStorage, bstrLicKey);}

   // Attributes
   public:
	   long GetSizeX();
	   void SetSizeX(long);
	   long GetSizeY();
	   void SetSizeY(long);
	   BOOL GetCanGrab();
	   void SetCanGrab(BOOL);
	   BOOL GetCanProcess();
	   void SetCanProcess(BOOL);
	   long GetFileFormat();
	   void SetFileFormat(long);
	   long GetLocation();
	   void SetLocation(long);
	   BOOL GetCanDisplay();
	   void SetCanDisplay(BOOL);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   long GetBand();
	   void SetBand(long);
	   long GetDataDepth();
	   void SetDataDepth(long);
	   long GetDataType();
	   void SetDataType(long);
	   long GetFormat();
	   void SetFormat(long);
	   long GetMilID();
	   void SetMilID(long);
	   long GetNumberOfBands();
	   void SetNumberOfBands(long);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   BOOL GetModifiedEvent();
	   void SetModifiedEvent(BOOL);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   long GetHostAddress();
	   void SetHostAddress(long);
	   long GetPhysicalAddress();
	   void SetPhysicalAddress(long);
	   long GetPitchByte();
	   void SetPitchByte(long);
	   long GetDataSize();
	   void SetDataSize(long);
	   long GetCompressionType();
	   void SetCompressionType(long);
	   long GetModificationCount();
	   void SetModificationCount(long);
	   BOOL GetLUTEnabled();
	   void SetLUTEnabled(BOOL);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   ILUT GetLUT();
	   void SetLUT(LPCTSTR lpszNewValue);
	   ISystem GetOwnerSystem();
	   void SetOwnerSystem(LPCTSTR lpszNewValue);
	   IImage GetParentImage();
	   void SetParentImage(LPCTSTR lpszNewValue);
	   IDispatch* GetCalibration();
	   void SetCalibration(LPCTSTR lpszNewValue);
	   IImageCalibrationInfo GetCalibrationInfo();
	   void SetCalibrationInfo(LPCTSTR lpszNewValue);
	   IRegionOfInterest GetChildRegion();
	   void SetChildRegion(LPCTSTR lpszNewValue);
	   IJPEGAlgorithm GetJPEGAlgorithm();
	   void SetJPEGAlgorithm(LPCTSTR lpszNewValue);
	   IPictureDisp* GetPicture();
	   void SetPicture(IPictureDisp* pPictureDisp);
	   long FileInquire(LPCTSTR FileName, long ParamToInquire);
	   long Inquire(long ParamToInquire);
	   void Save(LPCTSTR FileName);
	   void Allocate();
	   void Control(long ControlType, const VARIANT& ControlValue);
	   void Clear(long Color = 0);
	   void Free();
	   void CopyClip(LPCTSTR SourceImage, long OffsetX, long OffsetY);
	   void CopyConditional(LPCTSTR SourceImage, LPCTSTR ConditionImage, long Condition, long ConditionValue);
	   void CopyMask(LPCTSTR SourceImage, long MaskValue);
	   void Load(LPCTSTR FileName = NULL, BOOL AdjustImage = FALSE);
	   void Copy(LPCTSTR SourceImage, long Band = imAllBands);
	   void Get(const VARIANT& UserArray, long Format = imPlanar, long Band = imAllBands, long OffsetX = 0, long OffsetY = 0, long SizeX = -1, long SizeY = -1);
	   void Put(const VARIANT& UserArray, long Format = imPlanar, long Band = imAllBands, long OffsetX = 0, long OffsetY = 0, long SizeX = -1, long SizeY = -1);
	   long GetLine(const VARIANT& UserArray, long StartX, long StartY, long EndX, long EndY);
	   void AssignMemory(const VARIANT& UserArray, long ControlFlag, long Pitch);
	   long PutLine(const VARIANT& UserArray, long StartX, long StartY, long EndX, long EndY);
	   void Modified(long ModificationType = 0);
	   IImage Clone();
	   void CopyRegion(LPCTSTR SourceImage, long SourceBand, long SourceOffsetX, long SourceOffsetY, long Band, long OffsetX, long OffsetY, long SizeX, long SizeY);
      void CopyToClipboard();
      void PasteFromClipboard(BOOL AdjustImage = FALSE);
	   void AboutBox();
   };


   /////////////////////////////////////////////////////////////////////////////
   // IImage wrapper class

   class IImage : public COleDispatchDriver
   {
      CString m_strName;

   public:
	   IImage() {}		// Calls COleDispatchDriver default constructor
	   IImage(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	   IImage(const IImage& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}

      static IImage NewInstance();

   // Attributes
   public:
	   long GetSizeX();
	   void SetSizeX(long);
	   long GetSizeY();
	   void SetSizeY(long);
	   BOOL GetCanGrab();
	   void SetCanGrab(BOOL);
	   BOOL GetCanProcess();
	   void SetCanProcess(BOOL);
	   long GetFileFormat();
	   void SetFileFormat(long);
	   long GetLocation();
	   void SetLocation(long);
	   BOOL GetCanDisplay();
	   void SetCanDisplay(BOOL);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   long GetBand();
	   void SetBand(long);
	   long GetDataDepth();
	   void SetDataDepth(long);
	   long GetDataType();
	   void SetDataType(long);
	   long GetFormat();
	   void SetFormat(long);
	   long GetMilID();
	   void SetMilID(long);
	   long GetNumberOfBands();
	   void SetNumberOfBands(long);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   BOOL GetModifiedEvent();
	   void SetModifiedEvent(BOOL);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   long GetHostAddress();
	   void SetHostAddress(long);
	   long GetPhysicalAddress();
	   void SetPhysicalAddress(long);
	   long GetPitchByte();
	   void SetPitchByte(long);
	   long GetDataSize();
	   void SetDataSize(long);
	   long GetCompressionType();
	   void SetCompressionType(long);
	   long GetModificationCount();
	   void SetModificationCount(long);
	   BOOL GetLUTEnabled();
	   void SetLUTEnabled(BOOL);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   ILUT GetLUT();
	   void SetLUT(LPCTSTR lpszNewValue);
	   ISystem GetOwnerSystem();
	   void SetOwnerSystem(LPCTSTR lpszNewValue);
	   IImage GetParentImage();
	   void SetParentImage(LPCTSTR lpszNewValue);
	   IDispatch* GetCalibration();
	   void SetCalibration(LPCTSTR lpszNewValue);
	   IImageCalibrationInfo GetCalibrationInfo();
	   void SetCalibrationInfo(LPCTSTR lpszNewValue);
	   IRegionOfInterest GetChildRegion();
	   void SetChildRegion(LPCTSTR lpszNewValue);
	   IJPEGAlgorithm GetJPEGAlgorithm();
	   void SetJPEGAlgorithm(LPCTSTR lpszNewValue);
	   IPictureDisp* GetPicture();
	   void SetPicture(IPictureDisp* pPictureDisp);
	   long FileInquire(LPCTSTR FileName, long ParamToInquire);
	   long Inquire(long ParamToInquire);
	   void Save(LPCTSTR FileName);
	   void Allocate();
	   void Control(long ControlType, const VARIANT& ControlValue);
	   void Clear(long Color = 0);
	   void Free();
	   void CopyClip(LPCTSTR SourceImage, long OffsetX, long OffsetY);
	   void CopyConditional(LPCTSTR SourceImage, LPCTSTR ConditionImage, long Condition, long ConditionValue);
	   void CopyMask(LPCTSTR SourceImage, long MaskValue);
	   void Load(LPCTSTR FileName = NULL, BOOL AdjustImage = FALSE);
	   void Copy(LPCTSTR SourceImage, long Band = imAllBands);
	   void Get(const VARIANT& UserArray, long Format = imPlanar, long Band = imAllBands, long OffsetX = 0, long OffsetY = 0, long SizeX = -1, long SizeY = -1);
	   void Put(const VARIANT& UserArray, long Format = imPlanar, long Band = imAllBands, long OffsetX = 0, long OffsetY = 0, long SizeX = -1, long SizeY = -1);
	   long GetLine(const VARIANT& UserArray, long StartX, long StartY, long EndX, long EndY);
	   void AssignMemory(const VARIANT& UserArray, long ControlFlag, long Pitch);
	   long PutLine(const VARIANT& UserArray, long StartX, long StartY, long EndX, long EndY);
	   void Modified(long ModificationType = 0);
	   IImage Clone();
	   void CopyRegion(LPCTSTR SourceImage, long SourceBand, long SourceOffsetX, long SourceOffsetY, long Band, long OffsetX, long OffsetY, long SizeX, long SizeY);
      void CopyToClipboard();
      void PasteFromClipboard(BOOL AdjustImage = FALSE);
	   void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_IMAGE_CPLUSPLUS
