///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// Image Processing C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_IMAGEPROCESSING_CPLUSPLUS)
#define __ACTIVEMIL_IMAGEPROCESSING_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MILImageProcessing.OCX" no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CSystem wrapper class

   class CImageProcessing : public CWnd, public IImageProcessingPtr
   {
      virtual void PreSubclassWindow()
         {
         IImageProcessing* pImProc ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(IImageProcessing), (void**) &pImProc) ;
         IImageProcessingPtr::Attach(pImProc, false) ;
         }
   };

#else // !ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILImageProcessingTypes.h"

   // Dispatch interfaces referenced by this interface
   #include "System.h"
   #include "Image.h"
   #include "ImageProcessingResults.h"
   #include "LUT.h"
   #include "LUTs.h"
   #include "Kernel.h"
   #include "Kernels.h"
   #include "StructuringElement.h"
   #include "StructuringElements.h"
   #include "PolarParameters.h"
   #include "WarpQuadrilateral.h"
   #include "WarpRectangle.h"
   #include "WarpLUT.h"
   #include "WarpParameters.h"

   /////////////////////////////////////////////////////////////////////////////
   // CImageProcessing wrapper class

   class CImageProcessing : public CWnd
   {
       CString m_strName;

   protected:
	   DECLARE_DYNCREATE(CImageProcessing)
   public:
      CLSID const& GetClsid()
      {
         static CLSID const clsid
            = { 0x4fd68bb6, 0xa7e7, 0x11d1, { 0x9f, 0xd, 0x0, 0x60, 0x8c, 0xc9, 0xf9, 0xfb } };
         return clsid;
      }
      virtual BOOL Create(LPCTSTR lpszClassName,
         LPCTSTR lpszWindowName, DWORD dwStyle,
         const RECT& rect,
         CWnd* pParentWnd, UINT nID,
         CCreateContext* pContext = NULL)
      { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

      BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
         const RECT& rect, CWnd* pParentWnd, UINT nID,
         CFile* pPersist = NULL, BOOL bStorage = FALSE,
         BSTR bstrLicKey = NULL)
      { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
         pPersist, bStorage, bstrLicKey); }

   // Attributes
   public:
      CString GetName();
      void SetName(LPCTSTR);
      BOOL GetAutomaticAllocation();
      void SetAutomaticAllocation(BOOL);
      BOOL GetPropertyValidation();
      void SetPropertyValidation(BOOL);
      BOOL GetIsAllocated();
      void SetIsAllocated(BOOL);
      long GetResultType();
      void SetResultType(long);
      long GetResultSize();
      void SetResultSize(long);
      IImageProcessingResults GetResults();
      void SetResults(LPDISPATCH);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
      ISystem GetOwnerSystem();
      void SetOwnerSystem(LPCTSTR lpszNewValue);
      VARIANT GetSource1();
      void SetSource1(LPCTSTR lpszNewValue);
      void SetSource1(double Value);
      VARIANT GetSource2();
      void SetSource2(LPCTSTR lpszNewValue);
      void SetSource2(double Value);
      VARIANT GetSource3();
      void SetSource3(LPCTSTR lpszNewValue);
      void SetSource3(double Value);
      VARIANT GetSource4();
      void SetSource4(LPCTSTR lpszNewValue);
      void SetSource4(double Value);
      VARIANT GetSource5();
      void SetSource5(LPCTSTR lpszNewValue);
      void SetSource5(double Value);
      IImage GetDestination1();
      void SetDestination1(LPCTSTR lpszNewValue);
      IImage GetDestination2();
      void SetDestination2(LPCTSTR lpszNewValue);
      ILUTs GetLUTs();
      ILUT GetLUTs(long Index);
      void SetLUTs(LPCTSTR lpszNewValue);
      IKernels GetKernels();
      IKernel GetKernels(long Index);
      void SetKernels(LPCTSTR lpszNewValue);
      IStructuringElements GetStructuringElements();
      IStructuringElement GetStructuringElements(long Index);
      void SetStructuringElements(LPCTSTR lpszNewValue);
      IPolarParameters GetPolarParameters();
      void SetPolarParameters(LPCTSTR lpszNewValue);
      IWarpParameters GetWarpParameters();
      void SetWarpParameters(LPCTSTR lpszNewValue);
      void Allocate();
      void Free();
      void Add(BOOL Saturation = FALSE);
      void Subtract(BOOL AbsoluteValue = FALSE, BOOL Saturation = FALSE);
      void And();
      void Nand();
      void Or();
      void Xor();
      void Nor();
      void Minimum();
      void Maximum();
      void Multiply(BOOL Saturation = FALSE);
      void Divide(BOOL FixedPoint = FALSE);
      void Negate();
      void Not();
      void AbsoluteValue();
      void OffsetGain(BOOL Saturation = FALSE);
      void WeightedAverage(BOOL Saturation = FALSE);
      void MultiplyAndAccumulate1(BOOL Saturation = FALSE);
      void MultiplyAndAccumulate2(BOOL Saturation = FALSE);
      void Shift(long ShiftFactor);
      void Binarize(long Condition, double LowValue, double HighValue = 0.0);
      void Clip(long Condition, double LowValue, double HighValue, double LowReplaceValue, double HighReplaceValue);
      void LUTMap(long Index = 1);
      void Close(long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
      void Open(long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
      void Dilate(long Index = impPredefined, long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
      void Erode(long Index = impPredefined, long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
      void Thin(long Index = impPredefined, long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
      void Thick(long Index = impPredefined, long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
      void HitOrMiss(long Index = 1, long ProcessingMode = impGrayscale);
      void Match(long Index = 1, long ProcessingMode = impGrayscale);
      void ConnectMap(long Index = 1);
      void Distance(long DistanceTransform = impChamfer34);
      void Label(long Lattice = imp8Connected, long ProcessingMode = impGrayscale);
      void ZoneOfInfluence(long DistanceTransform = impChessboard);
      void Resize(double ScaleFactorX, double ScaleFactorY, long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable);
      void Translate(double DisplacementX, double DisplacementY, long OverscanMode = impOverscanEnable);
      void Rotate(double Angle, long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable, double SourceCenterX = impDefault, double SourceCenterY = impDefault, double DestinationCenterX = impDefault, double DestinationCenterY = impDefault);
      void Flip(long Operation);
      void Smooth();
      void Sharpen1();
      void Sharpen2();
      void HorizontalEdge();
      void VerticalEdge();
      void EdgeDetect1();
      void EdgeDetect2();
      void LaplacianEdge1();
      void LaplacianEdge2();
      void Convolve(long Index = 1);
      void Rank(long Index, long Number = impMedian, long ProcessingMode = impGrayscale);
      void EdgeDetect(BOOL FastAngleComputation= TRUE, BOOL FastGradientComputation = TRUE, long Threshold = 0);
      void FastFourierTransform(long ComputationMode, BOOL ForwardTransform = TRUE, BOOL Normalize = FALSE, BOOL CenterSpectrum = FALSE);
      void DiscreteCosineTransform(BOOL ForwardTransform = TRUE, BOOL Normalize = FALSE);
      void Convert(long ConversionType);
      void Histogram();
      void HistogramEqualize(long EqualizationMethod, double MinimumValue = 0, double MaximumValue = 255, double Alpha = 0, long TransformLUTIndex = 0);
      void Project(double ProjectionAngle);
      void FindExtremes(BOOL FindMinimum = TRUE, BOOL FindMaximum = TRUE);
      void LocateEvents(long Condition, double LowValue, double HighValue = 0.0);
      void CountDifferences();
      void Warp(long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable);
      void Watershed(long MinimumVariation, long ComputationMode = impWatershed, BOOL MaximaFilled = FALSE, BOOL StraightLines = FALSE, BOOL SkipLastLevel = FALSE, long Lattice = imp4Connected);
      void PolarToRectangular(long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable);
      void RectangularToPolar(long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable);
      void Xnor();
      void AboutBox();
   };


   /////////////////////////////////////////////////////////////////////////////
   // IImageProcessing wrapper class

   class IImageProcessing : public COleDispatchDriver
   {
     CString m_strName;

   public:
	   IImageProcessing() {}		// Calls COleDispatchDriver default constructor
	   IImageProcessing(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	   IImageProcessing(const IImageProcessing& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}

     static IImageProcessing NewInstance();

   // Attributes
   public:
     CString GetName();
     void SetName(LPCTSTR);
     BOOL GetAutomaticAllocation();
     void SetAutomaticAllocation(BOOL);
     BOOL GetPropertyValidation();
     void SetPropertyValidation(BOOL);
     BOOL GetIsAllocated();
     void SetIsAllocated(BOOL);
     long GetResultType();
     void SetResultType(long);
     long GetResultSize();
     void SetResultSize(long);
     IImageProcessingResults GetResults();
     void SetResults(LPDISPATCH);

     // Default property
     operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
     ISystem GetOwnerSystem();
     void SetOwnerSystem(LPCTSTR lpszNewValue);
     VARIANT GetSource1();
     void SetSource1(LPCTSTR lpszNewValue);
     void SetSource1(double Value);
     VARIANT GetSource2();
     void SetSource2(LPCTSTR lpszNewValue);
     void SetSource2(double Value);
     VARIANT GetSource3();
     void SetSource3(LPCTSTR lpszNewValue);
     void SetSource3(double Value);
     VARIANT GetSource4();
     void SetSource4(LPCTSTR lpszNewValue);
     void SetSource4(double Value);
     VARIANT GetSource5();
     void SetSource5(LPCTSTR lpszNewValue);
     void SetSource5(double Value);
     IImage GetDestination1();
     void SetDestination1(LPCTSTR lpszNewValue);
     IImage GetDestination2();
     void SetDestination2(LPCTSTR lpszNewValue);
     ILUTs GetLUTs();
     ILUT GetLUTs(long Index);
     void SetLUTs(LPCTSTR lpszNewValue);
     IKernels GetKernels();
     IKernel GetKernels(long Index);
     void SetKernels(LPCTSTR lpszNewValue);
     IStructuringElements GetStructuringElements();
     IStructuringElement GetStructuringElements(long Index);
     void SetStructuringElements(LPCTSTR lpszNewValue);
     IPolarParameters GetPolarParameters();
     void SetPolarParameters(LPCTSTR lpszNewValue);
     IWarpParameters GetWarpParameters();
     void SetWarpParameters(LPCTSTR lpszNewValue);
     void Allocate();
     void Free();
     void Add(BOOL Saturation = FALSE);
     void Subtract(BOOL AbsoluteValue = FALSE, BOOL Saturation = FALSE);
     void And();
     void Nand();
     void Or();
     void Xor();
     void Nor();
     void Minimum();
     void Maximum();
     void Multiply(BOOL Saturation = FALSE);
     void Divide(BOOL FixedPoint = FALSE);
     void Negate();
     void Not();
     void AbsoluteValue();
     void OffsetGain(BOOL Saturation = FALSE);
     void WeightedAverage(BOOL Saturation = FALSE);
     void MultiplyAndAccumulate1(BOOL Saturation = FALSE);
     void MultiplyAndAccumulate2(BOOL Saturation = FALSE);
     void Shift(long ShiftFactor);
     void Binarize(long Condition, double LowValue, double HighValue = 0.0);
     void Clip(long Condition, double LowValue, double HighValue, double LowReplaceValue, double HighReplaceValue);
     void LUTMap(long Index = 1);
     void Close(long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
     void Open(long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
     void Dilate(long Index = impPredefined, long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
     void Erode(long Index = impPredefined, long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
     void Thin(long Index = impPredefined, long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
     void Thick(long Index = impPredefined, long NumberOfIterations = 1, long ProcessingMode = impGrayscale);
     void HitOrMiss(long Index = 1, long ProcessingMode = impGrayscale);
     void Match(long Index = 1, long ProcessingMode = impGrayscale);
     void ConnectMap(long Index = 1);
     void Distance(long DistanceTransform = impChamfer34);
     void Label(long Lattice = imp8Connected, long ProcessingMode = impGrayscale);
     void ZoneOfInfluence(long DistanceTransform = impChessboard);
     void Resize(double ScaleFactorX, double ScaleFactorY, long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable);
     void Translate(double DisplacementX, double DisplacementY, long OverscanMode = impOverscanEnable);
     void Rotate(double Angle, long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable, double SourceCenterX = impDefault, double SourceCenterY = impDefault, double DestinationCenterX = impDefault, double DestinationCenterY = impDefault);
     void Flip(long Operation);
     void Smooth();
     void Sharpen1();
     void Sharpen2();
     void HorizontalEdge();
     void VerticalEdge();
     void EdgeDetect1();
     void EdgeDetect2();
     void LaplacianEdge1();
     void LaplacianEdge2();
     void Convolve(long Index = 1);
     void Rank(long Index, long Number = impMedian, long ProcessingMode = impGrayscale);
     void EdgeDetect(BOOL FastAngleComputation= TRUE, BOOL FastGradientComputation = TRUE, long Threshold = 0);
     void FastFourierTransform(long ComputationMode, BOOL ForwardTransform = TRUE, BOOL Normalize = FALSE, BOOL CenterSpectrum = FALSE);
     void DiscreteCosineTransform(BOOL ForwardTransform = TRUE, BOOL Normalize = FALSE);
     void Convert(long ConversionType);
     void Histogram();
     void HistogramEqualize(long EqualizationMethod, double MinimumValue = 0, double MaximumValue = 255, double Alpha = 0, long TransformLUTIndex = 0);
     void Project(double ProjectionAngle);
     void FindExtremes(BOOL FindMinimum = TRUE, BOOL FindMaximum = TRUE);
     void LocateEvents(long Condition, double LowValue, double HighValue = 0.0);
     void CountDifferences();
     void Warp(long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable);
     void Watershed(long MinimumVariation, long ComputationMode = impWatershed, BOOL MaximaFilled = FALSE, BOOL StraightLines = FALSE, BOOL SkipLastLevel = FALSE, long Lattice = imp4Connected);
     void PolarToRectangular(long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable);
     void RectangularToPolar(long InterpolationMode = impNearestNeighbor, long OverscanMode = impOverscanEnable);
     void Xnor();
     void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_IMAGEPROCESSING_CPLUSPLUS
