///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// Measurement C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_MEASUREMENT_CPLUSPLUS)
#define __ACTIVEMIL_MEASUREMENT_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MILMeasurement.OCX"  no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CMeasurement wrapper class

   class CMeasurement : public CWnd, public IMeasurementPtr
   {
      virtual void PreSubclassWindow()
         {
         IMeasurement* pMeas ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(IMeasurement), (void**) &pMeas) ;
         IMeasurementPtr::Attach(pMeas, false) ;
         }
   };

#else // !ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILMeasurementTypes.h"

   // Dispatch interfaces referenced by this interface
   #include "System.h"
   #include "Image.h"
   #include "ResultEdgeValues.h"
   #include "MeasurementEdgeResult.h"
   #include "MeasurementResult.h"
   #include "MeasurementResults.h"
   #include "MeasurementMarker.h"
   #include "MeasurementMarkers.h"
   #include "MeasurementList.h"
   #include "MeasurementPixelAspectRatio.h"

   // typedefs to map the old interface names to the new names
   #if !defined(ACTIVEMIL_EXCLUDE_OLD_INTERFACES)
   #define ACTIVEMIL_EXCLUDE_OLD_INTERFACES
      typedef IMeasurementResultEdgeValues IResultEdgeValues;
   #endif // ACTIVEMIL_EXCLUDE_OLD_INTERFACES

   /////////////////////////////////////////////////////////////////////////////
   // CMeasurement wrapper class

   class CMeasurement : public CWnd
   {
     CString m_strName;

   protected:
	   DECLARE_DYNCREATE(CMeasurement)
   public:
     CLSID const& GetClsid()
	   {
		   static CLSID const clsid
			   = { 0x242f02f6, 0xa862, 0x11d1, { 0x9f, 0xd, 0x0, 0x60, 0x8c, 0xc9, 0xf9, 0xfb } };
		   return clsid;
	   }
	   virtual BOOL Create(LPCTSTR lpszClassName,
		   LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect,
		   CWnd* pParentWnd, UINT nID,
		   CCreateContext* pContext = NULL)
	   { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

       BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect, CWnd* pParentWnd, UINT nID,
		   CFile* pPersist = NULL, BOOL bStorage = FALSE,
		   BSTR bstrLicKey = NULL)
	   { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
		   pPersist, bStorage, bstrLicKey); }

   // Attributes
   public:
	   long GetMilID();
	   void SetMilID(long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   IMeasurementResults GetResults();
     IMeasurementResult GetResults(long Index);
	   void SetResults(LPDISPATCH);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   IMeasurementMarkers GetMarkers();
     IMeasurementMarker GetMarkers(long Index);
	   void SetMarkers(LPCTSTR lpszNewValue);
	   ISystem GetOwnerSystem();
	   void SetOwnerSystem(LPCTSTR lpszNewValue);
	   IImage GetImage();
	   void SetImage(LPCTSTR lpszNewValue);
	   IMeasurementList GetMeasurementList();
	   void SetMeasurementList(LPCTSTR lpszNewValue);
	   IMeasurementPixelAspectRatio GetPixelAspectRatio();
	   void SetPixelAspectRatio(LPCTSTR lpszNewValue);
	   void Allocate();
	   void Free();
	   VARIANT Inquire(long ParamToInquire);
	   void Control(long ControlType, const VARIANT& ControlValue);
	   void Modified(long ModificationType = 0);
	   void Calculate(long Index1 = 1, long Index2 = 2);
	   void FindMarker(long Index = 1);
	   void AboutBox();
   };


   /////////////////////////////////////////////////////////////////////////////
   // IMeasurement wrapper class

   class IMeasurement : public COleDispatchDriver
   {
     CString m_strName;

   public:
	   IMeasurement() {}		// Calls COleDispatchDriver default constructor
	   IMeasurement(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	   IMeasurement(const IMeasurement& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}

     static IMeasurement NewInstance();

   // Attributes
   public:
	   long GetMilID();
	   void SetMilID(long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   IMeasurementResults GetResults();
     IMeasurementResult GetResults(long Index);
	   void SetResults(LPDISPATCH);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   IMeasurementMarkers GetMarkers();
     IMeasurementMarker GetMarkers(long Index);
	   void SetMarkers(LPCTSTR lpszNewValue);
	   ISystem GetOwnerSystem();
	   void SetOwnerSystem(LPCTSTR lpszNewValue);
	   IImage GetImage();
	   void SetImage(LPCTSTR lpszNewValue);
	   IMeasurementList GetMeasurementList();
	   void SetMeasurementList(LPCTSTR lpszNewValue);
	   IMeasurementPixelAspectRatio GetPixelAspectRatio();
	   void SetPixelAspectRatio(LPCTSTR lpszNewValue);
	   void Allocate();
	   void Free();
	   VARIANT Inquire(long ParamToInquire);
	   void Control(long ControlType, const VARIANT& ControlValue);
	   void Modified(long ModificationType = 0);
	   void Calculate(long Index1 = 1, long Index2 = 2);
	   void FindMarker(long Index = 1);
	   void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_MEASUREMENT_CPLUSPLUS
