///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// Pattern Matching C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_PATTERNMATCHING_CPLUSPLUS)
#define __ACTIVEMIL_PATTERNMATCHING_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MILPatternMatching.OCX" no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CSystem wrapper class

   class CPatternMatching : public CWnd, public IPatternMatchingPtr
   {
      virtual void PreSubclassWindow()
         {
         IPatternMatching* pPat ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(IPatternMatching), (void**) &pPat) ;
         IPatternMatchingPtr::Attach(pPat, false) ;
         }
   };

#else // !ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILPatternMatchingTypes.h"

   // Dispatch interfaces referenced by this interface
   #include "System.h"
   #include "Image.h"
   #include "RegionOfInterest.h"
   #include "Angle.h"
   #include "PatternMatchingSearchAlgorithm.h"
   #include "PatternMatchingModel.h"
   #include "PatternMatchingModels.h"
   #include "PatternMatchingResult.h"
   #include "PatternMatchingResults.h"

   /////////////////////////////////////////////////////////////////////////////
   // CPatternMatching wrapper class

   class CPatternMatching : public CWnd
   {
      CString m_strName;

   protected:
      DECLARE_DYNCREATE(CPatternMatching)
   public:
      CLSID const& GetClsid()
      {
         static CLSID const clsid
            = { 0xb1599d66, 0xa7ca, 0x11d1, { 0x9f, 0xc, 0x0, 0x60, 0x8c, 0xc9, 0xf9, 0xfb } };
         return clsid;
      }
      virtual BOOL Create(LPCTSTR lpszClassName,
         LPCTSTR lpszWindowName, DWORD dwStyle,
         const RECT& rect,
         CWnd* pParentWnd, UINT nID,
         CCreateContext* pContext = NULL)
      { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

      BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
         const RECT& rect, CWnd* pParentWnd, UINT nID,
         CFile* pPersist = NULL, BOOL bStorage = FALSE,
         BSTR bstrLicKey = NULL)
      { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
         pPersist, bStorage, bstrLicKey); }

   // Attributes
   public:
      CString GetName();
      void SetName(LPCTSTR);
      BOOL GetAutomaticAllocation();
      void SetAutomaticAllocation(BOOL);
      BOOL GetPropertyValidation();
      void SetPropertyValidation(BOOL);
      BOOL GetIsAllocated();
      void SetIsAllocated(BOOL);
      IPatternMatchingResults GetResults();
      IPatternMatchingResult GetResults(long Index);
      void SetResults(LPDISPATCH);
      BOOL GetMultipleModelMode();
      void SetMultipleModelMode(BOOL);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
      ISystem GetOwnerSystem();
      void SetOwnerSystem(LPCTSTR lpszNewValue);
      IImage GetImage();
      void SetImage(LPCTSTR lpszNewValue);
      IPatternMatchingModels GetModels();
      IPatternMatchingModel GetModels(long Index);
      void SetModels(LPCTSTR lpszNewValue);
      void Allocate();
      void Free();
      void Modified(long ModificationType = 0);
      void FindModel();
      void FindModel(long Index);
      void FindModel(const VARIANT& Index);
      void FindOrientation(long IndexOrRange = 1);
      void AboutBox();
   };


   /////////////////////////////////////////////////////////////////////////////
   // IPatternMatching wrapper class

   class IPatternMatching : public COleDispatchDriver
   {
     CString m_strName;

   public:
	   IPatternMatching() {}		// Calls COleDispatchDriver default constructor
	   IPatternMatching(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	   IPatternMatching(const IPatternMatching& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}

     static IPatternMatching NewInstance();

   // Attributes
   public:
      CString GetName();
      void SetName(LPCTSTR);
      BOOL GetAutomaticAllocation();
      void SetAutomaticAllocation(BOOL);
      BOOL GetPropertyValidation();
      void SetPropertyValidation(BOOL);
      BOOL GetIsAllocated();
      void SetIsAllocated(BOOL);
      IPatternMatchingResults GetResults();
      IPatternMatchingResult GetResults(long Index);
      void SetResults(LPDISPATCH);
      BOOL GetMultipleModelMode();
      void SetMultipleModelMode(BOOL);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
      ISystem GetOwnerSystem();
      void SetOwnerSystem(LPCTSTR lpszNewValue);
      IImage GetImage();
      void SetImage(LPCTSTR lpszNewValue);
      IPatternMatchingModels GetModels();
      IPatternMatchingModel GetModels(long Index);
      void SetModels(LPCTSTR lpszNewValue);
      void Allocate();
      void Free();
      void Modified(long ModificationType = 0);
      void FindModel();
      void FindModel(long Index);
      void FindModel(const VARIANT& Index);
      void FindOrientation(long IndexOrRange = 1);
      void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_PATTERNMATCHING_CPLUSPLUS
