///////////////////////////////////////////////////////////////////////////////
//
// Matrox Imaging ActiveX Controls
// System C++ Declarations
//
// Matrox Electronics Ltd.
// Copyright 1998
// All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined(__ACTIVEMIL_SYSTEM_CPLUSPLUS)
#define __ACTIVEMIL_SYSTEM_CPLUSPLUS

#if !defined(ACTIVEMIL_IN_PROJECT)
#error ActiveMIL support has not been added to this project.  Make sure that the "Matrox ActiveMIL VC++ Add-In" is enabled, and that "Add ActiveMIL To Project" has been called from the add-in's toolbar.
#endif

#if defined(ACTIVEMIL_USE_COM_CLASS)

   #pragma warning(push)
   #pragma warning(disable : 4146)
   #import "MIL.OCX" no_namespace
   #pragma warning(pop)


   /////////////////////////////////////////////////////////////////////////////
   // CSystem wrapper class

   class CSystem : public CWnd, public ISystemPtr
   {
      virtual void PreSubclassWindow()
         {
         ISystem* pSyst ;
         CWnd::GetControlUnknown()->QueryInterface(__uuidof(ISystem), (void**) &pSyst) ;
         ISystemPtr::Attach(pSyst, false) ;
         }
   };

#else // !ACTIVEMIL_USE_COM_CLASS

   // Include enumerated types
   #include "ActiveMILBaseTypes.h"

   // Dispatch interfaces referenced by this interface
   #include "GrabInDisplay.h"

   /////////////////////////////////////////////////////////////////////////////
   // CSystem wrapper class

   class CSystem : public CWnd
   {
      CString m_strName;

   protected:
	   DECLARE_DYNCREATE(CSystem)
   public:
	   CLSID const& GetClsid()
	   {
		   static CLSID const clsid
			   = { 0x6d9f7f74, 0x9658, 0x11d0, { 0xbd, 0xb5, 0x0, 0x60, 0x8c, 0xc9, 0xf9, 0xfb } };
		   return clsid;
	   }
	   virtual BOOL Create(LPCTSTR lpszClassName,
		   LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect,
		   CWnd* pParentWnd, UINT nID,
		   CCreateContext* pContext = NULL)
	   { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID); }

       BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
		   const RECT& rect, CWnd* pParentWnd, UINT nID,
		   CFile* pPersist = NULL, BOOL bStorage = FALSE,
		   BSTR bstrLicKey = NULL)
	   { return CreateControl(GetClsid(), lpszWindowName, dwStyle, rect, pParentWnd, nID,
		   pPersist, bStorage, bstrLicKey); }

   // Attributes
   public:
	   CString GetBoardType();
	   void SetBoardType(LPCTSTR);
	   long GetBoardRevision();
	   void SetBoardRevision(long);
	   long GetRGBModuleNumber();
	   void SetRGBModuleNumber(long);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetDCFSupported();
	   void SetDCFSupported(BOOL);
	   BOOL GetDualScreenMode();
	   void SetDualScreenMode(BOOL);
	   long GetDeviceNumber();
	   void SetDeviceNumber(long);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   long GetInitializationFlag();
	   void SetInitializationFlag(long);
	   long GetMilID();
	   void SetMilID(long);
	   BOOL GetMMXEnabled();
	   void SetMMXEnabled(BOOL);
	   long GetNumberOfDigitizers();
	   void SetNumberOfDigitizers(long);
	   long GetNumberOfDisplays();
	   void SetNumberOfDisplays(long);
	   long GetNumberOfProcessors();
	   void SetNumberOfProcessors(long);
	   long GetPhysicalAddressOverlay();
	   void SetPhysicalAddressOverlay(long);
	   long GetPhysicalAddressUnderlay();
	   void SetPhysicalAddressUnderlay(long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   CString GetSystemType();
	   void SetSystemType(LPCTSTR);
	   CString GetProcessingSystem();
	   void SetProcessingSystem(LPCTSTR);
	   CString GetProcessingSystemType();
	   void SetProcessingSystemType(LPCTSTR);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   BOOL GetCompressionSupported();
	   void SetCompressionSupported(BOOL);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   IGrabInDisplay GetGrabInDisplay();
	   void SetGrabInDisplay(LPCTSTR lpszNewValue);
	   void Free();
	   void Allocate();
	   VARIANT Inquire(long ParamToInquire);
	   void Control(long ControlType, const VARIANT& ControlFlag);
	   void Modified(long ModificationType = 0);
	   void AboutBox();
   };

   /////////////////////////////////////////////////////////////////////////////
   // ISystem wrapper class

   class ISystem : public COleDispatchDriver
   {
      CString m_strName;

   public:
	   ISystem() {}		// Calls COleDispatchDriver default constructor
	   ISystem(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	   ISystem(const ISystem& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}

     static ISystem NewInstance();

    // Attributes
   public:
	   CString GetBoardType();
	   void SetBoardType(LPCTSTR);
	   long GetBoardRevision();
	   void SetBoardRevision(long);
	   long GetRGBModuleNumber();
	   void SetRGBModuleNumber(long);
	   BOOL GetAutomaticAllocation();
	   void SetAutomaticAllocation(BOOL);
	   BOOL GetDCFSupported();
	   void SetDCFSupported(BOOL);
	   BOOL GetDualScreenMode();
	   void SetDualScreenMode(BOOL);
	   long GetDeviceNumber();
	   void SetDeviceNumber(long);
	   BOOL GetPropertyValidation();
	   void SetPropertyValidation(BOOL);
	   long GetInitializationFlag();
	   void SetInitializationFlag(long);
	   long GetMilID();
	   void SetMilID(long);
	   BOOL GetMMXEnabled();
	   void SetMMXEnabled(BOOL);
	   long GetNumberOfDigitizers();
	   void SetNumberOfDigitizers(long);
	   long GetNumberOfDisplays();
	   void SetNumberOfDisplays(long);
	   long GetNumberOfProcessors();
	   void SetNumberOfProcessors(long);
	   long GetPhysicalAddressOverlay();
	   void SetPhysicalAddressOverlay(long);
	   long GetPhysicalAddressUnderlay();
	   void SetPhysicalAddressUnderlay(long);
	   CString GetName();
	   void SetName(LPCTSTR);
	   CString GetSystemType();
	   void SetSystemType(LPCTSTR);
	   CString GetProcessingSystem();
	   void SetProcessingSystem(LPCTSTR);
	   CString GetProcessingSystemType();
	   void SetProcessingSystemType(LPCTSTR);
	   BOOL GetIsAllocated();
	   void SetIsAllocated(BOOL);
	   BOOL GetCompressionSupported();
	   void SetCompressionSupported(BOOL);

      // Default property
      operator LPCTSTR() { m_strName.LockBuffer(); m_strName = GetName(); return m_strName; }

   // Operations
   public:
	   IGrabInDisplay GetGrabInDisplay();
	   void SetGrabInDisplay(LPCTSTR lpszNewValue);
	   void Free();
	   void Allocate();
	   VARIANT Inquire(long ParamToInquire);
	   void Control(long ControlType, const VARIANT& ControlFlag);
	   void Modified(long ModificationType = 0);
	   void AboutBox();
   };

#endif // ACTIVEMIL_USE_COM_CLASS

#endif // __ACTIVEMIL_SYSTEM_CPLUSPLUS
