// MColorLite example

// Synopsis:   This program loads a color image into a color image buffer,
//             and displays it in a MilDisplay.
//             It then adds color annotation to it.

// NOTE: Check the history logs to determine which properties were set at 
//       design time.

// ExampleDlg.cpp: implementation file
//

#include "stdafx.h"
#include "ExampleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Constants for processing
const long IMAGE_LUMINANCE_OFFSET =  40;
const long IMAGE_MAX_VALUE        = 255;


// Location of image file
static const char EXAMPLE_IMAGE_PATH[] = "..\\..\\..\\..\\..\\images\\bird.mim";  


BEGIN_MESSAGE_MAP(CExampleDlg, CDialog)
	//{{AFX_MSG_MAP(CExampleDlg)
	ON_BN_CLICKED(IDC_NEXTBUTTON, OnNextbutton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// The one and only CExampleApp object
//
CExampleApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
CExampleDlg::CExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CExampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExampleDlg)
	m_strCommentText = _T("");
	//}}AFX_DATA_INIT

   m_nStepCount = 1;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg data exchange
//
void CExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExampleDlg)
	DDX_Text(pDX, IDC_COMMENTTEXT, m_strCommentText);
	DDX_Control(pDX, IDC_DISPLAY, m_Display);
	DDX_Control(pDX, IDC_IMAGE_BUFFER, m_ImageBuffer);
	DDX_Control(pDX, IDC_CHILD_IMAGE_1, m_ChildImage1);
	DDX_Control(pDX, IDC_CHILD_IMAGE_2, m_ChildImage2);
	DDX_Control(pDX, IDC_BLUE_GRAPHIC_CONTEXT, m_BlueGraphicContext);
	DDX_Control(pDX, IDC_GREEN_GRAPHIC_CONTEXT, m_GreenGraphicContext);
	DDX_Control(pDX, IDC_RED_GRAPHIC_CONTEXT, m_RedGraphicContext);
	//}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg message handlers
//
BOOL CExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Resize the dialog 
   AdjustDialog(EXAMPLE_IMAGE_PATH);

   ///////
   // Step 1 : Load source image into image buffer

   // Get the size of the image
   long nImageSizeX = m_ImageBuffer.FileInquire(EXAMPLE_IMAGE_PATH, mSizeX);
   long nImageSizeY = m_ImageBuffer.FileInquire(EXAMPLE_IMAGE_PATH, mSizeY);

   // Allocate m_ImageBuffer with a width of twice the image by setting its size
   // and loading the image without resizing
   //
   // Note: "ImageBuffer" has the attribute AutomaticAllocation set to FALSE
   m_ImageBuffer.SetSizeX(2*nImageSizeX);
   m_ImageBuffer.SetSizeY(nImageSizeY);

   // Allocate image and clear it to prevent the display of garbage data in
   // the area outside the example image
   m_ImageBuffer.Allocate();
   m_ImageBuffer.Clear(0);

   // Load the example image in the image without resizing it
   m_ImageBuffer.Load(EXAMPLE_IMAGE_PATH, FALSE);

   // Arrange the child images to be side by side within the image buffer
   m_ChildImage1.GetChildRegion().SetSizeX(nImageSizeX);
   m_ChildImage1.GetChildRegion().SetSizeY(nImageSizeY);

   m_ChildImage2.GetChildRegion().SetSizeX(nImageSizeX);
   m_ChildImage2.GetChildRegion().SetSizeY(nImageSizeY);
   m_ChildImage2.GetChildRegion().SetOffsetX(nImageSizeX);
   m_ChildImage2.GetChildRegion().SetOffsetY(0);

   // Print message
   m_strCommentText = "A color source image was loaded and displayed.  "
                      "Click Next to continue.  ";
   
	UpdateData(FALSE);
	return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// OnNextButton message handler.
// This function drives the example through each step, performing appropriate
// actions
//
void CExampleDlg::OnNextbutton() 
{
   // Increments the step counter and updates text in the dialog 
   AdjustStepNumber();

   switch (m_nStepCount)
      {
      ///////
      // Step 2: 
      case 2:
         {
         // Copy image to right sub image
         m_ChildImage2.Copy(m_ChildImage1, mAllBands);

         // Draw annotations in each components of the copied image
         IRegionOfInterest RedDrawingRegion = m_RedGraphicContext.GetDrawingRegion();
         RedDrawingRegion.SetStartX(m_ChildImage2.GetSizeX() / 16);
         RedDrawingRegion.SetStartY(m_ChildImage2.GetSizeY() / 8);
         m_RedGraphicContext.SetForegroundColor(0xFF);
         m_RedGraphicContext.Text("TOUCAN");

         IRegionOfInterest GreenDrawingRegion = m_GreenGraphicContext.GetDrawingRegion();
         GreenDrawingRegion.SetStartX(m_ChildImage2.GetSizeX() / 16);
         GreenDrawingRegion.SetStartY(m_ChildImage2.GetSizeY() / 8);
         m_GreenGraphicContext.SetForegroundColor(0x80);
         m_GreenGraphicContext.Text("TOUCAN");

         IRegionOfInterest BlueDrawingRegion = m_BlueGraphicContext.GetDrawingRegion();
         BlueDrawingRegion.SetStartX(m_ChildImage2.GetSizeX() / 16);
         BlueDrawingRegion.SetStartY(m_ChildImage2.GetSizeY() / 8);
         m_BlueGraphicContext.SetForegroundColor(0x0);
         m_BlueGraphicContext.Text("TOUCAN");

         
         // Print message
         m_strCommentText = "The color source image on the left was copied in the "
                            "image on the right and color text annotation was done in it.  "
                            "Click Next to end.  ";
         break;
         }

      ///////
      // Last Step: End the application
      default:
         {
         // Close dialog application
         EndDialog(IDOK);
         break;
         }
      }
	UpdateData(FALSE);
}
