// MError example

// Synopsis:   This program illustrates the use of C++ exception handling
//             to manage ActiveMIL errors.

// NOTE: Check the history logs to determine which properties were set a 
//       design time.

// ExampleDlg.cpp: implementation file
//

#include "stdafx.h"
#include "ExampleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


BEGIN_MESSAGE_MAP(CExampleDlg, CDialog)
	//{{AFX_MSG_MAP(CExampleDlg)
	ON_BN_CLICKED(IDC_NEXTBUTTON, OnNextbutton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////
// The one and only CExampleApp object
//
CExampleApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
CExampleDlg::CExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CExampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExampleDlg)
	m_strCommentText = _T("");
	//}}AFX_DATA_INIT

   m_nStepCount = 1;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg data exchange
//
void CExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExampleDlg)
	DDX_Control(pDX, IDC_SYSTEM, m_System);
	DDX_Control(pDX, IDC_DISPLAY, m_Display);
	DDX_Text(pDX, IDC_COMMENTTEXT, m_strCommentText);
	DDX_Control(pDX, IDC_GRAPHIC_CONTEXT, m_GraphicContext);
	DDX_Control(pDX, IDC_IMAGE_BUFFER, m_ImageBuffer);
	//}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg message handlers
//
BOOL CExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Resize the dialog 
   AdjustDialog();

   // Step 1 : Display message
   m_strCommentText = "This program illustrates the use of C++ exception handling "
                      "to manage ActiveMIL errors.  "
                      "Click Next to continue.  ";

	UpdateData(FALSE);
	return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// OnNextButton message handler.
// This function drives the example through each step, performing appropriate
// actions
//
void CExampleDlg::OnNextbutton() 
{
   // Increments the step counter and updates text in the dialog 
   AdjustStepNumber();

   switch (m_nStepCount)
      {
      // Step 2: Try to allocate image buffer and draw in the allocated image
      case 2:
         {
         // Try to allocate image buffer and handle eventual exception(s)
         try 
            {
            // ****uncomment the line below to see exception handling in action****

            // For the generation of an error by trying to allocate a image of -1 pixels width...
            //m_ImageBuffer.SetSizeX(-1);

            // Allocate image buffer
            m_ImageBuffer.Allocate();

            // Clear it
            m_ImageBuffer.Clear(0);

            // Associate it with the display
            m_Display.SetImage(m_ImageBuffer);

            m_GraphicContext.Arc(0.0, 360.0, TRUE);
            m_strCommentText = "Image succesfully allocated.  "
                               "A circle was drawn in the displayed image buffer.  "
                               "Click Next to end.  ";

            }
         catch (COleDispatchException *e)
            {
            // Display error message
            MessageBox (e->m_strDescription + "\n\nClick OK to end.  ",  // LPCTSTR lpszText
                        e->m_strSource);                                 // LPCTSTR lpszCaption

            // Always call Delete() on caught exceptions
            e->Delete();

            // Close dialog application
            EndDialog(IDOK);
            }

         break;
         }

      // Last Step: End the application
      default:
         // Close dialog application
         EndDialog(IDOK);
         break;
      }
	UpdateData(FALSE);
}
