// MMultipleDisplayLite example

// Synopsis:   This program shows how to display more than one
//             image buffer at a time on a single display. It
//             allocates a displayable image buffer, allocates
//             two child images from it, and then uses the child
//             images as the source and destination for write text.
//
//             Note the display will be zoomed if the system's
//             display supports it.

// NOTE: Check the history logs to determine which properties were set a
//       design time.

// ExampleDlg.cpp: implementation file
//

#include "stdafx.h"
#include "ExampleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Location of image file
const char EXAMPLE_IMAGE_PATH[] = "..\\..\\..\\..\\..\\images\\cell.mim";  


// Constants
const long ZOOM_VALUE = 2;

BEGIN_MESSAGE_MAP(CExampleDlg, CDialog)
	//{{AFX_MSG_MAP(CExampleDlg)
	ON_BN_CLICKED(IDC_NEXTBUTTON, OnNextbutton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// The one and only CExampleApp object
//
CExampleApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
CExampleDlg::CExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CExampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExampleDlg)
	m_strCommentText = _T("");
	//}}AFX_DATA_INIT

   m_nStepCount = 1;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg data exchange
//
void CExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExampleDlg)
	DDX_Text(pDX, IDC_COMMENTTEXT, m_strCommentText);
	DDX_Control(pDX, IDC_IMAGE_BUFFER, m_ImageBuffer);
	DDX_Control(pDX, IDC_DISPLAY, m_Display);
	DDX_Control(pDX, IDC_GRAPHIC_CONTEXT_1, m_GraphicContext1);
	DDX_Control(pDX, IDC_GRAPHIC_CONTEXT_2, m_GraphicContext2);
	DDX_Control(pDX, IDC_CHILD_IMAGE_1, m_ChildImage1);
	DDX_Control(pDX, IDC_CHILD_IMAGE_2, m_ChildImage2);
	//}}AFX_DATA_MAP
}



/////////////////////////////////////////////////////////////////////////////
// CExampleDlg message handlers
//
BOOL CExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Resize the dialog 
   AdjustDialog(EXAMPLE_IMAGE_PATH);

   ///////
   // Step 1 : Load source image into image buffer and write text into each child image
   m_ChildImage1.Load(EXAMPLE_IMAGE_PATH);
   m_ChildImage2.Copy(m_ChildImage1, mAllBands);

   m_GraphicContext1.GetDrawingRegion().SetStartX((m_ChildImage1.GetSizeX() / 4) + 1);
   m_GraphicContext1.GetDrawingRegion().SetStartY(m_ChildImage1.GetSizeY() / 2);
   m_GraphicContext2.GetDrawingRegion().SetStartX((m_ChildImage2.GetSizeX() / 4) - 7);
   m_GraphicContext2.GetDrawingRegion().SetStartY(m_ChildImage2.GetSizeY() / 2);
   m_GraphicContext1.Text("Source");
   m_GraphicContext2.Text("Destination");

   // Print message.
   m_strCommentText = "A copy was performed between the sub-image on the left side of "
                      "the screen and the sub-image on the right side of the screen and "
                      "text was written into each of them.  Click Next to continue.  ";
   
	UpdateData(FALSE);
	return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// OnNextButton message handler.
// This function drives the example through each step, performing appropriate
// actions
//
void CExampleDlg::OnNextbutton() 
{
   // Increments the step counter and updates text in the dialog 
   AdjustStepNumber();

   switch (m_nStepCount)
      {
      ///////
      // Step 2: Zoom the image
      case 2:
         {
         // Zoom displayed image.
         m_Display.Zoom(ZOOM_VALUE, ZOOM_VALUE, FALSE);

         // Print message.
         CString str;
         str.Format("Display zoomed by %d in X and Y (if supported).  "
                    "Click Next to end.  ", ZOOM_VALUE);

         m_strCommentText = str;
         break;
         }
      ///////
      // Last Step: End the application
      case 3:
      default:
         // Close dialog application
         EndDialog(IDOK);
         break;
      }
	UpdateData(FALSE);
}

