// MGrabEvent example

// Synopsis:   This example grabs a sequence of images using a hook function 
//             and plays it back.
//
//             Note this assumes that the copy operation is shorter than a 
//             grab and that the PC has sufficient bandwith to support the
//             operations.

// NOTE: Check the history logs to determine which properties were set a
//       design time.

// ExampleDlg.cpp: implementation file
//

#include "stdafx.h"
#include <afxpriv.h>              // For WM_KICKIDLE
#include "ExampleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


BEGIN_MESSAGE_MAP(CExampleDlg, CDialog)
	//{{AFX_MSG_MAP(CExampleDlg)
	ON_BN_CLICKED(IDC_NEXTBUTTON, OnNextbutton)
	//}}AFX_MSG_MAP
   ON_MESSAGE(WM_KICKIDLE, OnKickIdle)
END_MESSAGE_MAP()


BEGIN_EVENTSINK_MAP(CExampleDlg, CDialog)
    //{{AFX_EVENTSINK_MAP(CExampleDlg)
	ON_EVENT(CExampleDlg, IDC_DIGITIZER, 1 /* GrabStart */, OnGrabStartDigitizer, VTS_NONE)
	//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()


/////////////////////////////////////////////////////////////////////////////
// The one and only CExampleApp object
//
CExampleApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
CExampleDlg::CExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CExampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExampleDlg)
	m_strCommentText = _T("");
	//}}AFX_DATA_INIT

   m_nStepCount = 1;
   m_nGrabStartCount = -2;
   m_nFrameCount = 0;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg data exchange
//
void CExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExampleDlg)
	DDX_Control(pDX, IDC_NEXTBUTTON, m_NextButton);
	DDX_Text(pDX, IDC_COMMENTTEXT, m_strCommentText);
	DDX_Control(pDX, IDC_IMAGE_BUFFER, m_ImageBuffer);
	DDX_Control(pDX, IDC_DISPLAY, m_Display);
	DDX_Control(pDX, IDC_DIGITIZER, m_Digitizer);
	DDX_Control(pDX, IDC_SYSTEM, m_System);
	DDX_Control(pDX, IDC_APPLICATION, m_Application);
	//}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg message handlers
//
BOOL CExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Resize the dialog 
   AdjustDialog();

   ///////
   // Step 1: Resize and allocate the ImageBuffer, and start the grab.

   m_ImageBuffer.SetSizeX((long)(m_Digitizer.GetSizeX() * m_Digitizer.GetScaleX()));
   m_ImageBuffer.SetSizeY((long)(m_Digitizer.GetSizeY() * m_Digitizer.GetScaleY()));
   m_ImageBuffer.Allocate();
   m_Digitizer.GrabContinuous();

   // Print message
   m_strCommentText = "This will record a sequence using ActiveMIL Digitizer events.  "
                      "Click Next to continue.  ";

	UpdateData(FALSE);
	return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// OnNextButton message handler.
// This function drives the example through each step, performing appropriate
// actions
//
void CExampleDlg::OnNextbutton() 
{
   // Increments the step counter and updates text in the dialog 
   AdjustStepNumber();

   switch (m_nStepCount)
      {
      ///////
      // Step 2: Grab the sequence
      case 2:
         {
         // Stop continuous grab
         m_Digitizer.Halt();

         // Print message
         m_strCommentText = "Allocating sequence image...  ";
      	UpdateData(FALSE);

         // Allocate all the sequence images
         for (int i = 0; i < MAX_NUMBER_OF_FRAMES_TO_GRAB; i++)
            m_aSequenceImages[i] = m_ImageBuffer.Clone();

         // Set the Digitizer's GrabStartEvent property to enable the
         // OnGrabStartDigitizer function, which is responsible for the
         // sequence acquisition
         m_Digitizer.SetGrabStartEvent(TRUE);

         // Start the sequence grab, which will be continued in the event handlers.
         m_Digitizer.SetImage(m_aSequenceImages[m_nFrameCount]);
         m_Digitizer.Grab();

         // Disable the Next button
         m_NextButton.EnableWindow(FALSE);
         break;
         }

      ///////
      // Step 3: Play the sequence
      case 3:
         {
         // Wait for last grab
         m_Digitizer.GrabWait(mGrabEnd);

         // Print statistics
         CString str;
         str.Format("Playing the sequence; %d Frames grabbed at a frame rate of %.2f frames/sec "
                    "(%.2f ms/frame).  "
                    "Click Next to end.  ",
                    MAX_NUMBER_OF_FRAMES_TO_GRAB,
                    MAX_NUMBER_OF_FRAMES_TO_GRAB / m_dTime,
                    1000.0 * m_dTime / MAX_NUMBER_OF_FRAMES_TO_GRAB);
         m_strCommentText = str;

         // To be continued in OnKickIdle below...
         m_nFrameCount = 0;

         break;
         }

      ///////
      // Last Step: End the application
      default:
         {
         // Close dialog application
         EndDialog(IDOK);
         break;
         }
      }
	UpdateData(FALSE);
}


/////////////////////////////////////////////////////////////////////////////
// OnGrabStartDigitizer() event handler.
// This function gets called by the digitizer at the start of each grab
// provided that the Digitizer's GrabStartEvent property is set 
//
void CExampleDlg::OnGrabStartDigitizer() 
{
   EXAMPLE_EVENT_HANDLER_PROLOGUE

   m_nGrabStartCount++;

   // If we are starting, reset the timer
   if (m_nGrabStartCount == 0)
      m_Application.GetTimer().Reset();

   // Grab the next frame if we have not reached the total number of frames
   if (m_nGrabStartCount <= MAX_NUMBER_OF_FRAMES_TO_GRAB)
      {
      // Set the Digitizer to grab the next frame in the next image in sequence
      m_Digitizer.SetImage(m_aSequenceImages[m_nFrameCount]);
      if (m_nGrabStartCount > 0)
         m_nFrameCount++;

      // Start grab
      m_Digitizer.Grab();

      // Print status message
      if (m_nGrabStartCount > 0)
         {
         m_strCommentText.Format("Grabbing image %d of %d...", 
                                 m_nFrameCount, MAX_NUMBER_OF_FRAMES_TO_GRAB) ;
         }
      else
         {
         m_strCommentText = "Synchronizing start of sequence...";
         }

      UpdateData(FALSE);
      }

   // if we are finished, get the time and re-enable the next button
   if (m_nGrabStartCount == MAX_NUMBER_OF_FRAMES_TO_GRAB)
      {
      // Get the time it took to grab the sequence
      m_dTime = m_Application.GetTimer().Read();

      // Enable the Next button
      m_NextButton.EnableWindow(TRUE);
      m_NextButton.SetFocus();

      m_strCommentText = "Sequence grabbed.  Click Next to play the recorded sequence.  ";
      UpdateData(FALSE);
      }

   EXAMPLE_EVENT_HANDLER_EPILOGUE
}


/////////////////////////////////////////////////////////////////////////////
// Handle MFC private message WM_KICKIDLE. LPARAM is the idle count. 
// This message is the equivalent of CWinApp::OnIdle for dialog based
// applications, and is shown as an alternative to using a secondary
// thread for continuous processing (the playback in this case).
//
LRESULT CExampleDlg::OnKickIdle(WPARAM, LPARAM)
{
   if (m_nStepCount == 3)
      {
      double dTimeWait = 0;

      // Reset timer and copy next image to display
      m_Application.GetTimer().Reset();
      m_ImageBuffer.Copy(m_aSequenceImages[m_nFrameCount], mAllBands);

      // Compute how much time we must still wait for this frame
      dTimeWait = m_Application.GetTimer().Read();
      dTimeWait = m_dTime / MAX_NUMBER_OF_FRAMES_TO_GRAB - dTimeWait;

      // Wait for remaining time
      m_Application.GetTimer().Wait(dTimeWait);

      // Adjust frame number
      m_nFrameCount = ++m_nFrameCount % MAX_NUMBER_OF_FRAMES_TO_GRAB;

      // Return TRUE to continue idle processing, since step 3 is a continuous
      // playback.
      return TRUE ;
      }
   
   // Return FALSE to discontinue idle processing, since we are not doing any 
   // continuous processing in these steps.
   return FALSE ;
}
