// MGrabEvent example

// This example grabs a sequence of images using a hook function and plays 
// it back.
//
// Note: This assumes that the copy operation is shorter than a grab       
//       and that the PC has sufficient bandwith to support the           
//       operations.

// ExampleDlg.h : header file
//
//{{AFX_INCLUDES()
#include "Image.h"
#include "Digitizer.h"
#include "Display.h"
#include "System.h"
#include "Application.h"
//}}AFX_INCLUDES

#if !defined(AFX_EXAMPLEDLG_H_MGrabEvent_INCLUDED_)
#define AFX_EXAMPLEDLG_H_MGrabEvent_INCLUDED_

#include "resource.h"		// main symbols

/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
class CExampleDlg : public CDialog
{
// Constants
   enum 
      {
      MAX_NUMBER_OF_FRAMES_TO_GRAB = 24   // the number of frames to grab
      };

// Construction
public:
	CExampleDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	//{{AFX_DATA(CExampleDlg)
	enum { IDD = IDD_EXAMPLE_DIALOG };
	CButton	      m_NextButton;
	CString	      m_strCommentText;
	CImage         m_ImageBuffer;
	CDisplay       m_Display;
	CDigitizer     m_Digitizer;
	CSystem        m_System;
	CApplication   m_Application;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CExampleDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
   long m_nStepCount;            // Keeps track of the current example step 
   void AdjustDialog();          // Resizes the dialog to fit any font size and digitizer resolution
   void AdjustStepNumber();      // Increments the step number and updates text in the dialog

   int m_nGrabStartCount;        // counts the number of GrabStart events
   int m_nFrameCount;            // index to the frame being grabbed 
   
   IImage m_aSequenceImages[MAX_NUMBER_OF_FRAMES_TO_GRAB]; // array of images to hold the grabbed sequence
   double m_dTime;

	// Generated message map functions
	//{{AFX_MSG(CExampleDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnNextbutton();
	afx_msg void OnGrabStartDigitizer();
   afx_msg LRESULT OnKickIdle(WPARAM, LPARAM);
	//}}AFX_MSG
	DECLARE_EVENTSINK_MAP()
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
// CExampleApp:
// See Example.cpp for the implementation of this class
//
class CExampleApp : public CWinApp
{
public:
   CExampleApp() {};

	//{{AFX_VIRTUAL(CExampleApp)
	inline virtual LRESULT ProcessWndProcException( CException* e, const MSG* pMsg );
	inline virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

};


/////////////////////////////////////////////////////////////////////////////
// CExampleApp initialization
//
BOOL CExampleApp::InitInstance()
{
	AfxEnableControlContainer();

	CExampleDlg dlg;
	m_pMainWnd = &dlg;
	dlg.DoModal();

	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleApp default exception handling
 
LRESULT CExampleApp::ProcessWndProcException( CException* e, const MSG* pMsg )
{
   e->ReportError() ;
   static_cast<CDialog*>(m_pMainWnd)->EndDialog(IDCANCEL) ;
 
   return 1 ;
}
 

/////////////////////////////////////////////////////////////////////////////
// EXAMPLE_EVENT_HANDLER_PROLOGUE and EXAMPLE_EVENT_HANDLER_EPILOGUE
// MFC does catch exceptions thrown by ActiveX controls in the
// context of event sinks (i.e. event handlers), but does not 
// rethrows them, they are deleted and lost.  These macros 
// simulate the behaviour you get in a message handler, and are
// used in the digitizer event handlers of this example.

#define EXAMPLE_EVENT_HANDLER_PROLOGUE    \
   try {

#define EXAMPLE_EVENT_HANDLER_EPILOGUE    \
      }                                   \
   catch (COleDispatchException *e)       \
      {                                   \
      e->ReportError();                   \
      e->Delete();                        \
      ExitProcess(1);                     \
      }
 
//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif 
