// MGrabSequence example

// This example grabs a sequence of images using a hook function and plays 
// it back.
//
// Note: This assumes that the copy operation is shorter than a grab       
//       and that the PC has sufficient bandwith to support the           
//       operations.

// ExampleDlg.h : header file
//
//{{AFX_INCLUDES()
#include "Image.h"
#include "Digitizer.h"
#include "Display.h"
#include "System.h"
#include "Application.h"
//}}AFX_INCLUDES

#if !defined(AFX_EXAMPLEDLG_H_MGrabSequence_INCLUDED_)
#define AFX_EXAMPLEDLG_H_MGrabSequence_INCLUDED_

#include "resource.h"		// main symbols

/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
class CExampleDlg : public CDialog
{
// Construction
public:
	CExampleDlg(CWnd* pParent = NULL);	// standard constructor
   ~CExampleDlg();

// Dialog Data
	//{{AFX_DATA(CExampleDlg)
	enum { IDD = IDD_EXAMPLE_DIALOG };
	CButton	m_optJPEGLossyCompression;
	CButton	m_optUncompressed;
	CButton	m_chkSaveFile;
	CButton        m_NextButton;
	CString        m_strCommentText;
	CImage         m_ImageBuffer;
	CDisplay	      m_Display;
	CDigitizer     m_Digitizer;
	CSystem        m_System;
	CApplication   m_Application;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CExampleDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

   // Constants
   enum 
      {
      MAXIMUM_NUMBER_OF_FRAMES = 100,
      Q_FACTOR = 50
      };

   // Array of images to hold the grabbed sequence
   CArray<IImage, IImage&> m_aSequenceImages; 

   long m_nStepCount;                     // Keeps track of the current example step 
   void AdjustDialog();                   // Resizes the dialog to fit any font size and digitizer resolution
   void AdjustStepNumber();               // Increments the step number and updates text in the dialog

   int m_nNumberOfFrames;                 // Number of frame that will be grabbed
   double m_dTime;                        // Time it took to grab the sequence
   BOOL m_bUseCompression;                // Grab compressed images or not
   double m_dAverageFramePlaybackTime ;   // Average time needed to playback a single frame
   
   // Playback thread related members
   long     m_bStopPlayback;   // flag for playback thread termination
   long     m_bKillPlayback;   // flag for playback thread termination
   HANDLE   m_hPlaybackThread; // pointer to playback thread object (used for synchronisation)
   static UINT PlaybackThreadFunction(LPVOID pParam); // playback thread control function
   void PlaybackWorkFunction();    // playback thread working function

   // Custom Windows message handler
   afx_msg LONG OnPlaybackWorkFunctionEnd( UINT, LONG );


	// Generated message map functions
	//{{AFX_MSG(CExampleDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnNextbutton();
	afx_msg void OnClose();
	afx_msg void OnOptUncompressed();
	afx_msg void OnOptJPEGLossyCompression();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
// CExampleApp:
// See Example.cpp for the implementation of this class
//
class CExampleApp : public CWinApp
{
public:
   CExampleApp() {};

	//{{AFX_VIRTUAL(CExampleApp)
	inline virtual LRESULT ProcessWndProcException( CException* e, const MSG* pMsg );
	inline virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

};


/////////////////////////////////////////////////////////////////////////////
// CExampleApp initialization
//
BOOL CExampleApp::InitInstance()
{
	AfxEnableControlContainer();

	CExampleDlg dlg;
	m_pMainWnd = &dlg;
	dlg.DoModal();

	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleApp default exception handling
 
LRESULT CExampleApp::ProcessWndProcException( CException* e, const MSG* pMsg )
{
   e->ReportError() ;
   static_cast<CDialog*>(m_pMainWnd)->EndDialog(IDCANCEL) ;
 
   return 1 ;
}
 
 
//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif 
