; File name: mblobcnt.cl
; Synopsis:  This program loads an image of some nuts, bolts and
;            washers and determines the number of each of these.
;

; Disable the automatic printing of the return values of commands
silence 1


; Target MIL image file specifications. 
macro IMAGE_FILE            "\bolts.mim" 
macro IMAGE_WIDTH           512
macro IMAGE_HEIGHT          480
macro IMAGE_THRESHOLD_VALUE  24

; Radius of the smallest particles to keep.
macro MIN_BLOB_RADIUS         3

; Minimum and maximum area of blobs.
macro MIN_BLOB_AREA          50.0
macro MAX_BLOB_AREA       50000.0

; Minimum hole compactness corresponding to a washer.
macro MIN_COMPACTNESS        1.5

  MIL_ID MilApplication       ; Application identifier. 
  MIL_ID MilSystem            ; System identifier. 
  MIL_ID MilDisplay           ; Display identifier.
  MIL_ID MilImage             ; Image buffer identifier. 
  MIL_ID BinImage             ; Binary image buffer identifier.
  MIL_ID BlobResult           ; Blob result buffer identifier. 
  MIL_ID FeatureList          ; Feature list identifier. 
  long   TotalBlobs           ; Total number of blobs. 
  long   BlobsWithHoles       ; Number of blobs with holes. 
  long   BlobsWithRoughHoles  ; Number of blobs with rough holes. 


  ; Allocate defaults 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

  ; Allocate a binary image buffer for fast processing.
   MbufAlloc2d M_DEFAULT IMAGE_WIDTH IMAGE_HEIGHT (1+M_UNSIGNED) (M_IMAGE+M_PROC) &BinImage
   
  ; Load blob image into image buffer. 
   setstr str_image [ addstr imagepath IMAGE_FILE ]
   MbufLoad [getstr str_image] MilImage

  ; Pause to show the original image. 
   echo "This program determines the number of nuts"
   echo "washers, and bolts in the displayed image."
   pause "Press <Enter> to continue."

  ; Binarize the image.
   MimBinarize MilImage BinImage M_GREATER_OR_EQUAL IMAGE_THRESHOLD_VALUE M_NULL
   
  ; Remove small particles and then remove small holes. 
   MimOpen BinImage BinImage MIN_BLOB_RADIUS M_BINARY
   MimClose BinImage BinImage MIN_BLOB_RADIUS M_BINARY

  ; Allocate a feature list.
   MblobAllocFeatureList MilSystem &FeatureList

  ; Specify that area will be used to select blobs of interest.
   MblobSelectFeature FeatureList M_AREA

  ; Allocate a buffer for the results. 
   MblobAllocResult MilSystem &BlobResult

  ; Calculate selected feature for each blob. 
   MblobCalculate BinImage M_NULL FeatureList BlobResult

  ; Exclude blobs whose area is too small. 
   MblobSelect BlobResult M_EXCLUDE M_AREA M_LESS_OR_EQUAL MIN_BLOB_AREA M_NULL

  ; Get the total number of blobs. 
   MblobGetNumber BlobResult &TotalBlobs
   echoln "There are "
   echoln [nbrtostr TotalBlobs 10]
   echo   " objects in the image."
   pause "Press <Enter> to continue."

  ; Reverse what is considered to be the background so that
  ; holes are seen as being blobs. 
   
   MblobControl BlobResult M_FOREGROUND_VALUE M_ZERO

  ; Add a feature to distinguish between types of holes. Since area
  ; has already been added to the feature list, and the processing 
  ; mode has been changed, all blobs will be re-included and the area 
  ; of holes will be calculated automatically.
   
   MblobSelectFeature FeatureList M_COMPACTNESS

  ; Calculate selected features for each blob. 
   MblobCalculate BinImage M_NULL FeatureList BlobResult

  ; Exclude small holes and large (the area around objects) holes. 
   MblobSelect BlobResult M_EXCLUDE M_AREA M_OUT_RANGE MIN_BLOB_AREA MAX_BLOB_AREA

  ; Get the number of blobs with holes. 
   MblobGetNumber BlobResult &BlobsWithHoles

  ; Exclude blobs whose holes are compact (i.e. nuts). 
   MblobSelect BlobResult M_EXCLUDE M_COMPACTNESS M_LESS_OR_EQUAL MIN_COMPACTNESS M_NULL

  ; Get the number of blobs with holes which are NOT compact. 
   MblobGetNumber BlobResult &BlobsWithRoughHoles

  ; Print results. 
     echo "There are:"
   echoln "           "
   echoln [nbrtostr (TotalBlobs-BlobsWithHoles) 10]
   echo   " bolts"
   echoln "           "
   echoln [nbrtostr (BlobsWithHoles - BlobsWithRoughHoles) 10]
   echo   " nuts"
   echoln "           "
   echoln [nbrtostr BlobsWithRoughHoles 10]
   echo   " washers"
   echo
   pause"Press <Enter> to end."


  ; Free all allocations. 
   MblobFree BlobResult 
   MblobFree FeatureList 
   MbufFree BinImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0
