; File name: mblobcog.cl
; Synopsis:  This program counts the number of objects in an image, 
;            and marks the center of gravity of each.  


; Disable the automatic printing of the return values of commands
silence 1

 
; Target MIL image file specifications.   
macro IMAGE_FILE            "\bolts.mim" 
macro IMAGE_WIDTH           512
macro IMAGE_HEIGHT          480
macro IMAGE_THRESHOLD_VALUE  24 

; Maximum number of blobs and minimum area of blobs.  
macro MAX_BLOBS              10 
macro MIN_BLOB_AREA          50.0

; Radius of the smallest particles to keep. 
macro SMALL_PARTICLE_RADIUS   3

; Size of the cross-hair used to mark centers of gravity. 
macro CSIZE 10
 
  MIL_ID MilApplication         ; Application identifier.  
  MIL_ID MilSystem              ; System identifier. 
  MIL_ID MilDisplay             ; Display identifier.      
  MIL_ID MilImage               ; Image buffer identifier.
  MIL_ID BinImage               ; Binary image buffer identifier. 
  MIL_ID BlobResult             ; Blob result buffer identifier. 
  MIL_ID FeatureList            ; Feature list identifier. 
  
  long   TotalBlobs             ; Total number of blobs. 
  allocarray CogX (4*MAX_BLOBS) ; X coordinate of center of gravity. 
  allocarray CogY (4*MAX_BLOBS) ; Y coordinate of center of gravity.
 
  ; Allocate defaults 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage
   
  ; Allocate a binary image buffer for fast processing.
   MbufAlloc2d M_DEFAULT IMAGE_WIDTH IMAGE_HEIGHT (1+M_UNSIGNED) (M_IMAGE+M_PROC) &BinImage

  ; Load source image into image buffer.  
   setstr str_image [ addstr imagepath IMAGE_FILE ]
   MbufLoad [getstr str_image] MilImage 
  
  ; Pause to show the original image.  
  echo "This program determines the number of objects in the"
  echo "displayed image and marks the center of gravity of each."
  pause "Press <Enter> to continue."	

  ; Binarize image. 
   MimBinarize MilImage BinImage M_GREATER_OR_EQUAL IMAGE_THRESHOLD_VALUE M_NULL
 
  ; Remove small particles and then remove small holes. 
  MimOpen BinImage BinImage SMALL_PARTICLE_RADIUS M_BINARY
  MimClose BinImage BinImage SMALL_PARTICLE_RADIUS M_BINARY
 
  ; Allocate a feature list.  
   MblobAllocFeatureList MilSystem &FeatureList 
  
  ; Enable feature to select blobs of interest.  
   MblobSelectFeature FeatureList M_AREA 
 
  ; Allocate a blob result buffer. 
   MblobAllocResult MilSystem &BlobResult 
 
  ; Calculate selected features for each blob.  
   MblobCalculate BinImage M_NULL FeatureList BlobResult 
 
  ; Exclude blobs whose area is too small.  
   MblobSelect BlobResult M_EXCLUDE M_AREA M_LESS_OR_EQUAL MIN_BLOB_AREA M_NULL 
 
  ; Get the total number of selected blobs.  
   MblobGetNumber BlobResult &TotalBlobs 
  echoln "There are "
  echoln [nbrtostr TotalBlobs 10]
  echo   " objects in the image."
  pause "Press <Enter> to continue."
  
  ; Select center of gravity feature.  
   MblobSelectFeature FeatureList M_CENTER_OF_GRAVITY_X 
   MblobSelectFeature FeatureList M_CENTER_OF_GRAVITY_Y 
 
  ; Calculate selected features for each blob.  
   MblobCalculate BinImage M_NULL FeatureList BlobResult 
 
  ; Check for array overflow.  
   if(TotalBlobs > MAX_BLOBS) 
    {
     echo "Error: too many blobs."
    }
   else
    {

    ; Get the results.  
     MblobGetResult BlobResult (M_CENTER_OF_GRAVITY_X+M_TYPE_LONG) [getarray CogX] 
     MblobGetResult BlobResult (M_CENTER_OF_GRAVITY_Y+M_TYPE_LONG) [getarray CogY] 
    
    ; Draw gray cross-hairs at the center of gravity of each blob.  
     MgraColor M_DEFAULT 250 


     for n 0 TotalBlobs 1
     {
     setnbr cog_x_ ([getnbrarray CogX _n]-CSIZE)
     setnbr cog_y [getnbrarray CogY _n]
     setnbr cog_x_cs ([getnbrarray CogX _n]+CSIZE)
     setnbr cog_x [getnbrarray CogX _n]
     setnbr cog_y_ ([getnbrarray CogY _n]-CSIZE)
     setnbr cog_y_cs ([getnbrarray CogY _n]+CSIZE)

     MgraLine M_DEFAULT MilImage [getnbr cog_x_] [getnbr cog_y] [getnbr cog_x_cs] [getnbr cog_y]
     MgraLine M_DEFAULT MilImage [getnbr cog_x] [getnbr cog_y_] [getnbr cog_x] [getnbr cog_y_cs]
     }  
    
      echo
      echo "The calculated centers of gravity have been marked."
      pause "Press <Enter> to end."

    }
   ; Free all allocations. 
   MblobFree BlobResult 
   MblobFree FeatureList 
   MbufFree BinImage
   MappFreeDefault MilApplication  MilSystem MilDisplay M_NULL MilImage
 
; Enable the automatic printing of the return values of commands
silence 0
