;File name: mconvol.cl
;Synopsis:  This program loads an image of some tissue and then
;           does a 3x3 custom convolution (smoothing) on it.
;


; Disable the automatic printing of the return values of commands
silence 1

;Target MIL image file specifications.
macro IMAGE_FILE             "\wafer.mim"
macro IMAGE_WIDTH            512
macro IMAGE_HEIGHT           480

;Kernel informations.
macro KERNEL_WIDTH     3
macro KERNEL_HEIGHT    3
macro KERNEL_DEPTH     8

;Average kernel information data definition.
allocarray KernelData (3 * 3)
setchrarray KernelData 0 1
setchrarray KernelData 1 2
setchrarray KernelData 2 1
setchrarray KernelData 3 2
setchrarray KernelData 4 4
setchrarray KernelData 5 2
setchrarray KernelData 6 1
setchrarray KernelData 7 2
setchrarray KernelData 8 1

MIL_ID MilApplication ;Application identifier.     
MIL_ID MilSystem      ;System identifier.          
MIL_ID MilDisplay     ;Display identifier.         
MIL_ID MilImage       ;Image buffer identifier.    
MIL_ID MilSubImage    ;Sub-image buffer identifier.
MIL_ID MilKernel      ;Custom kernel identifier.   

;Allocate defaults.
MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

;Restrict the region to be processed to the image size.
MbufChild2d MilImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage

;Load source image into an image buffer.
setstr str_image [ addstr imagepath IMAGE_FILE ]
MbufLoad [getstr str_image] MilSubImage

;Pause to show the original image.
echo "This program does a convolution on the displayed image."
echo "It uses a custom smoothing kernel."
pause


;Allocate a MIL kernel.
MbufAlloc2d M_DEFAULT KERNEL_HEIGHT KERNEL_WIDTH (KERNEL_DEPTH + M_UNSIGNED) M_KERNEL &MilKernel

;Put the custom data in it.
MbufPut MilKernel [getarray KernelData]

;Set a normalization (divide) factor to have a kernel with
;    a sum sum equal to one.
MbufControlNeighborhood MilKernel M_NORMALIZATION_FACTOR 16

;Convolve the image using the kernel.
MimConvolve MilSubImage MilSubImage MilKernel

;Pause to show the result.
echo "The original image was smoothed using a custom kernel."
pause

;Free all allocations.
freearray KernelData
MbufFree MilKernel
MbufFree MilSubImage
MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0
