; File name: mcount.c
; Synopsis:  This program loads an image of some tissue and determines the
;            number of cell nuclei which are larger than a certain size.
 

; Disable the automatic printing of the return values of commands
silence 1

; Target MIL image file specifications. 
macro IMAGE_FILE            "\cell.mim" 
macro IMAGE_WIDTH            512
macro IMAGE_HEIGHT           480
macro IMAGE_THRESHOLD_VALUE  128

; Small particle radius (in pixels). 
macro SMALL_PARTICLE_RADIUS    2

  MIL_ID MilApplication  ; Application identifier.  
  MIL_ID MilSystem       ; System identifier.       
  MIL_ID MilDisplay      ; Display identifier.      
  MIL_ID MilImage        ; Image buffer identifier. 
  MIL_ID MilSubImage     ; Sub-image buffer identifier. 
  MIL_ID ExtrResult      ; Extreme result buffer identifier. 
  long   MaxLabelNumber  ; Highest label value. 

  ; Allocate defaults. 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

  ; Restrict the region to be processed to the image size. 
   MbufChild2d MilImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage

  ; Load source image into an image buffer. 
   setstr str_image [ addstr imagepath IMAGE_FILE ]
   MbufLoad [getstr str_image] MilSubImage

  ; Pause to show the original image. 
   echo "This program counts the number of large"
   echo "particles in the displayed image."
   pause "Press <Enter> to continue."

  ; Smooth the image to remove noise. 
   MimConvolve MilSubImage MilSubImage M_SMOOTH

  ; Binarize the image so that particles are represented
  ; in white and the background in black. 
   
   MimBinarize MilSubImage MilSubImage M_LESS_OR_EQUAL IMAGE_THRESHOLD_VALUE M_NULL

  ; Remove small particles. 
   MimOpen MilSubImage MilSubImage SMALL_PARTICLE_RADIUS M_BINARY

  ; Pause to show the remaining particle(s). 
   echo "These particles have been extracted from the original image."
   pause "Press <Enter> to continue."

  ; Label image in place. 
   MimLabel MilSubImage MilSubImage M_DEFAULT

  ; The largest label value corresponds to the extreme value of the image. 
   MimAllocResult M_DEFAULT 1 M_EXTREME_LIST &ExtrResult
   MimFindExtreme MilSubImage ExtrResult M_MAX_VALUE
   MimGetResult ExtrResult M_VALUE &MaxLabelNumber
   if (MaxLabelNumber)
      {
      MimArith MilSubImage (255/MaxLabelNumber) MilSubImage M_MULT_CONST
      }

  ; Print results. 
   echoln "There were "
   echoln [nbrtostr MaxLabelNumber 10]
   echo   " large particle(s) in the original image."
   pause "Press <Enter> to end."


   ; Free all allocations. 
   MimFree ExtrResult
   MbufFree MilSubImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0


