; File name: mdisplay.cl
; Synopsis:  This program allocates a displayable image buffer,
;            clears its contents, draws a filled circle and then
;            displays it. It also checks whether the allocation
;            was successful using the MIL error reporting 
;            mechanism.
;


; Disable the automatic printing of the return values of commands
silence 1

macro IMAGE_DEPTH 8

MIL_ID MilApplication ; Application identifier.
MIL_ID MilSystem      ; System identifier.  
MIL_ID MilDisplay     ; Display identifier.
MIL_ID MilImage       ; Image buffer identifier.  
long   ErrorCode      ; Error code value. 

 ; Allocate defaults. */
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL M_NULL

 ; Find the best size for the display image depending on the display type. 
   setnbr DispMode [MdispInquire MilDisplay M_DISP_MODE M_NULL]
   if ([getnbr DispMode] == M_WINDOWED )
      {
      ; Smallest possible. */
      setnbr sizex M_D_SIZ_X
      setnbr sizey M_D_SIZ_Y
      }
   else
      {
      ; The size of the entire display to avoid possible display artifact. 
      setnbr sizex [MdispInquire MilDisplay M_SIZE_X M_NULL]  
      setnbr sizey [MdispInquire MilDisplay M_SIZE_Y M_NULL]
      }

 ; Allocate a two-dimensional image buffer with the same dimensions
 ; as the displayable screen to perform graphics in it. 

   MbufAlloc2d MilSystem [getnbr sizex] [getnbr sizey] (IMAGE_DEPTH+M_UNSIGNED) (M_IMAGE+M_DISP) &MilImage
  
 ; Check the error status code set by the allocation command. If there
 ; was no error, draw and display a circle, otherwise print an error
 ; message and exit.

   MappGetError M_CURRENT &ErrorCode
   if (ErrorCode == M_NULL)
   {
  
     ; Clear buffer and draw a circle. 
      MbufClear MilImage 0
      MgraColor M_DEFAULT 255
      MgraArcFill M_DEFAULT MilImage 256 240 100 100 0.0 360.0
  
     ; Display the image buffer. 
      MdispSelect MilDisplay MilImage
      
     ; Enable keying on display if keying supported.
      if (M_DEF_DISPLAY_KEY_ENABLE_ON_ALLOC != 0)
         {
         if ([MdispInquire MilDisplay M_DISP_KEY_SUPPORTED M_NULL])
            {
            MdispOverlayKey MilDisplay M_KEY_ON_COLOR M_EQUAL 255 M_DEF_DISPLAY_KEY_COLOR
            }
         }

     ; Print a message.
      echo "A circle was drawn in the displayed image buffer."
      pause "Press <Enter> to end."
  
     ; Disable keying on display if keying supported. */
      if (M_DEF_DISPLAY_KEY_DISABLE_ON_FREE != 0)
         {
         if ([MdispInquire MilDisplay M_DISP_KEY_SUPPORTED 0])
            {
            MdispOverlayKey MilDisplay M_KEY_OFF M_NULL M_NULL M_NULL
            }
         }

     ; Release image buffer.
      MbufFree MilImage
   }
   else
   {
 
     ; Print an error message.
      echo "Error: Image buffer allocation failed."
   }
  
  ; Release defaults.
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL M_NULL

; Enable the automatic printing of the return values of commands
silence 0


