; File name: mhist.c 
; Synopsis:  This program loads an image of some tissue
;            and generates its histogram.


; Disable the automatic printing of the return values of commands
silence 1

; Target MIL image file specifications. 
macro IMAGE_FILE       "\cell.mim" 
macro IMAGE_WIDTH      512
macro IMAGE_HEIGHT     480

; Number of possible pixel intensities. 
macro NUM_INTENSITIES  256

  MIL_ID MilApplication             ; Application identifier   
  MIL_ID MilSystem                  ; System identifier.       
  MIL_ID MilDisplay                 ; Display identifier.      
  MIL_ID MilImage                   ; Image buffer identifier. 
  MIL_ID MilSubImage                ; Sub-image buffer identifier. 
  MIL_ID HistResult                 ; Histogram buffer identifier. 
  allocarray HistVals (NUM_INTENSITIES * 4) ; Histogram values. 
 
  ; Allocate the default system and image buffer. 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage
 
  ; Restrict the region to be processed to the image size. 
   MbufChild2d MilImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage
 
  ; Load source image into an image buffer. 
   setstr str_image [ addstr imagepath IMAGE_FILE ]
   MbufLoad [getstr str_image] MilSubImage
 
  ; Allocate a histogram result buffer. 
   MimAllocResult M_DEFAULT NUM_INTENSITIES M_HIST_LIST &HistResult
 
  ; Perform the histogram. 
   MimHistogram MilSubImage HistResult
 
  ; Get and print results. 
   MimGetResult HistResult M_VALUE [getarray HistVals]

   pause "Press <Enter> to print the histogram for the displayed image."

   for i 0 NUM_INTENSITIES 1
     {
     echoln "Intensity "
     echoln [nbrtostr _i 10]
     echoln " -- Number of pixels "
     echo [nbrtostr [getnbrarray HistVals _i] 10]

     if ( _i % 20 == 19)
        {
        pause "Press <Enter> to continue."
        }
     }
 
   echo "Done."

  ; Free all allocations. 
   MimFree HistResult
   MbufFree MilSubImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage


; Enable the automatic printing of the return values of commands
silence 0
