;File name: mmeasstr.c
;Synopsis:  This program measure the position, width and angle of
;           a stripe in an image, and marks its center and edges.
;           It also calculate the length and angle of a line going
;           from a reference point in the image to the center of
;           the located stripe.

; Disable the automatic printing of the return values of commands
silence 1

; Target image dimentions specifications.
macro IMAGE_W            512
macro IMAGE_H            480

; Measurement box specification
macro MEAS_W             (IMAGE_W / 2)
macro MEAS_H             101
macro MEAS_X             (IMAGE_W / 4)
macro MEAS_Y             ( (IMAGE_H / 2) - (MEAS_H / 2) ) 

; Target stripe specifications.
macro STRIPE_W           61
macro STRIPE_COLOR       128
macro STRIPE_ANGLE       2.0

; Reference point specification
macro REFERENCE_X        60
macro REFERENCE_Y        45

; Size and color of the cross to mark the positions.
macro CROSS_SIZE         10
macro CROSS_COLOR        240

MIL_ID MilApplication    ; Application identifier.       
MIL_ID MilSystem         ; System identifier.            
MIL_ID MilDisplay        ; Display identifier.           
MIL_ID MilImage          ; Image buffer identifier.      
MIL_ID MilSubImage       ; Sub-image buffer identifier.  
MIL_ID StripeMarker      ; Stripe marker identifier.     
MIL_ID PointMarker       ; Point marker identifier.      
MIL_ID CalcResult        ; Result buffer identifier.     
double StripeCenterX     ; Stripe X center position.     
double StripeCenterY     ; Stripe Y center position.     
double StripeW           ; Stripe width.                 
double StripeAngle       ; Stripe angle.                 
double StripeFirstEdgeX  ; Stripe left edge X position.  
double StripeFirstEdgeY  ; Stripe left edge Y position.  
double StripeSecondEdgeX ; Stripe right edge X position. 
double StripeSecondEdgeY ; Stripe right edge Y position. 
double ReferenceDistance ; Reference to stripe distance. 
double ReferenceAngle    ; Reference to stripe angle.    
double temp1             ;
double temp2             ;


; Allocate defaults
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

; Restrict the region to be processed to the image size.
   MbufChild2d MilImage 0 0 IMAGE_W IMAGE_H &MilSubImage

; Simulate the digitized image of a rotated vertical stripe
; in the center of the image.

   MgraColor M_DEFAULT STRIPE_COLOR
   MgraRectFill M_DEFAULT MilImage (MEAS_X+((MEAS_W/2)-(STRIPE_W/2)+0.5)) (MEAS_Y-((MEAS_H/2)+0.5)) (MEAS_X+((MEAS_W/2)+(STRIPE_W/2+0.5))-1) (MEAS_Y+(((MEAS_H/2)+0.5)*3))
   MimRotate MilSubImage MilSubImage STRIPE_ANGLE M_DEFAULT M_DEFAULT M_DEFAULT M_DEFAULT M_BILINEAR
   MimConvolve MilSubImage MilSubImage M_SMOOTH

    
; Pause to show the original image.
   echo "This program determines the position width and angle of the"
   echo "stripe in the displayed image and marks its center and edges."
   echo "It also determine the length and angle of a line going from"
   echo "a reference point in the image to the center of the stripe."
   pause "Press <Enter> to continue." 

; Allocate a stripe marker
   MmeasAllocMarker M_DEFAULT M_STRIPE M_DEFAULT &StripeMarker

; Specify the search box in the image where to find the stripe.
   MmeasSetMarker StripeMarker M_BOX_ORIGIN MEAS_X MEAS_Y
   MmeasSetMarker StripeMarker M_BOX_SIZE MEAS_W MEAS_H

; Specify the stripe approximative orientation and contrast
   MmeasSetMarker StripeMarker M_ORIENTATION M_VERTICAL M_NULL
   MmeasSetMarker StripeMarker M_CONTRAST STRIPE_COLOR M_NULL

; Find the stripe and measure its width and angle.
   MmeasFindMarker M_DEFAULT MilSubImage StripeMarker (M_POSITION + M_WIDTH + M_ANGLE)

; Get the stripe position width and angle.
   MmeasGetResult StripeMarker M_POSITION &StripeCenterX &StripeCenterY
   MmeasGetResult StripeMarker (M_POSITION + M_EDGE_FIRST) &StripeFirstEdgeX &StripeFirstEdgeY
   MmeasGetResult StripeMarker (M_POSITION + M_EDGE_SECOND) &StripeSecondEdgeX &StripeSecondEdgeY
   MmeasGetResult StripeMarker M_WIDTH    &StripeW M_NULL
   MmeasGetResult StripeMarker M_ANGLE    &StripeAngle M_NULL

; Draw a cross on the center of the stripe.
   MgraColor M_DEFAULT CROSS_COLOR
   MgraLine M_DEFAULT MilSubImage ((StripeCenterX+.5)-(CROSS_SIZE/2)) (StripeCenterY+.5) ((StripeCenterX+.5)+(CROSS_SIZE/2)) (StripeCenterY+.5)
   MgraLine M_DEFAULT MilSubImage (StripeCenterX+.5) ((StripeCenterY+.5)-(CROSS_SIZE/2)) (StripeCenterX+.5) ((StripeCenterY+.5)+(CROSS_SIZE/2))

; Draw a cross on the left edge of the stripe.
   MgraLine M_DEFAULT MilSubImage ((StripeFirstEdgeX+.5)-(CROSS_SIZE/2)) (StripeFirstEdgeY+.5) ((StripeFirstEdgeX+.5)+(CROSS_SIZE/2)) (StripeFirstEdgeY+.5)
   MgraLine M_DEFAULT MilSubImage (StripeFirstEdgeX+.5) ((StripeFirstEdgeY+.5)-(CROSS_SIZE/2)) (StripeFirstEdgeX+.5) ((StripeFirstEdgeY+.5)+(CROSS_SIZE/2))

; Draw a cross on the right edge of the stripe.
   MgraLine M_DEFAULT MilSubImage ((StripeSecondEdgeX+.5)-(CROSS_SIZE/2)) (StripeSecondEdgeY+.5) ((StripeSecondEdgeX+.5)+(CROSS_SIZE/2)) (StripeSecondEdgeY+.5)
   MgraLine M_DEFAULT MilSubImage (StripeSecondEdgeX+.5) ((StripeSecondEdgeY+.5)-(CROSS_SIZE/2)) (StripeSecondEdgeX+.5) ((StripeSecondEdgeY+.5)+(CROSS_SIZE/2))

; Print the result.
   echoln "The stripe in the image is at position: "
   echoln [dbltostr StripeCenterX 4]
   echoln " " 
   echoln [dbltostr StripeCenterY 4]
   echo   " and"
   echoln "is "
   echoln [dbltostr StripeW 4]
   echoln " pixels wide with an angle of "
   echoln [dbltostr StripeAngle 4]
   echo   " degrees."
   echo   "Its center and edges have been marked."
   pause  "Press <Enter> to continue."

; Allocate a point marker
   MmeasAllocMarker M_DEFAULT M_POINT M_DEFAULT &PointMarker

; Specify the reference point position in the image.
   MmeasSetMarker PointMarker M_POSITION REFERENCE_X REFERENCE_Y

; Allocate a calculation result buffer
   MmeasAllocResult M_DEFAULT M_CALCULATE &CalcResult

; Calculate the distance and angle between the point and the stripe.
   MmeasCalculate M_DEFAULT PointMarker StripeMarker CalcResult (M_DISTANCE + M_ANGLE)

; Get the distance and angle.
   MmeasGetResult CalcResult M_DISTANCE &ReferenceDistance M_NULL
   MmeasGetResult CalcResult M_ANGLE &ReferenceAngle M_NULL

; Draw a cross on the reference point and a line from that point
; to the center of the located stripe.

   MgraLine M_DEFAULT MilSubImage (REFERENCE_X-(CROSS_SIZE/2)) REFERENCE_Y (REFERENCE_X+(CROSS_SIZE/2)) REFERENCE_Y
   MgraLine M_DEFAULT MilSubImage REFERENCE_X (REFERENCE_Y-(CROSS_SIZE/2)) REFERENCE_X (REFERENCE_Y+(CROSS_SIZE/2))
   MgraLine M_DEFAULT MilSubImage REFERENCE_X REFERENCE_Y (StripeCenterX+0.5) (StripeCenterY+0.5)

; Print the result.
   echo   "The distance and angle from the drawn reference point"
   echoln "are "
   echoln [dbltostr ReferenceDistance 4]
   echoln " pixels and "
   echoln [dbltostr ReferenceAngle 4]
   echo   " degrees." 
   pause  "Press <Enter> to end."

; Free all allocations.
   MmeasFree CalcResult
   MmeasFree PointMarker
   MmeasFree StripeMarker
   MbufFree MilSubImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0
