; File name: mmultdis.cl
; Synopsis:  This program shows how to display more than one 
;            image buffer at a time. It allocates a displayable 
;            image buffer, allocates two child buffers from it, 
;            and then uses the child buffers as the source and
;            destination of a binarizing operation.
;
;            The display will be zoomed if the system's display
;            supports it.
 


; Disable the automatic printing of the return values of commands
silence 1

; MIL image file name. 
macro IMAGE_FILE            "\cell.mim" 

; MIL image file specifications. 
macro IMAGE_WIDTH            128
macro IMAGE_HEIGHT           240
macro IMAGE_TYPE              (8+M_UNSIGNED)
macro IMAGE_THRESHOLD_VALUE  128
macro ZOOM_VALUE               2


 MIL_ID MilApplication  ; Application identifier.  
 MIL_ID MilSystem       ; System identifier.        
 MIL_ID MilDisplay      ; Display identifier.       
 MIL_ID MilParentImage  ; Image buffer identifier.  
 MIL_ID MilSrcSubImage  ; Source image buffer identifier.       
 MIL_ID MilDstSubImage  ; Destination image buffer identifier.  

 ; Allocate the defaults. 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL M_NULL

 ; Allocate a display image buffer.
  MbufAlloc2d MilSystem (IMAGE_WIDTH*2) IMAGE_HEIGHT IMAGE_TYPE (M_IMAGE+M_DISP+M_PROC) &MilParentImage 
 
 ; Allocate two child buffers from the displayable parent buffer. 
  MbufChild2d MilParentImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSrcSubImage 
  MbufChild2d MilParentImage IMAGE_WIDTH 0 IMAGE_WIDTH IMAGE_HEIGHT &MilDstSubImage
      
 ; Clear the parent buffer.  
  MbufClear MilParentImage 0

 ; Display the parent buffer. 
  MdispSelect MilDisplay MilParentImage

 ; Load the entire source image into the source sub-image buffer.  
  setstr str_image [ addstr imagepath IMAGE_FILE ]
  MbufLoad [getstr str_image] MilSrcSubImage

; Binarize the source sub-image into the destination sub-image.
  MimBinarize MilSrcSubImage MilDstSubImage M_GREATER_OR_EQUAL IMAGE_THRESHOLD_VALUE M_NULL 
 
 ; Report on the Host screen what is being displayed. 
  echo "A binarizing operation was performed on the sub-image on the"
  echo "left side of the screen, and the result is being displayed in"
  echo "the sub-image on the right side of the screen."
  pause "Press <Enter> to continue."

 ; Report on the Host screen what is being displayed. 
  echo
  echoln "Display zoomed by "
  echoln [nbrtostr ZOOM_VALUE 10]
  echo   " in X and Y (if supported)."

 ; Zoom both sub-images by zooming the display. 
  MdispZoom MilDisplay ZOOM_VALUE ZOOM_VALUE

 ; Wait for a key 
  pause "Press a <Enter> to end."
      
 ; Return the display to normal.
  MdispZoom MilDisplay 1 1
   
 ; Close the display.
  MdispDeselect MilDisplay MilParentImage

 ; Free all allocations. 
  MbufFree MilDstSubImage
  MbufFree MilSrcSubImage
  MbufFree MilParentImage
  MappFreeDefault MilApplication MilSystem MilDisplay M_NULL M_NULL

; Enable the automatic printing of the return values of commands
silence 0


