;File name: mocrread.c
;Synopsis:  This program calibrate an OCR font (semi-font) and use it
;           to read the string present in the image. The string read
;           is then printed to the screen and the calibrated font
;           saved to disk.

; Disable the automatic printing of the return values of commands
silence 1

; Target image character specifications.
macro IMAGE_FILE   "\ocrsemi1.mim"
macro X_MIN        22.0
macro X_MAX        23.0
macro X_STEP       0.50
macro Y_MIN        43.0
macro Y_MAX        44.0
macro Y_STEP       0.50

; Target reading specifications.
macro POS_X        30  
macro POS_Y        40
macro WIDTH        420
macro HEIGHT       70
macro SCORE_MIN    50.0

; Font file names.
macro FONT_FILE_NAME   "\semi1292.mfo"
macro FONT_FILE_OUT    "\semicali.mfo"

; Length of the string to read (null terminated)
macro STRING_LENGTH    13

MIL_ID MilApplication          ; Application identifier. 
MIL_ID MilSystem               ; System identifier.      
MIL_ID MilDisplay              ; Display identifier.     
MIL_ID MilImage                ; Image buffer identifier.
MIL_ID MilSubImage             ; Sub-image buffer identifier. 
MIL_ID OcrFont                 ; OCR font identifier.         
MIL_ID OcrResult               ; OCR result buffer identifier.
setstr String "M940902-MXD5"   ; Array of characters to read.
double Score                   ; Reading score.              

; Allocate defaults
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

; Set the foreground color for graphics to white
   MgraColor M_DEFAULT 255

; Load source image into image buffer.
   setstr str_image [ addstr imagepath IMAGE_FILE ]
   MbufLoad [getstr str_image] MilImage

; Restrict the region of the image where to read the string.
   MbufChild2d MilImage POS_X POS_Y WIDTH HEIGHT &MilSubImage

; Restore the OCR character font from disk.
   setstr str_image [ addstr imagepath FONT_FILE_NAME ]
   MocrRestoreFont [ getstr str_image ] M_RESTORE MilSystem &OcrFont

   echo "The OCR font will be calibrated using the displayed image."
   echo "Calibrating font..."

; Calibrate the OCR font.
   MocrCalibrateFont MilSubImage OcrFont [getstr String] X_MIN X_MAX X_STEP Y_MIN Y_MAX Y_STEP M_DEFAULT

; Set the user specific character constraints for each string position.
   MocrSetConstraint OcrFont 0  M_LETTER  "ABCDEFGHIJKLMNOPQRSTUVWXYZ" ; A to Z only  
   MocrSetConstraint OcrFont 1  M_DIGIT   "9"                          ; 9      only  
   MocrSetConstraint OcrFont 2  M_DIGIT   "0123456789"                 ; 0 to 9 only  
   MocrSetConstraint OcrFont 3  M_DIGIT   "0123456789"     ; 0 to 9 only  
   MocrSetConstraint OcrFont 4  M_DIGIT   "0123456789"     ; 0 to 9 only  
   MocrSetConstraint OcrFont 5  M_DIGIT   "0123456789"     ; 0 to 9 only  
   MocrSetConstraint OcrFont 6  M_DIGIT   "0123456789"     ; 0 to 9 only  
   MocrSetConstraint OcrFont 7  M_DEFAULT "-"        ; -      only  
   MocrSetConstraint OcrFont 8  M_LETTER  "M"        ; M      only  
   MocrSetConstraint OcrFont 9  M_LETTER  "X"        ; X      only  
   MocrSetConstraint OcrFont 10 M_LETTER  "ABCDEFGH" ; SEMI checksum
   MocrSetConstraint OcrFont 11 M_DIGIT   "01234567" ; SEMI checksum

; Pause to signal the following read operation.
   echo "The string present in the displayed image will be read and"
   echo "the result will be printed."
   pause "Press <Enter> to continue."

; Allocate an OCR result buffer.
   MocrAllocResult MilSystem M_DEFAULT &OcrResult

; Read the string.
   MocrReadString MilSubImage OcrFont OcrResult

; Get the string and its reading score.
   MocrGetResult OcrResult M_STRING [getstrptr String]
   MocrGetResult OcrResult M_SCORE  &Score

; Print the result.
   echo  "The string read is:"
   echo [getstr String]
   echoln  "Score: "
   echoln  [dbltostr Score 4]
   echo    " %"

; Draw the string under the reading region.
   MgraFont M_DEFAULT M_FONT_DEFAULT_LARGE
   MgraText M_DEFAULT MilImage (POS_X + (WIDTH / 4)) (POS_Y + HEIGHT) [getstr String]

; Save the calibrated font if the reading score was sufficent.
   if ( Score > SCORE_MIN)
      {
	  setstr str_image [ addstr imagepath FONT_FILE_OUT ]
      MocrSaveFont [ getstr str_image ] M_SAVE OcrFont
      echo "Read sucessful calibrated OCR font was saved to disk."
      }
   else 
      {
      echo "Error: Read score too low calibrated OCR font not saved."
      }
   pause "Press <Enter> to end."

; Free all allocations.
   MocrFree OcrFont
   MocrFree OcrResult
   MbufFree MilSubImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0
