;File name: mopen.cl
;Synopsis: This program loads an image of some tissue and then
;                   do opening on it using two methods.


; Disable the automatic printing of the return values of commands
silence 1

;Target MIL image file specifications.
macro IMAGE_FILE                "\cell.mim"
macro IMAGE_WIDTH               240
macro IMAGE_HEIGHT              240
macro IMAGE_DEPTH               8
macro IMAGE_THRESHOLD_VALUE     128

;Structuring element information.
macro STRUCT_ELEM_WIDTH         5
macro STRUCT_ELEM_HEIGHT        5
macro STRUCT_ELEM_DEPTH         32

;Small particle radius (in pixels).
macro SMALL_PARTICLE_RADIUS     2


MIL_ID MilApplication ;Application identifier.
MIL_ID MilSystem      ;System identifier. 
MIL_ID MilDisplay     ;Display identifier.   
MIL_ID MilImage       ;Image buffer identifier.
MIL_ID BinImage       ;Binary Image buffer identifier.
MIL_ID MilSubImage0   ;Sub-image buffer identifier for original image.
MIL_ID MilSubImage1   ;Sub-image buffer identifier for binarization
MIL_ID MilSubImage2   ;Sub-image buffer identifier for common open.
MIL_ID MilSubImage3   ;Sub-image buffer identifier for customized Open.
MIL_ID StructElem     ;Structing element buffer.


allocarray StructArray (5 * 5 * 4)
setnbrarray StructArray 0  M_DONT_CARE
setnbrarray StructArray 1  M_DONT_CARE
setnbrarray StructArray 2  1
setnbrarray StructArray 3  M_DONT_CARE
setnbrarray StructArray 4  M_DONT_CARE
setnbrarray StructArray 5  M_DONT_CARE
setnbrarray StructArray 6  1
setnbrarray StructArray 7  1
setnbrarray StructArray 8  1
setnbrarray StructArray 9  M_DONT_CARE
setnbrarray StructArray 10 1
setnbrarray StructArray 11 1
setnbrarray StructArray 12 1
setnbrarray StructArray 13 1
setnbrarray StructArray 14 1
setnbrarray StructArray 15 M_DONT_CARE
setnbrarray StructArray 16 1
setnbrarray StructArray 17 1
setnbrarray StructArray 18 1
setnbrarray StructArray 19 M_DONT_CARE
setnbrarray StructArray 20 M_DONT_CARE
setnbrarray StructArray 21 M_DONT_CARE
setnbrarray StructArray 22 1
setnbrarray StructArray 23 M_DONT_CARE
setnbrarray StructArray 24 M_DONT_CARE


;Allocate defaults.
MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

; Allocate a binary image buffer for fast processing.
MbufAlloc2d M_DEFAULT IMAGE_WIDTH IMAGE_HEIGHT (1+M_UNSIGNED) (M_IMAGE+M_PROC) &BinImage

;Define 4 processing buffers in the display buffer, restricting
;the regions to be processed to the image size.

MbufChild2d MilImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage0
MbufChild2d MilImage IMAGE_WIDTH 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage1
MbufChild2d MilImage 0 IMAGE_HEIGHT IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage2
MbufChild2d MilImage IMAGE_WIDTH IMAGE_HEIGHT IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage3

;Load source image into image buffer for original image.
setstr str_image [ addstr imagepath IMAGE_FILE ]
MbufLoad [getstr str_image] MilSubImage0

;Allocate a structuring element.
MbufAlloc2d M_DEFAULT STRUCT_ELEM_WIDTH STRUCT_ELEM_HEIGHT (STRUCT_ELEM_DEPTH+M_SIGNED) M_STRUCT_ELEMENT &StructElem

;Load buffer with data.
MbufPut2d StructElem 0 0 STRUCT_ELEM_WIDTH STRUCT_ELEM_HEIGHT [getarray StructArray]

;Pause to show the original image.
echo  "This program does the opening of an image using two different"
echo  "structuring elements."
pause "Press <Enter> to continue."

;Smooth the image to remove noise.
MimConvolve MilSubImage0 MilSubImage0 M_SMOOTH

;Binarize the image so that particles are represented in white and
;the background in black placing result in subimage1 on the display.

MimBinarize MilSubImage0 MilSubImage1 M_LESS_OR_EQUAL IMAGE_THRESHOLD_VALUE M_NULL

;Copy the binarized image to a binary buffer for fast processing. 
MbufCopy MilSubImage1 BinImage

;Opening using common method, placing result in subimage2.
MimOpen BinImage MilSubImage2 SMALL_PARTICLE_RADIUS M_BINARY

;Opening (Erode and Dilate) using customized method,
;placing result in subimage3.
MimMorphic BinImage BinImage StructElem M_ERODE (SMALL_PARTICLE_RADIUS / 2) M_BINARY
MimMorphic BinImage MilSubImage3 StructElem M_DILATE (SMALL_PARTICLE_RADIUS / 2) M_BINARY

                                                                                                                            
;Pause to show the opened particle(s).
echo  "The top right image is the binarized source image. When opening it"
echo  "using the standard method, the bottom left image results, whereas"
echo  "when opening it using the customized method, the bottom right image"
echo  "results and best preserves the original shape of the objects."
pause "Press <Enter> to end."

;Free structuring element buffer.
MbufFree StructElem
freearray StructArray

;Free the allocated child buffers.
MbufFree MilSubImage0
MbufFree MilSubImage1
MbufFree MilSubImage2
MbufFree MilSubImage3
MbufFree BinImage

;Release defaults.
MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0
