; File name: mperim.c 
; Synopsis:  This program finds the exoskeletons of the
;            perimeters of dark objects in an image. 
 


; Disable the automatic printing of the return values of commands
silence 1

; Source MIL image file specifications.  
macro IMAGE_FILE             "\cell.mim" 
macro IMAGE_WIDTH            512
macro IMAGE_HEIGHT           480
macro IMAGE_DEPTH              8
macro IMAGE_THRESHOLD_VALUE  128

; Small particle radius (in pixels). 
macro SMALL_PARTICLE_RADIUS    2

  MIL_ID MilApplication  ; Application identifier.  
  MIL_ID MilSystem       ; System identifier.       
  MIL_ID MilDisplay      ; Display identifier.      
  MIL_ID MilImage        ; Image buffer identifier. 
  MIL_ID BinImage,       ; Binary image buffer identifier. 
  MIL_ID DilBinImage;    ; Dilated binary image buffer identifier. 

  ; Allocate defaults. 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

  ; Allocate 2 binary image buffers for fast processing.
  MbufAlloc2d M_DEFAULT IMAGE_WIDTH IMAGE_HEIGHT (1+M_UNSIGNED) (M_IMAGE+M_PROC) &BinImage
  MbufAlloc2d M_DEFAULT IMAGE_WIDTH IMAGE_HEIGHT (1+M_UNSIGNED) (M_IMAGE+M_PROC) &DilBinImage

  ; Load source image into an image buffer. 
   setstr str_image [ addstr imagepath IMAGE_FILE ]
   MbufLoad [getstr str_image] MilImage

  ; Pause to show the original image. 
   echo "This program finds the exoskeletons of"
   echo "the particles in the displayed image."
   pause "Press <Enter> to continue."

  ; Binarize the image.  
   MimBinarize MilImage BinImage M_LESS_OR_EQUAL IMAGE_THRESHOLD_VALUE M_NULL

  ; Remove small particles. 
   MimOpen BinImage BinImage SMALL_PARTICLE_RADIUS M_BINARY

  ; Dilate image (adds one pixel around all objects).  
   MimDilate BinImage DilBinImage 1 M_BINARY

  ; XOR the dilated image with the original image. 
   MimArith BinImage DilBinImage BinImage M_XOR

  ; Convert the binary image to a visible grayscale image (0-0xFF). 
  MimBinarize BinImage MilImage M_GREATER 0 M_NULL

   echo "Exoskeletons of the object's perimeters are being displayed."
   pause "Press <Enter> to end."

  ; Free all allocations. 
   MbufFree BinImage
   MbufFree DilBinImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0


