; File name: mrestmod.c 
; Synopsis:  This program restores a previously saved model and 
;            then copies the high speed internal version of the
;            model to the buffer being displayed. It also recalls
;            the model image and draws a box around the original
;            model position so that you can compare the original
;            image and the high speed version of the model. 
;


; Disable the automatic printing of the return values of commands
silence 1

; MIL image from which model was extracted, and its dimensions.  
macro IMAGE_FILE    "\board.mim" 
macro IMAGE_WIDTH   512
macro IMAGE_HEIGHT  480

; MIL model file created by msearch.c example. 
macro MODEL_FILE    "\chip.mmo"

  MIL_ID MilApplication      ; Application identifier.  
  MIL_ID MilSystem           ; System identifier. 
  MIL_ID MilDisplay          ; Display identifier.      
  MIL_ID MilImage            ; Image buffer identifier. 
  MIL_ID MilSubImage         ; Sub-image buffer identifier. 
  MIL_ID Model               ; Model identifier. 
  double Width               ; Model size. 
  double Height
  double XOrg                ; Model original top-left position. 
  double YOrg


  ; Allocate defaults. 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

  ; Restrict the region to be processed to the image size. 
   MbufChild2d MilImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage
 
  ; Load image from which we took the model. 
   setstr str_image [ addstr imagepath IMAGE_FILE ]
   MbufLoad [getstr str_image] MilSubImage

  ; Restore model. 
   setstr str_model [ addstr imagepath MODEL_FILE ]
   MpatRestore MilSystem [getstr str_model] &Model

  ; Read model's original top-left position and draw a box
  ; around the model in the model image. 
   
   MpatInquire Model M_ALLOC_SIZE_X &Width
   MpatInquire Model M_ALLOC_SIZE_Y &Height
   MpatInquire Model M_ALLOC_OFFSET_X &XOrg
   MpatInquire Model M_ALLOC_OFFSET_Y &YOrg
   MgraRect M_DEFAULT MilSubImage (XOrg - 1) (YOrg - 1) (XOrg + Width) (YOrg + Height)

  ; Copy the model to the buffer being displayed. 
   MpatCopy Model MilSubImage M_HIGH

  ; Wait for a key press before continuing.  
   echo "The model was copied into the top left corner of"
   echo "the image which it comes from."
   echo
   pause "Press <Enter> to end."

  ; Free all allocations. 
   MpatFree Model
   MbufFree MilSubImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0



