; File name: msearch.cl
; Synopsis:  This program defines a model and then searches for it
;            in a shifted version of the image. The model is saved
;            on disk for future use.
 


; Disable the automatic printing of the return values of commands
silence 1

; Source image file specifications.  
macro IMAGE_FILE    "\board.mim" 
macro IMAGE_WIDTH   512
macro IMAGE_HEIGHT  480


; Model position and size. 
macro MODEL_XPOS    304
macro MODEL_YPOS    126
macro MODEL_WIDTH    86
macro MODEL_HEIGHT  100

; Minimum match score to determine acceptability of model. 
macro MODEL_MIN_MATCH_SCORE 70.0

; Minimun accuracy for the search assuming MpatSetAccuracy(..,M_MEDIUM). 
macro MODEL_MIN_ACCURACY    0.25

; File in which to save model. 
macro MODEL_FILE    "\chip.mmo"

  MIL_ID MilApplication       ; Application identifier.  
  MIL_ID MilSystem            ; System identifier.       
  MIL_ID MilDisplay           ; Display identifier.      
  MIL_ID MilImage             ; Image buffer identifier. 
  MIL_ID MilSubImage          ; Sub-image buffer identifier. 
  MIL_ID Model                ; Model identifier.
  MIL_ID Result              ; Result identifier. 
  double XOrg           ; Model original positions.
  double YOrg
  double x              ; Model positions.  
  double y
  double Score          ; Model correlation score. 
  double SHIFT_X        ; Image shifting values. 
  setSHIFT_X       4.25
  double SHIFT_Y   
  setSHIFT_Y       7.25


 
  ; Allocate defaults. 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

  ; Set the foreground color for graphics to white
   MgraColor M_DEFAULT 255

  ; Restrict the region to be processed to the image size. 
   MbufChild2d MilImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage

  ; Load model image into an image buffer. 
   setstr str_image [ addstr imagepath IMAGE_FILE ]
   MbufLoad [getstr str_image] MilSubImage

  ; Allocate a normalized grayscale model. 
   MpatAllocModel MilSystem MilSubImage MODEL_XPOS MODEL_YPOS MODEL_WIDTH MODEL_HEIGHT M_NORMALIZED &Model

  ; Set the search accuracy to medium. 
   MpatSetAccuracy Model M_MEDIUM

  ; Set the search model speed to M_HIGH. 
   MpatSetSpeed Model M_HIGH

  ; Preprocess the model. 
   MpatPreprocModel MilSubImage Model M_DEFAULT

  ; Draw a box around the model in the model image. 
   MgraRect M_DEFAULT MilSubImage (MODEL_XPOS - 2) (MODEL_YPOS - 2) (MODEL_XPOS + MODEL_WIDTH + 1) (MODEL_YPOS + MODEL_HEIGHT + 1)

  ; Pause to show the original image and model position. 
   echo "A model was defined in the source image."
   pause "Press <Enter> to continue."

  ; Translate the image on a subpixel level. 
   MimTranslate MilSubImage MilSubImage SHIFT_X SHIFT_Y M_DEFAULT
   echoln "Source image was shifted by "
   echoln [dbltostr SHIFT_X 4]
   echoln " in X and "
   echoln [dbltostr SHIFT_Y 4]
   echo " in Y."
   echo

  ; Allocate result buffer. 
   MpatAllocResult MilSystem 1 &Result

  ; Find the model in the target buffer. 
   MpatFindModel MilSubImage Model Result

  ;If one model was found above the acceptance threshold set.
   if ([MpatGetNumber Result  M_NULL]  == 1)
   {
     ; Read results and draw a box around model occurrence. 
     MpatGetResult Result M_POSITION_X &x
     MpatGetResult Result M_POSITION_Y &y
     MpatGetResult Result M_SCORE &Score

     setnbr xstart ( x-((MODEL_WIDTH/2)-1) )
     setnbr ystart ( y-((MODEL_HEIGHT/2)-1) )
     setnbr xend ( x+(MODEL_WIDTH/2) )
     setnbr yend ( y+(MODEL_HEIGHT/2) )

     MgraRect M_DEFAULT MilSubImage [getnbr xstart] [getnbr ystart] [getnbr xend] [getnbr yend]

    ; Pause to show the shifted image and print out the difference
    ; to confirm the sub-pixel accuracy. 
   
     MpatInquire Model M_ORIGINAL_X &XOrg
     MpatInquire Model M_ORIGINAL_Y &YOrg
     echoln "The model was found to be shifted by "
     echoln [dbltostr (x - XOrg) 4]
     echoln " in X and "
     echoln [dbltostr (y - YOrg) 4]
     echo   " in Y."             
     echoln "Model match score is "
     echoln [dbltostr Score 4]
     echo   " percent."
     echo

    ; Save model to disk for future use if verification was successful. 

     setstr str_model [ addstr imagepath MODEL_FILE ]
     MpatSave [getstr str_model] Model

     echoln "Model was saved on disk as '"
     echoln [getstr str_model] 
     echo "'."
   }

  ; Wait for a key press to continue.  
   pause "Press <Enter> to end."

  ; Free all allocations. 
   MpatFree Result
   MpatFree Model
   MbufFree MilSubImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage


; Enable the automatic printing of the return values of commands
silence 0
