; File name: mshift.cl
; Synopsis:  This program finds the horizontal and vertical 
;            displacement of a wafer image. 
 


; Disable the automatic printing of the return values of commands
silence 1

; Source and target images file specifications.  
macro MODEL_IMAGE_FILE   "\wafer.mim"
macro TARGET_IMAGE_FILE  "\shfwafer.mim"
macro IMAGE_WIDTH        512
macro IMAGE_HEIGHT       480

; Model width, height, maximum displacement. 
macro MODEL_WIDTH         60
macro MODEL_HEIGHT        60
macro MODEL_MAX_DISPLACE  80
macro MODEL_POS_X_INIT    350
macro MODEL_POS_Y_INIT    325

  MIL_ID MilApplication          ; Application identifier.  
  MIL_ID MilSystem               ; System identifier.       
  MIL_ID MilDisplay              ; Display identifier.      
  MIL_ID MilImage                ; Image buffer identifier. 
  MIL_ID MilSubImage             ; Sub-image buffer identifier. 
  MIL_ID Model                   ; Model identifier.  
  MIL_ID Result                  ; Result buffer identifier. 
  long   Width                   ; Model size. Might get changed... 
  setWidth  MODEL_WIDTH
  long   Height                  ; ...if not large enough. 
  setHeight MODEL_HEIGHT 
  long   PosX                    ; Model position. 
  long   PosY
  long   AllocError              ; Allocation error variable. 
  double OrgX                    ; Original center of model. 
  double OrgY
  double x                       ; Result variables. 
  double y
  double Score


  ; Allocate defaults. 
    MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

  ; Restrict the region to be processed to the image size. 
    MbufChild2d MilImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage

  ; Load model image into an image buffer. 
    setstr str_model [ addstr imagepath MODEL_IMAGE_FILE ]
    MbufLoad [getstr str_model] MilSubImage

  ; Announce the model definition.
    echo "A model is being defined in the source image."

  ; Automatically allocate normalized grayscale type model.
   setPosX (MODEL_POS_X_INIT - (MODEL_WIDTH / 2))
   setPosY (MODEL_POS_Y_INIT - (MODEL_HEIGHT / 2))
   MpatAllocModel MilSystem MilImage PosX PosY MODEL_WIDTH MODEL_HEIGHT M_NORMALIZED &Model

  ; Check for a successfull model allocation. 
   MappGetError M_CURRENT &AllocError

   if (AllocError==0)
   {
     ; Draw box around model.  
     MgraRect M_DEFAULT MilImage (PosX-1) (PosY-1) (PosX+MODEL_WIDTH) (PosY+MODEL_HEIGHT)
     echo "Model successfully defined as shown on the displayed image."
     pause "Press <Enter> to continue."
   }
   else
   {
     echo "Error: Automatic model definition failed."
     pause "Press <Enter> to end."
      
     ; Free child image and defaults. 
      MbufFree MilSubImage
      MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

     ; Enable the automatic printing of the return values of commands
     silence 0

     return
   }

     ; Restrict search zone and set search positional accuracy to medium. 
     MpatInquire Model M_ORIGINAL_X &OrgX
     MpatInquire Model M_ORIGINAL_Y &OrgY
     MpatSetPosition Model (OrgX-MODEL_MAX_DISPLACE) (OrgY-MODEL_MAX_DISPLACE) (2*MODEL_MAX_DISPLACE) (2*MODEL_MAX_DISPLACE)

     ; Load target image into an image buffer. 
     setstr str_target [ addstr imagepath TARGET_IMAGE_FILE ]
     MbufLoad [getstr str_target] MilImage

     ; Allocate result. 
     MpatAllocResult MilSystem 1 &Result

     ; Find model.  
     MpatFindModel MilImage Model Result

     ; If one model was found above the acceptance threshold set.
     if ([MpatGetNumber Result  M_NULL]  == 1)
     {
       ; Get results.  
       MpatGetResult Result M_POSITION_X &x 
       MpatGetResult Result M_POSITION_Y &y
       MpatGetResult Result M_SCORE &Score

       ; Draw a box around occurrence. 
       setnbr xstart ( x-((MODEL_WIDTH/2)-1) )
       setnbr ystart ( y-((MODEL_HEIGHT/2)-1) )
       setnbr xend ( x+(MODEL_WIDTH/2) )
       setnbr yend ( y+(MODEL_HEIGHT/2) )

       MgraRect M_DEFAULT MilSubImage [getnbr xstart] [getnbr ystart] [getnbr xend] [getnbr yend] 

       ; Analyze and print results. 
       echo
       echo "A misaligned version of the source image was loaded."
       echo
       echoln "Image was found to be offset by "
       echoln [dbltostr (x - OrgX) 4]
       echoln " in X, and "
       echoln [dbltostr (y - OrgY) 4]
       echo " in Y."
       echoln "Model match score is "
       echoln [dbltostr Score 4]
       echo " percent."
       pause "Press <Enter> to continue."
   
     }
     else
     {
       echo "Error: Pattern not found properly."
       pause "Press <Enter> to end."
     }

  ; Free child image, result buffer , model and defaults. 
    MpatFree Result
    MpatFree Model
    MbufFree MilSubImage
    MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0


