; File name: msurvey.cl
; Synopsis:  This program grabs an image which is expected to be
;            the constant dark background, and then subtracts
;            this background from subsequent grabbed images. 
;


; Disable the automatic printing of the return values of commands
silence 1

macro  CAMSIZEBIT 8

  MIL_ID MilApplication  ; Application identifier.             
  MIL_ID MilSystem       ; System identifier.                  
  MIL_ID MilDisplay      ; Display identifier.                 
  MIL_ID MilCamera       ; Camera identifier.                   
  MIL_ID MilImage        ; Image buffer identifier.            
  MIL_ID GrabImage       ; Grab image buffer identifier.       
  MIL_ID BackgroundImage ; Background image buffer identifier. 
  long   CamSizeX        ; Camera width variable.              
  long   CamSizeY        ; Camera height variable.             


  ; Allocate defaults. 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay &MilCamera &MilImage

  ; Reads camera X, Y and depth dimensions. 
   MdigInquire MilCamera M_SIZE_X &CamSizeX
   MdigInquire MilCamera M_SIZE_Y &CamSizeY

  ; Allocate a second image buffer to store the background image. 
   MbufAlloc2d M_DEFAULT CamSizeX CamSizeY (CAMSIZEBIT+M_UNSIGNED) (M_IMAGE+M_PROC) &BackgroundImage

  ; Allocate a third image buffer to grab the changing image. 
   MbufAlloc2d MilSystem CamSizeX CamSizeY (CAMSIZEBIT+M_UNSIGNED) (M_IMAGE+M_GRAB+M_PROC) &GrabImage

  ; Grab the background image in the display buffer.  
   MdigGrabContinuous MilCamera MilImage

  ; When a key is pressed, halt. 
   echo "Point camera at a constant dark background and adjust focus."
   pause "Press <Enter> to continue."
   MdigHalt MilCamera

  ; Copy the displayed buffer into the background buffer. 
   MbufCopy MilImage BackgroundImage

  ; When a key is pressed, halt. 
   echo "Continuous subtraction in progress..."
   echo
   echo "Keeping camera in the same position, create motion with a"
   echo "bright object in front of the background."
   echo "Press <Enter> to stop operation and end."

  ; Grab and subtract background in loop. 
  while ( [keyhit] == 0 )
      {
      MdigGrab MilCamera GrabImage
      MimArith GrabImage BackgroundImage MilImage M_SUB_ABS 
      }

  ; Release defaults and image.  
   MbufFree GrabImage 
   MbufFree BackgroundImage 
   MappFreeDefault MilApplication MilSystem MilDisplay MilCamera MilImage


; Enable the automatic printing of the return values of commands
silence 0






