/* File name: mdisplay.c
 * Synopsis:  This program allocates a displayable image buffer,
 *            clears its contents, draws a filled circle and then
 *            displays it. It also checks whether the allocation
 *            was successful using the MIL error reporting 
 *            mechanism.
 */

#include <stdio.h> 
#include <stdlib.h> 
#include <mil.h> 

#define IMAGE_DEPTH  8L

void main(void)
{ 
   MIL_ID MilApplication,  /* Application identifier.  */
          MilSystem,       /* System identifier.       */
          MilDisplay,      /* Display identifier.      */
          MilImage;        /* Image buffer identifier. */
   long   ErrorCode;       /* Error code value.        */

   /* Allocate defaults. */
   MappAllocDefault(M_SETUP, &MilApplication, &MilSystem, &MilDisplay,
                    M_NULL, M_NULL);

   /* Allocate an image buffer with the default dimensions to do graphics. */
   MbufAlloc2d(MilSystem, M_DEF_IMAGE_SIZE_X_MIN,M_DEF_IMAGE_SIZE_Y_MIN,
               M_DEF_IMAGE_TYPE, M_IMAGE+M_DISP, &MilImage);
  
   /* Check the error status code set by the allocation command. If there
    * was no error, draw and display a circle, otherwise print an error
    * message and exit.
    */
   MappGetError(M_CURRENT, &ErrorCode);
   if (ErrorCode == M_NULL)
      {
      /* Clear buffer and draw a circle. */ 
      MbufClear(MilImage, 0L);
      MgraColor(M_DEFAULT, 255L);
      MgraArcFill(M_DEFAULT, MilImage, 256L, 240L, 100L, 100L, 0.0, 360.0);
  
      /* Display the image buffer. */
      MdispSelect(MilDisplay, MilImage);

      /* Print a message. */
      printf("A circle was drawn in the displayed image buffer.\n");
      printf("Press <Enter> to end.\n");
      getchar();

      /* Release image buffer. */
      MbufFree(MilImage);
      }
   else
      {
      /* Print an error message. */
      printf("Error: Image buffer allocation failed.\n");
      printf("Press <Enter> to end.\n");
      getchar();
      }
  
   /* Release defaults. */
   MappFreeDefault(MilApplication, MilSystem, MilDisplay, M_NULL, M_NULL);
}

