/* File name: DoubleGrab.c 
 * Synopsis:  This program shows how to Grab with two different cameras at the same time with two
 *            continuous grabs. if two continuous grags are not supported, the program simulates a
 *            continuous grab using many simple grabs
 *
 *     Thread usage:
 *      - The main thread starts a processing thread in each of the 4 different
 *        quarters of a display buffer. The main thread then waits for a key to
 *        be pressed to stop them.
 *      - The top-left and bottom-left threads work in a loop, as follows: the
 *        top-left thread draws a rectangle in its buffer (in a size
 *        different from the previous rectangle), then sends an event to the 
 *        bottom-left thread. The bottom-left thread waits for the event from 
 *        the top-left thread, copies the contents of the top-left buffer into 
 *        its buffer, draws a circle in the rectangle, then sends an event to 
 *        the top-left thread. When the top-left thread receives the event, the 
 *        loop continues.
 *      - The top-right and bottom-right threads work exactly the same way as the
 *        top-left and bottom-left threads.
 *
 *      Note that the top and bottom threads (of each half) could be set to do
 *      something else while waiting for each other. 
 */

/* headers */
#include <stdio.h> 
#include <mil.h>



/* Main function: */
/* -------------- */
void main(void)
{ 
   MIL_ID MilApplication,        /* Application identifier.                   */
          MilSystem,             /* System identifier.                        */
          MilDisplay,            /* Display identifier.                       */
          MilImage,              /* Image buffer identifiers.                 */
          MilDigitizer,          /* Digitizer identifier.    */
          MilSystem2,
          MilDisplay2,           /* Display identifier.      */
          MilDigitizer2,         /* Digitizer identifier.    */ 
          MilImage2;

   /* Allocate defaults*/
   MappAllocDefault(M_SETUP, &MilApplication, &MilSystem,
                             &MilDisplay, &MilDigitizer, &MilImage);
   
   /* Allocate everything that is needed for the second digitizer. */
   MsysAlloc(M_SYSTEM_SETUP, M_DEFAULT, M_COMPLETE, &MilSystem2);
   MdispAlloc(MilSystem2, M_DEV1, "M_DEFAULT", M_WINDOWED, &MilDisplay2);
   MdigAlloc(MilSystem2, M_DEV1, "M_DEFAULT", M_DEFAULT, &MilDigitizer2);
   MbufAllocColor(MilSystem2,
                  MbufInquire(MilImage, M_SIZE_BAND, M_NULL),
                  MbufInquire(MilImage, M_SIZE_X, M_NULL),
                  MbufInquire(MilImage, M_SIZE_Y, M_NULL),
                  MbufInquire(MilImage, M_TYPE, M_NULL),
                  MbufInquire(MilImage, M_ATTRIBUTE, M_NULL),
                  &MilImage2);
   MdispSelect(MilDisplay2, MilImage2);


   /* Start continuous grabs.*/
   MdigGrabContinuous(MilDigitizer, MilImage );
   MdigGrabContinuous(MilDigitizer2, MilImage2 );
   printf("Press any key to end grab\n");
   getchar();


   /* Stop continuous grabs. */
   MdigHalt(MilDigitizer );
   MdigHalt(MilDigitizer2 );

   /*Free the second digitizer*/
   MbufFree(MilImage2);
   MdispFree(MilDisplay2);
   MdigFree(MilDigitizer2);
   MsysFree(MilSystem2);

   /* Release defaults. */
   MappFreeDefault(MilApplication, MilSystem, MilDisplay, MilDigitizer, MilImage);
   }

