/* File name: mgrabhk.c
/* This example grab a sequence of images using a hook functions and      */
/* plays it back. It also use events to signal the end of each grab       */
/* and to do some annotations to the buffer.                              */
/*                                                                        */

/* Headers */
#include <stdio.h> 
#include <stdlib.h> 
#include <conio.h> 
#include <mil.h>

/* Number of buffers in the sequence. */
#define NB_GRAB       24

/* Target Image scale. */
#define IMAGE_SCALE   0.5

/* Hook functions prototypes. */
long MFTYPE GrabStart(long, MIL_ID, void MPTYPE *);
long MFTYPE GrabEnd(long, MIL_ID, void MPTYPE *);

/* Hook data structure. */
typedef volatile struct
   {
   MIL_ID  *MilImage; 
   MIL_ID  MilDigitizer;  
   MIL_ID  GrabEndEvent;
   long    NbGrabStart;
   long    NbGrabEnd;
   double  Time;
   } UserDataStruct;

/* Main function. */
void main(void)
{ 
   MIL_ID   MilApplication;
   MIL_ID   MilSystem     ;
   MIL_ID   MilDigitizer  ;
   MIL_ID   MilDisplay    ;
   MIL_ID   MilImage[NB_GRAB];
   MIL_ID   MilImageDisp  ;
   long n;
   double TimeWait = 0.0;
   MILTCHAR FrameIndex[10];
   UserDataStruct UserStruct;
         
   /* Allocations */
   MappAlloc(M_DEFAULT, &MilApplication);
   MsysAlloc(M_DEF_SYSTEM_TYPE, M_DEF_SYSTEM_NUM, M_SETUP, &MilSystem);
   MdigAlloc(MilSystem, M_DEFAULT, M_DEF_DIGITIZER_FORMAT, M_DEFAULT, &MilDigitizer);
   MdispAlloc(MilSystem, M_DEFAULT, M_DEF_DISPLAY_FORMAT, M_DEFAULT, &MilDisplay);
   for (n=0; n<NB_GRAB; n++)
      {
      MbufAlloc2d(MilSystem,  
               (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)*IMAGE_SCALE),
               (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)*IMAGE_SCALE),
               8L+M_UNSIGNED, 
               M_IMAGE+M_GRAB, &MilImage[n]);
      }
   MbufAlloc2d(MilSystem,  
               (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)*IMAGE_SCALE),
               (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)*IMAGE_SCALE),
               8L+M_UNSIGNED, 
               M_IMAGE+M_GRAB+M_PROC+M_DISP, &MilImageDisp);

   /* MIL event allocation */
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, (long *)&(UserStruct.GrabEndEvent));
   
   /* Display activation. */
   MbufClear(MilImageDisp, 0x0);            
   MdispSelect(MilDisplay, MilImageDisp);

   /* Clear the sequence buffers. */
   for (n=0; n<NB_GRAB; n++)
      MbufClear(MilImage[n], 0xFF);
  
   /* Initialize hook structure. */
   UserStruct.MilDigitizer = MilDigitizer;
   UserStruct.MilImage     = MilImage;
   UserStruct.NbGrabStart  = -2;
   UserStruct.NbGrabEnd    = -2;
   UserStruct.Time         =  0;

   /* Grab on display at specified scale. */
   MdigControl(MilDigitizer, M_GRAB_SCALE, IMAGE_SCALE);
   MdigGrabContinuous(MilDigitizer, MilImageDisp);
  
   /* Print a message and stop the grab. */
   printf("Press enter to record the sequence using grab hook (interrupt).\n");
   getchar();
   MdigHalt(MilDigitizer);
  
   /* Hook the functions to grab the sequence. */
   MdigHookFunction(MilDigitizer, M_GRAB_START, GrabStart, (void *)(&UserStruct));
   MdigHookFunction(MilDigitizer, M_GRAB_END,   GrabEnd,   (void *)(&UserStruct));

   /* Put digitizer in asynchronous mode. */
   MdigControl(MilDigitizer, M_GRAB_MODE, M_ASYNCHRONOUS);

   /* Start sequence with a grab in the first buffer. */
   MdigGrab(MilDigitizer, MilImage[0]);
   
   /* At this point the CPU is free to do other task and the sequence will be
    * grabbed during that time. Here, the CPU will only print and draw the   
    * index of the buffer grabbed.
    */
   while (UserStruct.NbGrabEnd < NB_GRAB)
     {
     long GrabEndIndex;
     MappControlThread(UserStruct.GrabEndEvent, M_EVENT_WAIT, M_DEFAULT, M_NULL);
     GrabEndIndex = UserStruct.NbGrabEnd-1;
     if (GrabEndIndex >= 0)
         {
         printf("\rBuffer #%ld grabbed   ",GrabEndIndex);
         MOs_ltoa(GrabEndIndex, FrameIndex, 10);
         MgraText(M_DEFAULT, MilImage[GrabEndIndex], 32, 32, FrameIndex);
         MbufCopy(MilImage[GrabEndIndex],MilImageDisp);
         }
     }

   /* Wait last grab end. */
   MdigGrabWait(MilDigitizer, M_GRAB_END);

   /* Print statistics. */
   printf("\n%ld frames grabbed, at a frame rate of %.2f frames/sec (%.2f ms/frame).\n",
          UserStruct.NbGrabStart, UserStruct.NbGrabStart/UserStruct.Time, 1000.0*UserStruct.Time/UserStruct.NbGrabStart);
          
   /* Play the sequence until <enter> is pressed. */
   printf("Press <enter> to end the playback.\n");
   while( !kbhit() )
      {
      /* Play the sequence once. */
      for (n=0; n<NB_GRAB; n++)
         {
         /* Copy one image to the screen. */
         MappTimer(M_TIMER_RESET, M_NULL);
         MbufCopy(MilImage[n],MilImageDisp);
         MappControlThread(M_DEFAULT, M_THREAD_WAIT, M_DEFAULT, M_NULL);
         MappTimer(M_TIMER_READ, &TimeWait);
         TimeWait = (UserStruct.Time/UserStruct.NbGrabStart) - TimeWait;
         MappTimer(M_TIMER_WAIT, &TimeWait);
         }
      }   
   getchar();   
      
   /* Unhook functions. */
   MdigHookFunction(MilDigitizer, M_GRAB_START+M_UNHOOK, GrabStart, (void *)(&UserStruct));
   MdigHookFunction(MilDigitizer, M_GRAB_END+M_UNHOOK, GrabEnd, (void *)(&UserStruct));

   /* Free event. */
   MappControlThread(UserStruct.GrabEndEvent, M_EVENT_FREE, M_DEFAULT, M_NULL);
   
   /* Free allocations. */
   MbufFree(MilImageDisp);
   for (n=0; n<NB_GRAB; n++)
      {
      MbufFree(MilImage[n]);
      }
   MdispFree(MilDisplay);
   MdigFree(MilDigitizer);
   MsysFree(MilSystem);
   MappFree(MilApplication);
}  


/* Grab Start hook function: 
 *      - This function is used to queue a grab at the beginning of
 *        the each current grab.
 *      - It is also used to mesure the frame rate. This is done 
 *        by starting a timer at the beginning of a dummy grab and 
 *        by stopping the timer at the beginning of the last grab.  
 */
long MFTYPE GrabStart(long HookType, MIL_ID EventId, void MPTYPE *UserStructPtr)
{
  UserDataStruct *UserPtr=(UserDataStruct*)UserStructPtr;

  /* Increment grab start count */
  UserPtr->NbGrabStart++;

  /* Start the timer when needed */
  if(UserPtr->NbGrabStart == 0)
  	 MappTimer(M_TIMER_RESET, (double *)&UserPtr->Time);
     
  /* Queue a grab and stop the timer at the end */
  if (UserPtr->NbGrabStart < NB_GRAB)
     MdigGrab(UserPtr->MilDigitizer, 
              UserPtr->MilImage[(UserPtr->NbGrabStart < 0) ? 0 : UserPtr->NbGrabStart]);
  else   
     MappTimer(M_TIMER_READ, (double *)&UserPtr->Time);
        
  return(0);
}


/* Grab end hook function: 
 *      - This function is used to signal to a waiting thread (here the main())
 *        that a grab is completed and that the data can be processed.
 *
 *  Note: Time spend in the hook function should be minimal. External 
 *	      thread waiting on an event should be used to do processing.
 */
long MFTYPE GrabEnd(long HookType, MIL_ID EventId, void MPTYPE *UserStructPtr)
{
  UserDataStruct *UserPtr=(UserDataStruct*)UserStructPtr;
  
  /* Increment grab count */
  UserPtr->NbGrabEnd++;
        
  /* Signal the end of grab event to the main thread waiting */
  MappControlThread(UserPtr->GrabEndEvent, M_EVENT_SET, M_SIGNALED, M_NULL);
  
  return(0);
}

