/***********************************************************************/
/* This example grab a sequence of images and play it back.            */
/*                                                                     */
/* Note: This assume that the copy operation is shorter than a grab    */
/*       and that the PC has sufficient bandwith to support the        */
/*       operations.                                                   */

/* headers */
#include <stdio.h> 
#include <stdlib.h>
#include <conio.h>
#include <mil.h> 

/* Number of buffers in the sequence. */
#define NB_GRAB            100

/* Image scale. */
#define IMAGE_SCALE        0.5

/* Quantization factor to use during the compression */
/*  Permissible values from 1 to 99 both inclusively */
/*  Used only if the compression is available        */
#define Q_FACTOR           50

/* Set to 0 to disable AVI Import/Export */
#define AVI_IMPORT_EXPORT  1   

/* Sequence file */
#define SEQUENCE_FILE M_IMAGE_PATH MT("seq.avi")

/* Main function. */
void main(void)
{ 
   MIL_ID   MilApplication  ;
   MIL_ID   MilSystem       ;
   MIL_ID   MilDigitizer    ;
   MIL_ID   MilDisplay      ;
   MIL_ID   MilImage[NB_GRAB];
   MIL_ID   MilImageDisp    ;
  
   double Time            = 0.0;
   double TimeWait        = 0.0;
   double TotalReplay     = 0.0;
   double ImageScale      = IMAGE_SCALE;
   long   n               = 0;
   long   ExtendedAttrib  = 0;
   long   NbFrames        = 0;

#if !M_MIL_USE_DOS_32   
   long   Selection       = 1;
#endif
         
   /* Allocate defaults. */
   MappAllocDefault(M_SETUP, &MilApplication, &MilSystem,
                    &MilDisplay, &MilDigitizer, M_NULL);

#if !M_MIL_USE_DOS_32   
   /* Select between grabbing a compressed or a non compressed sequence */
   if ( MsysInquire(MilSystem, M_COMPRESSION_SUPPORTED, M_NULL) )
      {
      printf( "You may choose between :\n" );
      printf( "1) Grab a sequence of non-compressed images.\n" );
      printf( "2) Grab a sequence of images compressed with a lossy jpeg algorithm.\n" );
      printf( "Please enter your selection : " );
      scanf( "%d", &Selection ); getchar();
      printf( "Selection Done : %d\n", Selection );
      if ( Selection == 2 )
         {
         ExtendedAttrib = M_COMPRESS | M_JPEG_LOSSY;
         if ( MsysInquire(MilSystem, M_BOARD_TYPE, M_NULL) & M_COMPRESSION_MODULE )   
            {
            /* Set full size compression since special hardware is present */
            ImageScale=1.0;
            
            /* The compression algorithm used depends on the digitizer */
            if ( MdigInquire( MilDigitizer, M_SCAN_MODE, M_NULL ) == M_INTERLACE )
               ExtendedAttrib = M_COMPRESS | M_JPEG_LOSSY_INTERLACED;
            }
         }
      else if ( Selection < 0 || Selection > 2 )
         {
         printf( "Invalid selection.\nEnd of Program." );
         exit(0);
         }
      }
#endif
   
   /* Allocate display buffer. */   
   MbufAllocColor(MilSystem,  
                  MdigInquire(MilDigitizer, M_SIZE_BAND, M_NULL),
                  (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)*ImageScale),
                  (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)*ImageScale),
                  8L+M_UNSIGNED, 
                  M_IMAGE+M_GRAB+M_PROC+M_DISP, &MilImageDisp);
            
   /* Allocate grab buffers to hold the sequence. */                  
   MappControl(M_ERROR, M_PRINT_DISABLE);                  
   for (n=0; n<NB_GRAB; n++)
      {
      MbufAllocColor(MilSystem,  
                     MdigInquire(MilDigitizer, M_SIZE_BAND, M_NULL),
                     (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)*ImageScale),
                     (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)*ImageScale),
                     8L+M_UNSIGNED, 
                     M_IMAGE+M_GRAB+ExtendedAttrib, &MilImage[n]);
      if (MilImage[n])                     
         {
         NbFrames++;
         if ( ExtendedAttrib )
            MbufControl( MilImage[n], M_Q_FACTOR, Q_FACTOR );
         }
      else
         {
         break;
         }
      }
   MappGetError(M_GLOBAL, M_NULL); /* Reset errors */
   MappControl(M_ERROR, M_PRINT_ENABLE);                  
   
   /* Display activation. */
   MbufClear(MilImageDisp, 0x0);            
   MdispSelect(MilDisplay, MilImageDisp);
 
   /* Grab continuous on display at the specified scale. */
   MdigControl(MilDigitizer, M_GRAB_SCALE, ImageScale);
   MdigGrabContinuous(MilDigitizer,MilImageDisp);
  
   /* Print a message. */
   printf("Press <Enter> to record the sequence.\n");
   getchar();
  
   /* Halt continuous grab and put digitizer in asynchronous mode. */
   MdigHalt(MilDigitizer);
   MdigControl(MilDigitizer, M_GRAB_MODE, M_ASYNCHRONOUS);
  
   /* Synchronize the timer at the beginning of a frame by sending a dummy
      grab and then call the first real grab of the sequence. */
   MdigGrab(MilDigitizer, MilImage[0]);
   MdigGrab(MilDigitizer, MilImage[0]);
   MappTimer(M_TIMER_RESET, M_NULL);

   /* Grab the sequence. */
   for (n=1; n<NbFrames; n++)
      {
      /* Grab one buffer at a time. */
      MdigGrab(MilDigitizer, MilImage[n]);
      }
      
   /* Wait last grab end. */
   MdigGrabWait(MilDigitizer, M_GRAB_END);
   MappTimer(M_TIMER_READ, &Time);

   /* Print statistics. */
   printf("%ld frames grabbed, at a frame rate of %.2f frames/sec (%.2f ms/frame).\n",
          NbFrames, NbFrames/Time, 1000.0*Time/NbFrames);
      
   #if (AVI_IMPORT_EXPORT && !M_MIL_USE_DOS_32)
   /* Save the sequence into a file */
   printf( "Saving the sequence into an AVI file...\n" );
   MbufExportSequence( SEQUENCE_FILE, 
                       M_DEFAULT, 
                       MilImage, 
                       NbFrames, 
                       (long)(NbFrames/Time), 
                       M_DEFAULT );

   /* Free all the buffers of the sequence. */
   for (n=0; n<NbFrames; n++)
      {
      MbufFree(MilImage[n]);
      }

   /* Restore the sequence from the file. */
   printf( "Restoring the sequence from the file...\n" );
   NbFrames = MbufDiskInquire( SEQUENCE_FILE, M_NUMBER_OF_IMAGES, M_NULL );
   MbufImportSequence( SEQUENCE_FILE, 
                       M_DEFAULT, 
                       M_RESTORE, 
                       MilSystem, 
                       MilImage, 
                       0, 
                       NbFrames, 
                       M_DEFAULT );
   #endif /* #if AVI_IMPORT_EXPORT */

   /* Play the sequence until a key is pressed. */
   printf("Press <Enter> to end the playback.\n");
   do   
      {
      TotalReplay = 0.0;
      /* Play the sequence once. */
      for (n=0; n<NbFrames; n++)
         {
         /* Copy one image to the screen. */
         MappTimer(M_TIMER_RESET, M_NULL);
         MbufCopy(MilImage[n],MilImageDisp);
         MappControlThread(M_DEFAULT, M_THREAD_WAIT, M_DEFAULT, M_NULL);
         MappTimer(M_TIMER_READ, &TimeWait);
         TotalReplay += TimeWait;
         TimeWait = (Time/NbFrames) - TimeWait;
         MappTimer(M_TIMER_WAIT, &TimeWait);
         TotalReplay += (TimeWait > 0) ? TimeWait: 0.0;
         }
      }   
   while( !kbhit() );
   getchar();   

   /* Print statistics. */
   printf("%ld frames replayed, at a frame rate of %.2f frames/sec (%.2f ms/frame).\n",
          NbFrames, NbFrames/TotalReplay, 1000.0*TotalReplay/NbFrames);
   printf("Press <Enter> to end.\n");
   getchar();

      
   /* Free all allocated buffers. */
   for (n=0; n<NbFrames; n++)
      {
      MbufFree(MilImage[n]);
      }
   MbufFree(MilImageDisp);
   
   /* Release defaults. */
   MappFreeDefault(MilApplication, MilSystem, MilDisplay,
                   MilDigitizer, M_NULL);
}  
