/***************************************************************************************** 
*  This example presents a generic way to grab with two digitizers on the Matrox CORONA.
*
*    1. Allocated two digitizers (M_DEV0 and M_DEV1) with the same format. And pass 
*       M_DISPLAY_DIGITIZER as InitFlag in the second digitizer (M_DEV1).
*    2. The display digitizer (M_DEV1) can only grab in display frame buffer memory so
*       the grab buffer must have the M_ON_BOARD attribute.
*
*  Note: Only standard cameras are supported with this feature.
*        Modification has to be done in the dcf to allow grab by the RGB path.
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include "mil.h"

void main(void)
   {
   MIL_ID  MilApplication;
   MIL_ID  MilSystem;
   MIL_ID  MilDisplay[2];
   MIL_ID  MilDigitizer[2];
   MIL_ID  MilImage[2];
   long    BufferSizeBand;
   long    MDisplayMode;

   /* System allocation. */
   MappAlloc(M_DEFAULT, &MilApplication);
   MsysAlloc(M_SYSTEM_CORONA, M_DEF_SYSTEM_NUM, M_SETUP, &MilSystem);

   /* Allocate 2 displays, one for each digitizers. */
   MdispAlloc(MilSystem, M_DEV0, M_DEF_DISPLAY_FORMAT, M_WINDOWED+M_OVR, &MilDisplay[0]);
   MdispAlloc(MilSystem, M_DEV1, M_DEF_DISPLAY_FORMAT, M_DEFAULT, &MilDisplay[1]);

   /* Allocate 2 digitizers, one that uses the default grab path (M_DEFAULT) and      */
   /* the other that uses the rainbow-runner grab path (M_DISPLAY_DIGITIZER).         */
   MdigAlloc(MilSystem, M_DEV0, M_DEF_DIGITIZER_FORMAT, M_DEFAULT          , &MilDigitizer[0]);
   MdigAlloc(MilSystem, M_DEV1, M_DEF_DIGITIZER_FORMAT, M_DISPLAY_DIGITIZER, &MilDigitizer[1]);

   /* The grab buffers must have the same size band as the display. */
   if ((long)(MdispInquire(MilDisplay[0], M_VGA_PIXEL_FORMAT, M_NULL)) == M_MONO8)
      BufferSizeBand = 1;
   else
      BufferSizeBand = 3;

   /* Allocate the first grab buffer for the first digitizer. */
   MbufAllocColor(MilSystem,  
               BufferSizeBand,
               (long)(MdigInquire(MilDigitizer[0], M_SIZE_X, M_NULL)),
               (long)(MdigInquire(MilDigitizer[0], M_SIZE_Y, M_NULL)),
               8L+M_UNSIGNED, 
               M_IMAGE+M_GRAB+M_PROC+M_DISP, &MilImage[0]);

   /* Allocate the second grab buffer for the display digitizer.        */
   /* The M_ON_BOARD attribute must be added to allocate the buffer     */
   /* in the display frame buffer memory.  That's because the rainbow-  */
   /* runner can only grab in that memory.                              */
   /* The size band of the M_ON_BOARD buffer must be the same as the    */
   /* display.                                                          */
   MappControl(M_ERROR, M_PRINT_DISABLE); 
   MbufAllocColor(MilSystem,
               BufferSizeBand,
               (long)(MdigInquire(MilDigitizer[0], M_SIZE_X, M_NULL)),
               (long)(MdigInquire(MilDigitizer[0], M_SIZE_Y, M_NULL)),
               8L+M_UNSIGNED, 
               M_IMAGE+M_GRAB+M_DISP+M_ON_BOARD, &MilImage[1]);
   MappControl(M_ERROR, M_PRINT_ENABLE);
  
   if (!MilImage[1])                     
      {
      printf("Not enough memory on board to allocate buffer.\n");
      printf("Make sure you are using a 24 bits display.\n");
      printf("Or try to reduce your display resolution.\n");
      }
   else
      {
      MbufClear(MilImage[0], M_BLACK);
      MbufClear(MilImage[1], M_BLACK);
      /* Grab live with the display digitizer.    */
      MdispSelect (MilDisplay[1], MilImage[1]);
      MsysInquire(MilSystem, M_DUAL_SCREEN_MODE, &MDisplayMode);

      if (MDisplayMode == M_DISABLE)
         MdigControl(MilDigitizer[1], M_GRAB_SCALE, M_FILL_DISPLAY);
         
         MdigGrabContinuous(MilDigitizer[1], MilImage[1]);
         
         /* Grab monoshots with the first digitizer and copy to the display. */
         /* These monoshots are done in parallel with the live grab.         */
         MdispSelect (MilDisplay[0], MilImage[0]);

         printf("Press <enter> to stop both grab.\n");
         while( !kbhit() )
            MdigGrab(MilDigitizer[0], MilImage[0]);
         
         
         getchar();   
         
         /* Stop the live grab. */
         MdigHalt    (MilDigitizer[1]);
         printf("Press <enter> to end.\n");
         getchar();
         
         /* Free allocations. */
         MbufFree (MilImage[1]);
      }

   /* Free allocations. */
   MbufFree (MilImage[0]);   
   MdigFree (MilDigitizer[0]);
   MdigFree (MilDigitizer[1]);
   MdispFree(MilDisplay[0]);
   MdispFree(MilDisplay[1]);
   MsysFree (MilSystem);
   MappFree (MilApplication);
   }

