/* File name: muart.c 
 * Synopsis:  This program allows you to test all the uart features
 *            to read and write data.  The user must physically link
 *            the uart connector on his Matrox Board to the specified
 *            com port on the same computer.
 * 
 * Note :     This example will only run with Matrox boards who support
 *            the uart features.  These boards are :
 *            Meteor 2, Meteor 2 MC,
 *            Corona revision 2+ and Corona LC revision 2+
 */

/* headers */
#include <stdio.h> 
#include <mil.h> 
#include <windows.h>

/* String of data to be sent */
#define TEST_STRING        "This is a test string."

/* Uart configuration parameters */

/* See board specific notes for a complete list of available baud rates */
#define CONFIG_SPEED       19200

/* only 1 or 2 stop bits are allowed */
#define CONFIG_STOP_BITS   1

/* only 7 or 8 data bits are allowed */
#define CONFIG_DATA_LENGTH 8

/* Parity can be set to M_DISABLE, M_ODD or M_EVEN */
#define CONFIG_PARITY      M_DISABLE

/* Standard windows com port name */
#define COM_PORT_NUMBER    "COM1"

/* Hooked Data Struct */
typedef struct 
{
   MIL_ID Digitizer;
   char   *ReadBuffer;
   long   ReadSize;
   long   ReadPosition;
}UartHook;

/* Hooked Function prototypes */
long MFTYPE ReadHook(long , MIL_ID , void MPTYPE *);

void main(void)
{
   MIL_ID MilApplication,  /* Application identifier.  */
          MilSystem,       /* System identifier.       */
          MilDigitizer;    /* Digitizer identifier.    */
   
   DCB dcb;                /* Windows com port configuration struct */
   HANDLE hCom;            /* Windows com port handle  */

   char *TestString;
   char *ReadBuffer;
   long SendSize;
   long ReadSize;
   long SentSize;
   long i;
   long volatile DoSomething=0;

   UartHook UserStruct;

   /* string buffers allocations. */
   TestString = TEST_STRING;
   SendSize = strlen( TestString );
   ReadBuffer = malloc ( SendSize + 1 );
   for ( i=0; i<=SendSize ; i++ )
      ReadBuffer[i] = '\0';
  
   /* Mil allocations. */
   MappAlloc( M_DEFAULT, &MilApplication );
   MsysAlloc( M_SYSTEM_SETUP, M_DEFAULT, M_DEFAULT, &MilSystem);
   MdigAlloc( MilSystem, M_DEFAULT, M_CAMERA_SETUP, M_DEFAULT, &MilDigitizer);

   /* Changing uart configuration on Matrox Board. */
   MdigControl( MilDigitizer, M_UART_PARITY,       CONFIG_PARITY      );
   MdigControl( MilDigitizer, M_UART_SPEED,        CONFIG_SPEED       );
   MdigControl( MilDigitizer, M_UART_DATA_LENGTH,  CONFIG_DATA_LENGTH );
   MdigControl( MilDigitizer, M_UART_STOP_BITS,    CONFIG_STOP_BITS   );
   
   /* Openning windows com port handle. */
   hCom = CreateFile ( COM_PORT_NUMBER , GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL );

   if ( hCom == INVALID_HANDLE_VALUE )
   {
      printf( " Unable to open com port\n");
      return;
   }

   if ( !GetCommState ( hCom, &dcb ))
   {
      printf( " Unable to get com port state\n");
      return;
   }

   /* Changing windows com port configuration. */
   dcb.BaudRate = CONFIG_SPEED;
   dcb.ByteSize = CONFIG_DATA_LENGTH;
   
   if       ( CONFIG_STOP_BITS == 1 )
      dcb.StopBits = ONESTOPBIT;
   else if  ( CONFIG_STOP_BITS == 2 )
      dcb.StopBits = TWOSTOPBITS;
   
   if       ( CONFIG_PARITY == M_DISABLE )
      dcb.Parity = NOPARITY;
   else if  ( CONFIG_PARITY == M_EVEN    )
      dcb.Parity = EVENPARITY;
   else if  ( CONFIG_PARITY == M_ODD     )
      dcb.Parity = ODDPARITY;

   dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;

   if ( !SetCommState( hCom, &dcb ) )
   {
      printf( " Unable to set com port to desired configuration\n");
      return;
   }
 
   /* Test to send data from the Matrox Uart. */ 
   printf("The program will now send a string from your Matrox board to your com port.\n");
   printf("press <Enter> to send data.\n");
   getchar();
   
   /* Setting the string delimiter to the standard '\0' character. */
   MdigControl( MilDigitizer, M_UART_WRITE_STRING_LENGTH, M_DEFAULT );
   
   /* Sending string trough the Matrox Uart. */
   MdigControl( MilDigitizer, M_UART_WRITE_STRING, ( unsigned long ) TestString );

   /* Reading string trough the windows com port. */
   if ( !ReadFile( hCom, ReadBuffer, SendSize, &ReadSize, NULL))
   {
      printf( " Unable to read data trough com port\n");
      return;
   }

   /* display the result. */
   printf( "String read trough com port: \n");
   printf( ReadBuffer );

   /* Test to read data from the Matrox board uart with M_UART_STRING_RECEIVE. */ 
   printf("\n\nThe program will now send a string from your com port to your Matrox board.\n");
   printf("press <Enter> to send data.\n");
   getchar();
   
   /* Clearing the read buffer. */
   for ( i=0; i<=SendSize ; i++ )
      ReadBuffer[i] = '\0';
   
   /* Writing data trough the windows com port. */
   if ( !WriteFile( hCom, TestString, SendSize, &SentSize, NULL))
   {
      printf( " Unable to write data trough com port\n");
      return;
   }
   
   /* setting the read timeout between each characters to 10 seconds. */
   MdigControl( MilDigitizer, M_UART_TIMEOUT, 10000 );

   /* setting the delimiter to the standard '\0' character. */
   MdigControl( MilDigitizer, M_UART_STRING_DELIMITER, M_DEFAULT );

   /* setting the string length to stop on the set delimiter. */
   MdigControl( MilDigitizer, M_UART_READ_STRING_LENGTH, M_DEFAULT );

   /* setting the maximum string length to prevent global protection faults. */
   MdigControl( MilDigitizer, M_UART_READ_STRING_MAXIMUM_LENGTH, SendSize );

   /* reading incomming data trough the Matrox board Uart. */
   MdigControl( MilDigitizer, M_UART_READ_STRING, (long)ReadBuffer );
   
   /* display the result. */
   printf( "String read trough Mil Uart: \n");
   printf( ReadBuffer );

   /* Test to read data from the Matrox board uart with a hooked function. */
   printf("\n\nThe program will now send a string from your com port to your Matrox board\n");
   printf("and read the data trough a hooked function.\n");
   printf("press <Enter> to send data.\n");
   getchar();
   
   /* clearing the read buffer. */
   for ( i=0; i<=SendSize ; i++ )
      ReadBuffer[i] = '\0';

   /* Initialize uart hook structure. */
   UserStruct.Digitizer = MilDigitizer;
   UserStruct.ReadBuffer = ReadBuffer;
   UserStruct.ReadSize = SendSize;
   UserStruct.ReadPosition =0;

   /* Hook the function to M_UART_RECEIVED_DATA. */
   MdigHookFunction(MilDigitizer, M_UART_DATA_RECEIVED, ReadHook, (void *)(&UserStruct));

   /* Send data trough the windows com port */
   if ( !WriteFile( hCom, TestString, SendSize, &SentSize, NULL))
   {
      printf( " Unable to write data trough com port\n");
      return;
   }
   
   /* At this point the CPU is free to do other tasks and the incomming string */
   /* will be read during that time. Here, the CPU will only wait the end of   */
   /* the string reading function.                                             */
   while (UserStruct.ReadPosition < SendSize )
      DoSomething=DoSomething;

   /* display the result. */
   printf( "String read trough hooked function: \n");
   printf( ReadBuffer );

   /* Pause to show the result. */
   printf("\nTransfer complete.\n");
   printf("Press <Enter> to end.\n");
   getchar();

   /* Unhook functions. */
   MdigHookFunction(MilDigitizer, M_UART_DATA_RECEIVED+M_UNHOOK, ReadHook, (void *)(&UserStruct));

   /* Free allocations. */
   MdigFree( MilDigitizer );
   MsysFree( MilSystem );
   MappFree( MilApplication );
}

/* Uart read hook function. */
long MFTYPE ReadHook(long HookType, MIL_ID EventId, void MPTYPE *UserStructPtr)
{
   UartHook *Params = ( UartHook* ) UserStructPtr;
   long Pending = M_TRUE;
   char TestChar=0;
   char *ReadChar = &TestChar;

   /* While there are still chars left in the incomming buffer, read them. */
   MdigInquire( Params->Digitizer, M_UART_DATA_PENDING, &Pending );
   while ( Pending == M_TRUE )
   {
      MdigControl( Params->Digitizer, M_UART_READ_CHAR, (long)(ReadChar) );
      MdigInquire( Params->Digitizer, M_UART_DATA_PENDING, &Pending );
      if ( Params->ReadPosition < Params->ReadSize )
         Params->ReadBuffer[Params->ReadPosition++] = TestChar;
   }
   return 0;
}