/* File name: mvmgrab.c
 * This example illustrates a VM grab with a Matrox Corona and a
 * Matrox Genesis board.
 * The Genesis grabs the frame from the camera and transfers it to 
 * the Corona by the VM channel.  The Corona receives the data as 
 * a classic grab but with the source of grab switched to VM grab.
 *                                    
 * Note: Double buffering is crucial if a display is allocated on the
 *       Genesis.
 *       Since the data transfered from the Genesis by the VM channel is 
 *       progressive by default, a progressive DCF must be selected for
 *       compatibility.
 *       The Genesis must be allocated first.
 */

#include <stdio.h>
#include <stdlib.h>
#include <mil.h>   
/* Genesis native include */
#include "imapi.h"

/* Title to put on the display window */
#define WINDOW_TITLE_COR "CORONA DISPLAY"
#define WINDOW_TITLE_GEN "GENESIS DISPLAY"

/* Overlay update function prototype */
void MyOverlayDraw(MIL_ID System, MIL_ID Display, long WindowedMode);

void main(void)
{
  MIL_ID MilApplication,
         MilSystemGen,
         MilSystem,
         MilDigitizer,
         MilDigitizerGen,
         MilDisplay,
         MilDisplayGen,
         MilImageGen[2],
         MilImageDispGen,
         MilImage;
  
  long     NativeSysThreadId;
  long     NativeDigId;
  long     NativeBufId[2];
  long     WindowedMode = M_FALSE;
  long     nbFrame      = 0;
  long     VMSizeX, VMSizeY;
  long     i        = 0;
  double   Time1    = 0.0; 
  double   Time2    = 0.0;
  double   AccTime  = 0.0;

  MappAlloc(M_DEFAULT, &MilApplication);

      /* Genesis system allocation */
  MsysAlloc(M_SYSTEM_GENESIS, M_DEFAULT, M_SETUP, &MilSystemGen);
  MdispAlloc(MilSystemGen, M_DEFAULT, "M_DEFAULT", M_DEFAULT, &MilDisplayGen);
  MdigAlloc (MilSystemGen, M_DEFAULT,  M_DEF_DIGITIZER_FORMAT, M_DEFAULT, &MilDigitizerGen);

  /* Corona system allocation */
  MsysAlloc(M_SYSTEM_CORONA, M_DEFAULT, M_SETUP, &MilSystem);
  MdigAlloc(MilSystem, M_DEFAULT, "vm.dcf", M_DEFAULT, &MilDigitizer);
  MdispAlloc(MilSystem, M_DEFAULT, M_DEF_DISPLAY_FORMAT, M_DEFAULT, &MilDisplay);

                                         
  /* Inquire the size of VM dcf */
  MdigInquire(MilDigitizer, M_SIZE_X, &VMSizeX);
  MdigInquire(MilDigitizer, M_SIZE_Y, &VMSizeY);

  /* Allocate 1 grab and display buffer and clear it. */
  MbufAllocColor(MilSystem, 1L, VMSizeX, VMSizeY, 8L, M_IMAGE+M_GRAB+M_DISP, &MilImage);
  MbufClear(MilImage, 0);

  /* Allocate 2 grab buffers and clear them. */
  MbufAllocColor(MilSystemGen, 1L, VMSizeX, VMSizeY, 8L, M_IMAGE+M_GRAB, &MilImageGen[0]);
  MbufAllocColor(MilSystemGen, 1L, VMSizeX, VMSizeY, 8L, M_IMAGE+M_GRAB, &MilImageGen[1]);
  MbufClear(MilImageGen[0], 0);
  MbufClear(MilImageGen[1], 0);

  /* Allocate 1 display buffer and clear it. */
  MbufAllocColor(MilSystemGen, 1L, VMSizeX, VMSizeY, 8L, M_IMAGE+M_GRAB+M_DISP, &MilImageDispGen);

  /* Display the buffer on the Genesis screen. */
  MdispSelect(MilDisplayGen, MilImageDispGen);

  /* Inquire Genesis native Id's */
  NativeSysThreadId   = MsysInquire(MilSystemGen    , M_NATIVE_THREAD_ID , M_NULL);
  NativeDigId         = MdigInquire(MilDigitizerGen , M_NATIVE_CAMERA_ID , M_NULL);
  NativeBufId[0]      = MbufInquire(MilImageGen[0]  , M_NATIVE_ID        , M_NULL);
  NativeBufId[1]      = MbufInquire(MilImageGen[1]  , M_NATIVE_ID        , M_NULL);

  /* Display the buffer on the Corona screen. */
  MdispSelect(MilDisplay, MilImage);

  /* Switch the Corona grab source to VM channel on the stream 14. */
  MdigControl(MilDigitizer, M_GRAB_VM, M_VM_STREAM_ID_14);

  /* If in Windowed mode, change display window title. */
  if(MdispInquire(MilDisplay, M_DISPLAY_MODE, M_NULL)==M_WINDOWED)
     WindowedMode=M_TRUE;
  else
     WindowedMode=M_FALSE;  

  if (WindowedMode)
     MdispControl(MilDisplay, M_WINDOW_TITLE_NAME, (long)WINDOW_TITLE_COR);

  /* Draw text and graphics in the overlay buffer. */
  MyOverlayDraw(MilSystem,MilDisplay,WindowedMode);

  /* Set the two Genesis buffers for VM transfer on the stream 14. */
  imBufPutField(NativeSysThreadId, NativeBufId[0], IM_CTL_STREAM_ID, 14);
  imBufPutField(NativeSysThreadId, NativeBufId[1], IM_CTL_STREAM_ID, 14);

  printf("Press enter key to start the grab\n");
  getchar();

  /* Grab continuous on the Corona display. */
  MdigGrabContinuous(MilDigitizer, MilImage);

  printf("VM grab in progress\n");
  printf("Press enter key to stop the grab\n");

  /* Put the digitizer in asynchronous mode. */
  MdigControl(MilDigitizerGen, M_GRAB_MODE, M_ASYNCHRONOUS);

  /* Grab the first buffer. */
  MdigGrab(MilDigitizerGen, MilImageGen[0]);

  /* Grab and transfer the buffers in double buffering. */
  i = 1;
  while (!kbhit())
     {
     /* Grab second buffer while transferring first buffer. */
     MdigGrab(MilDigitizerGen, MilImageGen[i]);

     /* Synchronize and start the timer. */
     if (nbFrame == 0)
        {
        MappTimer(M_TIMER_RESET, M_NULL);
        MappTimer(M_TIMER_READ, &Time1);
        }

     /* Switch buffers. */
     i = 1 - i;

     /* Display the first buffer on the Genesis screen */
     MbufCopy(MilImageGen[i], MilImageDispGen);

     /* Transfer the first buffer already grabbed.  */
     imBufCopyVM(NativeSysThreadId, NativeBufId[i], IM_VM_CHANNEL, NativeBufId[i], 0, 0);
     imSyncHost(NativeSysThreadId, 0, IM_COMPLETED);

     /* Count transfered buffers. */
     nbFrame++;
     }

  MappTimer(M_TIMER_READ, &Time2);

  /* Halt continuous grab. */
  MdigHalt(MilDigitizer);

  /* Print statistics. */
  AccTime = Time2 - Time1;
  printf("%ld frames grabbed, at a VM transfert rate of %.2f frames/sec (%.2f ms/frame).\n",
         nbFrame, nbFrame/AccTime, 1000.0*AccTime/nbFrame);

  printf("Press <Enter> to end.\n");
  getchar();

  /* Free allocations */
  MbufFree(MilImage);
  MbufFree(MilImageGen[0]);
  MbufFree(MilImageGen[1]);
  MbufFree(MilImageDispGen);
  MdigFree(MilDigitizer);
  MdigFree(MilDigitizerGen);
  MdispFree(MilDisplay);
  MdispFree(MilDisplayGen);
  MsysFree(MilSystem);
  MsysFree(MilSystemGen);
  MappFree(MilApplication);
}

/* --------------------------------------------------------------- */
/* Name:      MyOverlayDraw
 * Synopsis:  This function enables and writes graphics into the
 *            overlay buffer. 
 */            
void MyOverlayDraw(MIL_ID MilSystem, MIL_ID MilDisplay, long WindowedMode)
{
   MIL_ID         MilOverlayImage;
   long           ImageWidth, ImageHeight;
   long           TransparentColor;

   /* Prepare overlay buffer. */
   /***************************/

   /* Disable overlay display while preparing the data. */
   MdispControl(MilDisplay, M_WINDOW_OVR_SHOW, M_DISABLE);
	
   /* Enable writing Overlay graphics on top of display buffer. */
   MdispControl(MilDisplay, M_WINDOW_OVR_WRITE, M_ENABLE);
	
   /* Inquire the Overlay buffer associated with the displayed buffer. */
   MdispInquire(MilDisplay, M_WINDOW_OVR_BUF_ID, &MilOverlayImage);

   /* Inquire the current keying color. */
   MdispInquire(MilDisplay, M_KEY_COLOR, &TransparentColor);

   /* Clear the overlay buffer with the keying color. */
   MbufClear(MilOverlayImage, TransparentColor); 
   
   /* Enable the overlay display. */
   MdispControl(MilDisplay, M_WINDOW_OVR_SHOW, M_ENABLE);
   
   /* Draw MIL monochrome overlay annotation */
   /******************************************/

   /* Inquire MilOverlayImage size x and y. */
   ImageWidth  = MbufInquire(MilOverlayImage,M_SIZE_X,M_NULL);
   ImageHeight = MbufInquire(MilOverlayImage,M_SIZE_Y,M_NULL);
      
   /* Set graphic text to transparent background. */
   MgraControl(M_DEFAULT, M_BACKGROUND_MODE, M_TRANSPARENT);

   /* Set drawing color to white. */
   MgraColor(M_DEFAULT, 255);

   /* Print a string in the overlay image buffer. */
   MgraText(M_DEFAULT, MilOverlayImage, ImageWidth/9, ImageWidth/6,    " -------------------- ");
   MgraText(M_DEFAULT, MilOverlayImage, ImageWidth/9, ImageWidth/6+25, " - MIL Overlay Text - ");
   MgraText(M_DEFAULT, MilOverlayImage, ImageWidth/9, ImageWidth/6+50, " -------------------- ");
   
   /* Draw MIL color overlay annotation */
   /*************************************/

   /* If the overlay buffer is monochrome, associate a pseudo-color */
   /* LUT to the overlay display and draw color annotations.        */
   if (MbufInquire(MilOverlayImage, M_SIZE_BAND, M_NULL) == 1)
      {
      /* Local variables and arrays. */
      MIL_ID         MILLut, MILLutChildLow, MILLutChildHigh;
      
      /* LUT data for PSEUDO-COLOR:          R    G    B     Entry:    Color: */
      static unsigned char LutLow[30] =  {   0,   0,   0, // Index 0 = black 
					   128,   0,   0, // Index 1 = dark-red
					     0, 128,   0, // Index 2 = dark-green
					   128, 128,   0, // Index 3 = dark-yellow
					     0,   0, 128, // Index 4 = dark-blue
					   128,   0, 128, // Index 5 = dark-magenta
					     0, 128, 128, // Index 6 = dark-cyan
					   192, 192, 192, // Index 7 = bright-grey
					   192, 220, 192, // Index 8 = lite-green
					   166, 202, 240  // Index 9 = lite-blue
					 };
      static unsigned char LutHigh[30] = { 255, 251, 240, // Index 246 = lite-white
					   160, 160, 164, // Index 247 = lite-grey
					   128, 128, 128, // Index 248 = dark-grey
					   255,   0,   0, // Index 249 = red
					     0, 255,   0, // Index 250 = green
					   255, 255,   0, // Index 251 = yellow
					     0,   0, 255, // Index 252 = blue
					   255,   0, 255, // Index 253 = magenta
					     0, 255, 255, // Index 254 = cyan
					   255, 255, 255  // Index 255 = white
					 };

      /* Create a grey scale LUT with pseudo-colors in the 2 extremes. */
      /* This LUT match the Windows default palette in 8 bit VGA mode  */
      /* and give the best results for grayscale image display with    */
      /* color anotations.                                             */
      MbufAllocColor(MilSystem, 3L, 256L, 1L, 8L+M_UNSIGNED, M_LUT, &MILLut);
      MbufChild2d(MILLut, 0,   0, 10,  1, &MILLutChildLow);
      MbufChild2d(MILLut, 246, 0, 10,  1, &MILLutChildHigh);
      MgenLutRamp(MILLut, 0,   0, 255, 255);
      MbufPutColor(MILLutChildLow,  M_RGB24+M_PACKED, M_ALL_BAND, &LutLow);
      MbufPutColor(MILLutChildHigh, M_RGB24+M_PACKED, M_ALL_BAND, &LutHigh);
      
      #if !M_MIL_USE_DOS_32
      /* Insert the transparent color in the LUT */
      {
      PALETTEENTRY   PalEntry;
      HDC            hDC;
      COLORREF       ColRef;
      MIL_ID         MILLutChildKey;

      hDC = GetDC(NULL);
      GetSystemPaletteEntries(hDC, TransparentColor, 1, &PalEntry);
      ColRef = RGB(PalEntry.peRed, PalEntry.peGreen, PalEntry.peBlue);
      MbufChild1d(MILLut, TransparentColor, 1, &MILLutChildKey);
      MbufPutColor(MILLutChildKey, (M_RGB24 | M_PACKED), M_ALL_BAND, &ColRef);
      ReleaseDC(NULL, hDC);

      MbufFree(MILLutChildKey);
      }
      #endif

      /* Associate the pseudo-color LUT to the overlay display. */
      MdispControl(MilDisplay, M_WINDOW_OVR_LUT, MILLut);

      /* Free the LUT. */
      MbufFree(MILLutChildHigh);
      MbufFree(MILLutChildLow);
      MbufFree(MILLut);
      
      /* Set drawing color to the green index in the pseudo-color LUT. */
      MgraColor(M_DEFAULT, 250);

      /* Print a string in the overlay image buffer. */
      MgraText(M_DEFAULT, MilOverlayImage, ImageWidth*11/18, ImageWidth/6,    " -------------------- ");
      MgraText(M_DEFAULT, MilOverlayImage, ImageWidth*11/18, ImageWidth/6+25, " - MIL Overlay Text - ");
      MgraText(M_DEFAULT, MilOverlayImage, ImageWidth*11/18, ImageWidth/6+50, " -------------------- ");
      }
   else    
   /* If the overlay buffer is color, draw annotations only in the */
   /* selected color band to create color annotation.              */
      {
      /* Local variable. */
      MIL_ID MilOverlayImageGreen;

      /* Create a child buffer on the green component for drawing. */
      MbufChildColor(MilOverlayImage, M_GREEN , &MilOverlayImageGreen);
      
      /* Set drawing color to maximum intensity for green. */
      MgraColor(M_DEFAULT, 250);

      /* Print a string in the green component overlay image buffer. */
      MgraText(M_DEFAULT, MilOverlayImageGreen, ImageWidth*11/18, ImageWidth/6,    " -------------------------- ");
      MgraText(M_DEFAULT, MilOverlayImageGreen, ImageWidth*11/18, ImageWidth/6+25, " - MIL Overlay Text - ");
      MgraText(M_DEFAULT, MilOverlayImageGreen, ImageWidth*11/18, ImageWidth/6+50, " -------------------------- ");
      
      /* Free the child buffer. */
      MbufFree(MilOverlayImageGreen);
      }
   

   #if !M_MIL_USE_DOS_32
   /* Draw GDI color overlay annotation */
   /*************************************/
   if (WindowedMode)
   {
      HDC   hCustomDC;
      HPEN  hpen, hpenOld;
      char  chText[80];

      /* Create a device context to draw in the overlay buffer with GDI. */
      MbufControl(MilOverlayImage, M_WINDOW_DC_ALLOC, M_DEFAULT);

      /* Inquire the device context. */
      hCustomDC = ((HDC)MbufInquire(MilOverlayImage, M_WINDOW_DC, M_NULL));
      if (hCustomDC)
      {

	 /* Create a blue pen. */
	 hpen=CreatePen(PS_SOLID, 1, RGB(0, 0, 255)); 
	 hpenOld = (HPEN)(SelectObject(hCustomDC,hpen));
		
	 /* Draw a cross in the overlay buffer. */
	 MoveToEx(hCustomDC,0,ImageHeight/2,NULL);
	 LineTo(hCustomDC,ImageWidth,ImageHeight/2);
	 MoveToEx(hCustomDC,ImageWidth/2,0,NULL);
	 LineTo(hCustomDC,ImageWidth/2,ImageHeight);
	 
	 /* Write text in the overlay buffer. */
	 strcpy(chText, "GDI Overlay Text ");
	 SetTextColor(hCustomDC,RGB(0, 0, 255));
	 TextOut(hCustomDC,ImageWidth*3/18,ImageHeight*4/6, chText, strlen(chText));
	 SetTextColor(hCustomDC,RGB(255, 0, 0));
	 TextOut(hCustomDC,ImageWidth*12/18,ImageHeight*4/6, chText, strlen(chText));
	 
	 /* Deselect and destroy the blue pen. */
	 SelectObject(hCustomDC,hpenOld);
	 DeleteObject(hpen);
      }
      
      /* Delete created device context. */
      MbufControl(MilOverlayImage, M_WINDOW_DC_FREE, M_DEFAULT);
      
      /* Signal MIL that the overlay buffer was modified. */
      MbufControl(MilOverlayImage, M_MODIFIED, M_DEFAULT);
   }
   #endif
}

