/* File name: mcolor.c
 * Synopsis:  This program allocates a displayable color image buffer,
 *            displays it and loads its contents with a color image.
 *            It then do a copy of this image and write text into the
 *            color components (RGB) of the copy of the image.
 */

#include <stdio.h> 
#include <stdlib.h> 
#include <mil.h> 

/* Source MIL image file specifications. */ 
#define IMAGE_FILE             M_IMAGE_PATH"bird.mim"
#define IMAGE_WIDTH            256L
#define IMAGE_HEIGHT           240L
#define IMAGE_BAND               3L
#define IMAGE_DEPTH              8L

void main(void)
{ 
   MIL_ID MilApplication,   /* Application identifier.  */
          MilSystem,        /* System identifier.       */
          MilDisplay,       /* Display identifier.      */
          MilImage,         /* Image buffer identifier. */
          MilSubImage0,     /* Sub-image buffer identifier for source image.   */
          MilSubImage1,     /* Sub-image buffer identifier for copied image.   */
          MilSubImage1Red,  /* Sub-image buffer identifier for red component.  */
          MilSubImage1Green,/* Sub-image buffer identifier for green component.*/
          MilSubImage1Blue; /* Sub-image buffer identifier for blue component. */

   /* Allocate defaults. */
   MappAllocDefault(M_SETUP, &MilApplication, &MilSystem, &MilDisplay,
                    M_NULL, M_NULL);

   /* Allocate a color display image buffer to perform processing in it. */
   MbufAllocColor(MilSystem, IMAGE_BAND, IMAGE_WIDTH * 2, IMAGE_HEIGHT,
                  IMAGE_DEPTH+M_UNSIGNED, M_IMAGE+M_DISP+M_PROC, &MilImage);
  
   /* Clear the image buffer. */
   MbufClear(MilImage, 0L);
      
   /* Display the image buffer. */
   MdispSelect(MilDisplay, MilImage);
      
   /* Define 2 sub-image buffers in the display buffer, restricting the
      work regions to the image size.
    */
   MbufChild2d(MilImage, 0L, 0L, IMAGE_WIDTH, IMAGE_HEIGHT, 
               &MilSubImage0);
   MbufChild2d(MilImage, IMAGE_WIDTH, 0L, IMAGE_WIDTH, IMAGE_HEIGHT,
               &MilSubImage1);

   /* Load a color image in image 0. */
   MbufLoad(IMAGE_FILE, MilSubImage0);
      
   /* Print a message. */
   printf("A color source image was loaded and displayed.\n");
   printf("Press <Enter> to continue.\n");
   getchar();

   /* Copy the color image. */
   MbufCopy(MilSubImage0, MilSubImage1);

   /* Do child buffers that map to the red, green and blue components. */
   MbufChildColor(MilSubImage1, M_RED,   &MilSubImage1Red);
   MbufChildColor(MilSubImage1, M_GREEN, &MilSubImage1Green);
   MbufChildColor(MilSubImage1, M_BLUE,  &MilSubImage1Blue);
    
   /* Write color annotations in each component of the copied image. */
   MgraColor(M_DEFAULT, 0xFF);
   MgraText(M_DEFAULT, MilSubImage1Red, IMAGE_WIDTH/16, IMAGE_HEIGHT/8, 
            "TOUCAN");
   MgraColor(M_DEFAULT, 0x80);
   MgraText(M_DEFAULT, MilSubImage1Green, IMAGE_WIDTH/16, IMAGE_HEIGHT/8, 
            "TOUCAN");
   MgraColor(M_DEFAULT, 0x00);
   MgraText(M_DEFAULT, MilSubImage1Blue, IMAGE_WIDTH/16, IMAGE_HEIGHT/8, 
            "TOUCAN");

   /* Print a message. */
   printf("The color source image in the top left corner was copied in the\n");
   printf("top right corner image and color text annotation was done in it.\n");
   printf("Press <Enter> to end.\n");
   getchar();

   /* Release subimages and color image buffer. */
   MbufFree(MilSubImage1Red);
   MbufFree(MilSubImage1Green);
   MbufFree(MilSubImage1Blue);
   MbufFree(MilSubImage1);
   MbufFree(MilSubImage0);
   MbufFree(MilImage);

   /* Release defaults. */
   MappFreeDefault(MilApplication, MilSystem, MilDisplay, M_NULL, M_NULL);
}

