/* File name: mmet2dig.c
 * Synopsis:  This program allocates 2 grab buffers on the 
 *            on board memory of the meteor2/Dig and then 
 *            grabs in them.
 *            The data is then transfert to the HOST using
 *            the bus-master copy.
 *
 *            This technique can be used when experiencing
 *            grab over-runs on heavy loaded systems.
 *
 /

/* headers */
#include <stdio.h> 
#include <stdlib.h> 
#include <conio.h> 
#include <mil.h> 

/* Main function. */
void main(void)
{ 
  MIL_ID   MilApplication;
  MIL_ID   MilSystem     ;
  MIL_ID   MilDigitizer  ;
  MIL_ID   MilDisplay    ;
  MIL_ID   MilImageOnBoard[2];
  MIL_ID   MilImageDisp;

  // Allocations.
  MappAlloc(M_DEFAULT, &MilApplication);
  MsysAlloc(M_SYSTEM_METEOR_II_DIG, M_DEF_SYSTEM_NUM, M_SETUP, &MilSystem);
  MdigAlloc(MilSystem, M_DEFAULT, M_DEF_DIGITIZER_FORMAT,
                                  M_DEFAULT, &MilDigitizer);
  MdispAlloc(MilSystem, M_DEFAULT, M_DEF_DISPLAY_FORMAT, M_DEFAULT, &MilDisplay);

  /* Allocate a display buffer. */
  MbufAlloc2d(MilSystem,  
              (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)),
              (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)),
              8L+M_UNSIGNED, 
              M_IMAGE+M_PROC+M_DISP+M_NON_PAGED, &MilImageDisp);
  MbufClear(MilImageDisp, 0);

  /* Allocate 2 grab buffers on the on-board memory of the board. */
  MbufAlloc2d(MilSystem,  
              (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)),
              (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)),
              8L+M_UNSIGNED, 
              M_IMAGE+M_GRAB+M_PROC+M_ON_BOARD, &MilImageOnBoard[0]);

  MbufAlloc2d(MilSystem,  
              (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)),
              (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)),
              8L+M_UNSIGNED, 
              M_IMAGE+M_GRAB+M_PROC+M_ON_BOARD, &MilImageOnBoard[1]);

  
  /* Enable bus master copies from the Meteor2 to the host. */
  MsysControl(MilSystem, M_BUS_MASTER_COPY_TO_HOST, M_ENABLE);

  /* Display the buffer. */
  MdispSelect(MilDisplay, MilImageDisp);

  printf("Press enter to grab into the first buffer\n");
  getchar();

  /* Grab and copy to the host. */
  MdigGrab(MilDigitizer, MilImageOnBoard[0]);
  MbufCopy(MilImageOnBoard[0], MilImageDisp);

  printf("Press enter to grab into the second buffer\n");
  getchar();

  /* Grab and copy to the host. */
  MdigGrab(MilDigitizer, MilImageOnBoard[1]);
  MbufCopy(MilImageOnBoard[1], MilImageDisp);

  getchar();

  /* Free allocations. */
  MbufFree(MilImageOnBoard[0]);
  MbufFree(MilImageOnBoard[1]);
  MbufFree(MilImageDisp);
  MdispFree(MilDisplay);
  MdigFree(MilDigitizer);
  MsysFree(MilSystem);
  MappFree(MilApplication);
}  
