/**************************************************************************
 This example grab a sequence of compress images and saves them on disk.
 The sequence is then read from disk when replayed.

 Note: 
 This program uses a frame buffering stategie for acheving realtime 
 compression on the disk.  A grab-thread is created for grabing in 
 a circle array of buffers while the main thread saves the grabbed images.

 This program is optimized for software/hardware decompression with 
 the MeteorII family used with the Matrox G200/G400 Vga or the Matrox 4Sight. 

 This example needs 8 Megs of non-paged memory.  

/**************************************************************************/

/* headers */
#include <stdio.h> 
#include <stdlib.h> 
#include <conio.h> 
#include <mil.h>
#include <process.h>
#include <windows.h>

/* Number of images to compress in the sequence. */
#define NB_GRAB       200

/* Number of buffers allocated for the buffering. */
#define NB_BUFFERING_SIZE 25

/* Compression quality factor: 1->best quality, 99->worst. */
#define QFACTOR       50

/* Name of the file to save the images. */
#define FILE_NAME  "Sequence.dat"

/* Set to 1 for JPEG(non-interlaced) Set to 0 for MJPEG(interlaced). */
/* Note: Can not hardware decompress JPEG format on the Meteor2-STD. */
#define COMPRESSION_FORMAT   0

/* Increases the process priority.
   This can be used when the compression or the decompression is not
   realtime when using high quality compression.
*/
#define INCREASE_PROCESS_PRIORITY 0

/* Grabbing thread. */
unsigned long MFTYPE GrabThread(void *TParam);

/* Hook data structure. */
typedef struct
   {
   MIL_ID  *MilImage; 
   MIL_ID   MilDigitizer;
   MIL_ID   MilSystem;
   long     NbGrab;
   long     GrabIndex;
   long     NbSaved;
   long     SavedIndex;
   } UserDataStruct;

/* Main function. */
void main(void)
{ 
   MIL_ID   MilApplication;
   MIL_ID   MilSystem     ;
   MIL_ID   MilDigitizer  ;
   MIL_ID   MilDisplay    ;
   MIL_ID   MilImage[NB_GRAB];
   MIL_ID   MilImageDisp;
   MIL_ID   MilImageYUV16Disp;
   MIL_ID   MilImageDecomp;
   MIL_ID   MilImageCreateId;
   void     *MilImageCreatePtr;
   FILE     *JpegFile;
   double   Time;
   long     ExtendedAttrib = 0;
   long     SizeBand, SizeX, SizeY;
   UserDataStruct volatile UserStruct;
   long volatile  DoSomething=0;
   DWORD  ThreadId;
   long     n;
           
   /* Allocations  */
   MappAlloc(M_DEFAULT, &MilApplication);
   MsysAlloc(M_DEF_SYSTEM_TYPE, M_DEF_SYSTEM_NUM, M_SETUP, &MilSystem);
   MdispAlloc(MilSystem, M_DEFAULT, M_DEF_DISPLAY_FORMAT,
                         M_DEFAULT, &MilDisplay);
   MdigAlloc(MilSystem, M_DEF_DIGITIZER_NUM, M_DEF_DIGITIZER_FORMAT,
                        M_DEFAULT, &MilDigitizer);

   /* Set the digitizer to asynchronous. */
   MdigControl(MilDigitizer, M_GRAB_MODE, M_ASYNCHRONOUS);
   SizeBand = MdigInquire(MilDigitizer, M_SIZE_BAND, M_NULL);
   SizeX    = MdigInquire(MilDigitizer, M_SIZE_X, M_NULL);
   SizeY    = MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL);

   #if(COMPRESSION_FORMAT)
   ExtendedAttrib = M_JPEG_LOSSY;
   #else
   ExtendedAttrib = M_JPEG_LOSSY_INTERLACED;
   #endif

   /* Buffer allocations. */
   MbufAllocColor(MilSystem, SizeBand, SizeX, SizeY, 
                  8L+M_UNSIGNED, M_IMAGE+M_GRAB+M_PROC+M_DISP,&MilImageDisp);

   MbufAllocColor(MilSystem, 3, SizeX, SizeY, 8L+M_UNSIGNED, 
                  M_IMAGE+M_GRAB+M_PROC+M_DISP+M_YUV16+M_PACKED, &MilImageYUV16Disp);

   for (n=0; n<NB_BUFFERING_SIZE; n++)
      {
      MbufAllocColor(MilSystem, SizeBand, SizeX, SizeY, 8L+M_UNSIGNED,
                     M_IMAGE|M_GRAB|M_COMPRESS|ExtendedAttrib,
                     &MilImage[n]);

      MbufControl(MilImage[n], M_Q_FACTOR, QFACTOR);
      }

   /* Allocate some memory and a buffer for the decompression. */
   MilImageCreatePtr = malloc(SizeBand*SizeX*SizeY);
   MbufCreateColor(MilSystem, SizeBand, SizeX, SizeY, 
                   8L+M_UNSIGNED, 
                   M_IMAGE|M_COMPRESS|ExtendedAttrib,
                   M_HOST_ADDRESS+M_PITCH,
                   SizeX,
                   &MilImageCreatePtr,
                   &MilImageCreateId);

   /* Display activation. */
   MbufClear(MilImageDisp, 0x0);
   MdispSelect(MilDisplay, MilImageDisp);
  
   /* Initialize hook structure. */
   memset( (void *)(&UserStruct), 0, sizeof( UserStruct ) );
   UserStruct.MilSystem       = MilSystem;
   UserStruct.MilDigitizer    = MilDigitizer;
   UserStruct.MilImage        = MilImage;

   /* Grab live on display. */
   MdigGrabContinuous(MilDigitizer, MilImageDisp);
  
   /* Print a message and stop the grab. */
   printf("\nPress <Enter> to start the compression of %d frames.\n",NB_GRAB);

   getchar();
   MdigHalt(MilDigitizer);
  
   #if INCREASE_PROCESS_PRIORITY 
   SetPriorityClass(GetCurrentProcess(), REALTIME_PRIORITY_CLASS);
   #endif

   /* Start the grabbing thread and sequence. */
   _beginthreadex(NULL, 0L, &GrabThread, (void *)(&UserStruct), 0L, &ThreadId);

   /* Open the file for saving the images. */
   if( (JpegFile = fopen( FILE_NAME, "w+b" )) == NULL)
      {
      printf("\nError: Unable to open file: %s\n",FILE_NAME);
      return;
      }

   /* Synchronize the timer. */
   while (UserStruct.NbGrab == 0)
      DoSomething=DoSomething;
   MappTimer(M_TIMER_RESET, M_NULL);

   /* While the frames are being grabbed in the thread, we are saving them on disk. */
   while (UserStruct.NbSaved < NB_GRAB)
      {
      /* Save the grabbed images on disk. */
      if ((UserStruct.NbSaved < (UserStruct.NbGrab-2)) || 
          (UserStruct.NbGrab == NB_GRAB))
         {
         long SizeOfBufferingZone;
         long SizeByte  = MbufInquire(MilImage[UserStruct.SavedIndex],M_SIZE_BYTE,0);
         long *ImagePtr = (long *) MbufInquire(MilImage[UserStruct.SavedIndex],
                                               M_HOST_ADDRESS,0);

         /* Save the image size them the compressed data. */
         fwrite( &SizeByte, sizeof( long ), 1, JpegFile);
         if (fwrite( ImagePtr, sizeof( char ), SizeByte, JpegFile) != (size_t)SizeByte)
            {
            printf("\nError: Unable to save on disk. Disk full?\n");
            return;
            }

         /* Update indexes and check for wrap around. */
         UserStruct.NbSaved++;
         UserStruct.SavedIndex++;
         if (UserStruct.SavedIndex > (NB_BUFFERING_SIZE-1))
            UserStruct.SavedIndex = 0;

         /* Calculate buffering size and print message. */
         SizeOfBufferingZone = ((UserStruct.GrabIndex >= UserStruct.SavedIndex) ?
                                (UserStruct.GrabIndex - UserStruct.SavedIndex):
                                ((UserStruct.GrabIndex + NB_BUFFERING_SIZE) - UserStruct.SavedIndex));

         printf("\rFrame #: %d/%d    Buffering load: %d%%      ",
                 UserStruct.NbGrab,NB_GRAB,
                 (long) ((double)(SizeOfBufferingZone)*100/NB_BUFFERING_SIZE));
         }
      else
         {
         /* Nothing to save, give timeslice back to the grab thread. */
         Sleep(5);
         }
      }

   /* Close the file and read the timer. */
   fclose(JpegFile);
   MappTimer(M_TIMER_READ, &Time);

   /* Print final statistics. */
   printf("\n\n%ld frames grabbed and saved at a rate of %.2f frames/sec (%.2f msec/frame).\n\n",
          UserStruct.NbGrab, UserStruct.NbGrab/Time, 1000.0*Time/UserStruct.NbGrab);

   /** At this point the images are grabbed and saved on disk. 
       We will now start the playback. **/

   /* Use a YUV16 packed buffer to optimize the JPEG decompression when 
      the compress buffer is in color. */
   MilImageDecomp = MilImageDisp;
   if (SizeBand == 3)
      {
      /* Reduces display flashing. */
      MbufCopy(MilImageDisp, MilImageYUV16Disp);

      MdispSelect(MilDisplay, MilImageYUV16Disp);
      MilImageDecomp = MilImageYUV16Disp;

      /* When using hardware decompression, decompress directly in the VGA 
         displayable buffer when it is in YUV16 packed format. */
      if (MsysInquire(MilSystem, M_HARDWARE_DECOMPRESSION, M_NULL) == M_ENABLE)
         {
         MIL_ID MilWindowBufId;
         MdispControl(MilDisplay, M_WINDOW_BUF_WRITE, M_ENABLE);
         MdispInquire(MilDisplay, M_WINDOW_BUF_ID, &MilWindowBufId);
         if ((MilWindowBufId) && 
             (MbufInquire(MilWindowBufId, M_FORMAT, M_NULL) & M_INTERNAL_FORMAT) == (M_YUV16_YUYV))
            MilImageDecomp = MilWindowBufId;
         }
      }

   /* Open the file for the playback. */
   if( (JpegFile = fopen( FILE_NAME, "r+bc" )) == NULL)
      {
      printf("Unable to open file: %s\n",FILE_NAME);
      return;
      }

   /* Reset the timer for the playback. */
   MappTimer(M_TIMER_RESET, &Time);   
   n=0;
   while(!kbhit())
      {
      long SizeByte = 0;

      /* First read the size of the compressed image. */
      fread( &SizeByte, sizeof( long ), 1, JpegFile );

      /* Next read the compressed image directly in the memory
         of the buffer allocated with MbufCreateColor(). */
      fread( MilImageCreatePtr, sizeof( char ), SizeByte, JpegFile );

      /* Notify MIL that the buffer has been changed
         and set the new size byte. */
      MbufControl(MilImageCreateId, M_SIZE_BYTE, SizeByte);
      MbufControl(MilImageCreateId, M_MODIFIED, M_DEFAULT);

      /* Decompress the image and increase the counter. */
      /* When using hardware decompression, this command is 
         asynchronous. */
      MbufCopy(MilImageCreateId, MilImageDecomp);
      n++;
      
      /* Print frame rate. */
      MappTimer(M_TIMER_READ, &Time);
      printf("\rReplaying frame %d at %0.1f frames/sec     ",n, n/Time);
      if (n > UserStruct.NbGrab-2)
         { 
         n = 0;
         MappTimer(M_TIMER_RESET, &Time);
         fseek( JpegFile, 0, SEEK_SET);
         }
      }

   /* Put back the priority to normal. */
   #if INCREASE_PROCESS_PRIORITY 
   SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
   #endif

   //Retreive keystroke left in buffer by kbhit()
   getchar();

   /* Free allocations. */
   MbufFree(MilImageCreateId);
   free(MilImageCreatePtr);
   MbufFree(MilImageDisp);
   MbufFree(MilImageYUV16Disp);
   for (n=0; n<NB_BUFFERING_SIZE; n++)
      {
      MbufFree(MilImage[n]);
      }
   MdispFree(MilDisplay);
   MdigFree(MilDigitizer);
   MsysFree(MilSystem);
   MappFree(MilApplication);
}  


/* Grab hook function. */
unsigned long MFTYPE GrabThread(void *TParam)
{
   UserDataStruct volatile *UDSPtr=(UserDataStruct*)TParam;

   /* Give higher priority to this thread to make sure we dont 
      miss any frames. */
   SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);

   /* Grab until we reach the number of frames to grab. */
   while (UDSPtr->NbGrab < NB_GRAB)
      {
      /* Check for grab-save overflow. 
         This appends when it can not save as fast then the grabs. */
      if ((UDSPtr->NbGrab - UDSPtr->NbSaved) > (NB_BUFFERING_SIZE-2))
         {
         printf("\nSave overflow. Saved: %04d  Grabbed: %04d\n",UDSPtr->NbSaved,UDSPtr->NbGrab);
         Sleep(200);
         }
      else
         {
         if (UDSPtr->GrabIndex > (NB_BUFFERING_SIZE-1))
            {
            UDSPtr->GrabIndex = 0;
            }

         MdigGrab(UDSPtr->MilDigitizer, UDSPtr->MilImage[UDSPtr->GrabIndex]);

         UDSPtr->NbGrab++;
         UDSPtr->GrabIndex++;
         }
      }

   return(0);
}

