/* File name: mnatgen.c 
 * Synopsis:  This program show how to use GENESIS native library mixed
 *            with MIL calls.
 */

/* general includes */
#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 
#include <mil.h>
#include <imapi.h>

/* Operation control defines */
#define ALLOCATE 1
#define PROCESS  2
#define FREE     3

/* Native functions to grab  and warp an image. */
void GrabAndWarp(MIL_ID MilSystem, MIL_ID MilDisplay, MIL_ID MilCamera,
                                   MIL_ID MilImage, long Operation);

/* Main function: */
/* -------------- */
void main(void)
{ 
   MIL_ID MilApplication,  /* Application identifier.  */
          MilSystem,       /* System identifier.       */
          MilDisplay,      /* Display identifier.      */
          MilCamera,       /* Camera identifier.       */ 
          MilImage;        /* Image buffer identifier. */

   /* Allocate defaults. */
   MappAllocDefault(M_SETUP, &MilApplication, &MilSystem,
                             &MilDisplay, &MilCamera, &MilImage);
   /* Make sure that the system allocated is a Genesis */                             
   if ((MsysInquire(MilSystem, M_SYSTEM_TYPE, M_NULL)!= M_SYSTEM_GENESIS_TYPE) ||
       ((MsysInquire(MilSystem, M_BOARD_TYPE, M_NULL)!= M_GENESIS) &&
        (MsysInquire(MilSystem, M_BOARD_TYPE, M_NULL)!= M_GENESIS_WITH_DAC_MODULE))) 
      {
      printf("This example only runs on a Matrox Genesis main board system.\n");
      goto end;
      }
                    
   /* Allocate and initialialize work buffers */
   GrabAndWarp(MilSystem, MilDisplay, MilCamera, MilImage, ALLOCATE);
      
   /* Print a message on the host screen. */
   printf("Native function called in a loop...\n");
   printf("Press <Enter> to end...");
   
   /* Grab and warp grabbed image in a loop */
   while (!kbhit())
      {
      GrabAndWarp(MilSystem, MilDisplay, MilCamera, MilImage, PROCESS);
      }
   getchar();
      
   /* Free work buffers */
   GrabAndWarp(MilSystem, MilDisplay, MilCamera, MilImage, FREE);

end:   
   
   MappFreeDefault(MilApplication, MilSystem, MilDisplay, MilCamera, MilImage);
}


/* Native function: */
/* ---------------- */
void GrabAndWarp(MIL_ID MilSystem, MIL_ID MilDisplay, MIL_ID MilCamera,
                                   MIL_ID MilImage, long Operation)
{
   /* Warp coefficients and LUT ID variables.
    * (kept in static to avoid warp coeficients caculation at each call.)
    */
   static long NativeWarpBufId       = M_NULL;
   static long NativeWarpLutXBufId   = M_NULL;
   static long NativeWarpLutYBufId   = M_NULL;
   static long NativeGrabBufId       = M_NULL;
   static long NativeWarpResultBufId = M_NULL;

   
   /* Inquire useful MIL informations. */
   long SizeX    = MdigInquire(MilCamera, M_SIZE_X   , M_NULL);
   long SizeY    = MdigInquire(MilCamera, M_SIZE_Y   , M_NULL);
   long SizeBand = MdigInquire(MilCamera, M_SIZE_BAND, M_NULL);
   
   /* Miscelanous local variables */
   double CornerX1  = 0.0;
   double CornerY1  = 0.0;
   double CornerX2  = SizeX - 1.0;
   double CornerY2  = 0.0;
   double CornerX3  = 2.0 * SizeX;
   double CornerY3  = SizeY - 1.0;
   double CornerX4  = -1.0 * SizeX;
   double CornerY4  = SizeY - 1.0;
   long   SrcXStart = 0L;
   long   SrcYStart = 0L;
   long   SrcXEnd   = SizeX - 1L;
   long   SrcYEnd   = SizeY - 1L;
   
   /* Inquire Genesis native Id's */
   long NativeSysThreadId = MsysInquire(MilSystem, M_NATIVE_THREAD_ID,  M_NULL);
   long NativeDigCameraId = MdigInquire(MilCamera, M_NATIVE_CAMERA_ID , M_NULL);
   long NativeDigControlId= MdigInquire(MilCamera, M_NATIVE_CONTROL_ID, M_NULL);
   long NativeDigId       = MdigInquire(MilCamera, M_NATIVE_ID        , M_NULL);
   long NativeBufId       = MbufInquire(MilImage,  M_NATIVE_ID        , M_NULL);

   /* Notify MIL that we are entering native mode. */
   MsysControl(MilSystem, M_NATIVE_MODE_ENTER, M_NULL);
   
   /* Do the selected operation.*/
   switch (Operation)
   {
   /* Preallocate grab and warp buffers (done once for speed). */
   case ALLOCATE:
      {
      imBufAlloc(NativeSysThreadId, SizeX, SizeY, SizeBand, IM_UBYTE, IM_PROC,
                                                      &NativeGrabBufId);
      imBufAlloc(NativeSysThreadId, SizeX, SizeY, SizeBand, IM_UBYTE, IM_PROC,
                                                      &NativeWarpResultBufId);
      imBufAlloc(NativeSysThreadId, 3L, 3L, 1L, IM_FLOAT, IM_PROC,
                                                      &NativeWarpBufId);
      imBufAlloc(NativeSysThreadId, SizeX, SizeY, 1L, IM_SHORT, IM_PROC, 
                                                      &NativeWarpLutXBufId);
      imBufAlloc(NativeSysThreadId, SizeX, SizeY, 1L, IM_SHORT, IM_PROC,
                                                      &NativeWarpLutYBufId);
      if (NativeGrabBufId     && NativeWarpResultBufId && NativeWarpBufId &&
          NativeWarpLutXBufId && NativeWarpLutYBufId)
         {
         /* Calculate warp coeficients */
         imGenWarp4Corner(NativeSysThreadId, NativeWarpBufId, CornerX1,
                          CornerY1, CornerX2, CornerY2, CornerX3, CornerY3,
                          CornerX4, CornerY4, SrcXStart, SrcYStart, SrcXEnd,
                          SrcYEnd, IM_DEFAULT, 0L);
         imGenWarpLutMatrix(NativeSysThreadId, NativeWarpLutXBufId,
                            NativeWarpLutYBufId, NativeWarpBufId, 0L, 0L);
         }
      else  
         {
         printf("Error allocating resources...\n");
         }

      if (SizeBand > 1)
         {
         printf("Error: only monochrome cameras are supported...\n");
         }

      break;   
      }
     
   /* Grab and Warp buffer. */
   case PROCESS:
      {
      /* Process if allocations were sucessful */
      if ((SizeBand == 1)     &&
          NativeGrabBufId     && NativeWarpResultBufId && NativeWarpBufId &&
          NativeWarpLutXBufId && NativeWarpLutYBufId)
         {
         /* Grab the image */
         imDigGrab(NativeSysThreadId, NativeDigId, NativeDigCameraId,
                   NativeGrabBufId, 1L, NativeDigControlId, 0L);

         /* Warp the grabbed image. */
         imIntWarpLut(NativeSysThreadId, NativeGrabBufId, NativeWarpResultBufId,
                      NativeWarpLutXBufId, NativeWarpLutYBufId, 0L, 0L);

         /* Copy the result into the display buffer */
         imBufCopy(NativeSysThreadId, NativeWarpResultBufId, NativeBufId, 0L, 0L);
         }
      break;   
      }
     
   /* Free grab and warp buffers. */
   case FREE:
      {
      if (NativeGrabBufId)
         imBufFree(NativeSysThreadId, NativeGrabBufId);
      if (NativeWarpResultBufId)
         imBufFree(NativeSysThreadId, NativeWarpResultBufId);
      if (NativeWarpBufId)
         imBufFree(NativeSysThreadId, NativeWarpBufId);
      if (NativeWarpLutXBufId)
         imBufFree(NativeSysThreadId, NativeWarpLutXBufId);
      if (NativeWarpLutYBufId)
         imBufFree(NativeSysThreadId, NativeWarpLutYBufId);
      break;   
      }
   }
   
   /* Notify MIL that we leave native mode. */
   MsysControl(MilSystem, M_NATIVE_MODE_LEAVE, M_NULL);
   
   /* Notify MIL that the buffer was modified. */
   MbufControl(MilImage, M_MODIFIED, M_DEFAULT);
}
