//==============================================================================
//
// Filename:  ORION.H
// Owner   :  Matrox Imaging dept.
// Rev     :
// Content :  This file contains the new defines that are needed by the user
//            to use the MIL current library with the ORION.
//
// COPYRIGHT (c) Matrox Electronic Systems Ltd.
// All Rights Reserved
//==============================================================================

#ifndef __MIL_H
#include "mil.h"
#endif

#ifndef __ORION_H
#define __ORION_H

//==============================================================================
//                    ORION DRIVER RELEASE (5.00)
//
//    These are the specifics or not yet released defines for the ORION
//==============================================================================
#define M_IRQ_GLOBAL_OBJECT           125L

//==============================================================================
// PCI device information
//==============================================================================
#define M_PCI_VENDOR_ID             0x00 // (16 lsb)
#define M_PCI_DEVICE_ID             0x00 // (16 msb)
#define M_PCI_COMMAND               0x01 // (16 lsb)
#define M_PCI_STATUS                0x01 // (16 msb)
#define M_PCI_REVISION_ID           0x02 // (byte 0)
#define M_PCI_CLASS_CODE            0x02 // (byte 1,2,3)
#define M_PCI_LATENCY_TIMER         0x03 // (byte 1)
#define M_PCI_BASE_ADRS0            0x04 // 
#define M_PCI_BASE_ADRS1            0x05 // 
#define M_PCI_INT_LINE              0x0F // (byte 0)
#define M_PCI_INT_PIN               0x0F // (byte 1)

// Orion board type redefinition
#define M_DEVICE_ORION             M_ORION         
#define M_NB_ORION_BOARD_TYPE      1L

// Orion board type redefinition
#define KS0127          0x00000004
#define LUT             0x00000010
#define TOUCAN         0x00000100
#define ENCODER         0x00000800

/* User Hook identification*/
/******************************************************************************/
#define OLL_IRQ_TYPE_GRAB_FIELD_START_RELATED_MASK    0x0000000FL                              
#define OLLISR_START_OF_GRAB_FIELD_BIT                0x00000001L  // When grabbing
#define OLLISR_START_OF_GRAB_FIELD_ODD_BIT            0x00000002L  // When grabbing
#define OLLISR_START_OF_GRAB_FIELD_EVEN_BIT           0x00000004L  // When grabbing
                                                      
#define OLL_IRQ_TYPE_GRAB_FRAME_START_RELATED_MASK    0x000000F0L                              
#define OLLISR_START_OF_GRAB_FRAME_BIT                0x00000010L  // When grabbing
                                                      
#define OLL_IRQ_TYPE_GRAB_FIELD_END_RELATED_MASK      0x00000F00L
#define OLLISR_END_OF_GRAB_FIELD_BIT                  0x00000100L  // When grabbing
#define OLLISR_END_OF_GRAB_FIELD_ODD_BIT              0x00000200L  // When grabbing
#define OLLISR_END_OF_GRAB_FIELD_EVEN_BIT             0x00000400L  // When grabbing

#define OLL_IRQ_TYPE_GRAB_FRAME_END_RELATED_MASK      0x0000F000L
#define OLLISR_END_OF_GRAB_FRAME_BIT                  0x00001000L  // When grabbing

#define OLL_IRQ_TYPE_START_RELATED_MASK               0x000F0000L                              
#define OLLISR_START_OF_FRAME_BIT                     0x00010000L  // without grabbing
#define OLLISR_START_OF_FIELD_BIT                     0x00020000L  // without grabbing
#define OLLISR_START_OF_FIELD_ODD_BIT                 0x00040000L  // without grabbing
#define OLLISR_START_OF_FIELD_EVEN_BIT                0x00080000L  // without grabbing

#define OLL_IRQ_TYPE_GRAB_START_RELATED_MASK          0x00F00000L
#define OLLISR_START_OF_GRAB_BIT                      0x00100000L  // When grabbing
                                                      
#define OLL_IRQ_TYPE_GRAB_END_RELATED_MASK            0x0F000000L
#define OLLISR_END_OF_GRAB_BIT                        0x01000000L  // When grabbing
                                                  
#define OLL_IRQ_TYPE_TIMER_RELATED_MASK               0x20000000L
#define OLLISR_TIMER_IRQ_BIT                          0x20000000L


#define OLLISR_MUST_ALWAYS_BE_ENABLE_MASK             OLL_IRQ_TYPE_GRAB_FIELD_END_RELATED_MASK
                                                   
//==============================================================================
// Internal use only
//==============================================================================
#define M_SYS_DEBUG_ACTION_0        300000L
#define M_SYS_DEBUG_ACTION_1        300001L
#define M_SYS_DEBUG_ACTION_2        300002L
#define M_SYS_DEBUG_ACTION_3        300003L
#define M_SYS_DEBUG_ACTION_4        300004L
#define M_SYS_DEBUG_ACTION_5        300005L
#define M_SYS_DEBUG_ACTION_6        300006L
#define M_SYS_DEBUG_ACTION_7        300007L
#define M_SYS_DEBUG_ACTION_8        300008L
#define M_SYS_DEBUG_ACTION_9        300009L
#define M_SYS_DEBUG_ACTION_10       300010L
#define M_SYS_DEBUG_ACTION_11       300011L
#define M_SYS_DEBUG_ACTION_12       300012L

#define M_SEEPROM_SPACE             M_RESERVED_FOR_INTERNAL_USE_SYS_CONFIG

//==============================================================================
// MIL OPTIONAL COMPILE DEFINES prefixes
//==============================================================================
#if M_MIL_USE_DOS_32
   #define ORIONFARTYPE
#endif

#if M_MIL_USE_WINDOWS
   #define ORIONFARTYPE                      
#endif

//==============================================================================
// ORION Lib prototypes
//==============================================================================
#ifdef __cplusplus
extern "C"
{
#endif

typedef MFTYPE32 long (MFTYPE MPTYPE *OAPPHOOKFCTPTR)(long         FHookType, 
                                                      void MPTYPE *FExpansionFlagPtr);
                                      
MFTYPE32 long MFTYPE 
OappAlloc(                   long            InitFlag, 
                             MIL_ID MPTYPE  *ApplicationID,
                             void MPTYPE    *ThreadContextPtr);
                                     
MFTYPE32 void MFTYPE 
OappControl(                 long            ControlType, 
                             long            ControlFlag,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 long MFTYPE 
OappInquire(                 long            InquireType, 
                             long MPTYPE    *UserVarPtr,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 long MFTYPE 
OappGetError(                long            ErrorType, 
                             void MPTYPE    *UserPtr,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OappFree(                    MIL_ID          ApplicationID,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 long MFTYPE 
OsysAlloc(                   long            SystemType, 
                             long            Number, 
                             long            InitFlag, 
                             MIL_ID MPTYPE  *SystemID,
                             void MPTYPE    *ThreadContextPtr);
                                     
MFTYPE32 long MFTYPE 
OsysInquire(                 MIL_ID          SystemID,
                             long            InquireType, 
                             long MPTYPE    *UserVarPtr,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OsysControl(                 MIL_ID          SystemID, 
                             long            ControlType, 
                             long            ControlFlag,
                             void MPTYPE    *ThreadContextPtr);
                                     
MFTYPE32 void MFTYPE 
OsysConfigAccess(            MIL_ID          SystemID,
                             long            VendorId,      
                             long            DeviceId,      
                             long            DeviceNum,     
                             long            OperationFlag, 
                             long            OperationType, 
                             long            Offset,        
                             long            Size,
                             void           *UserArrayPtr,          
                             void MPTYPE    *ThreadContext);

MFTYPE32 void MFTYPE 
OsysFree(                    MIL_ID          SystemID,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 long MFTYPE 
OdigAlloc(                   MIL_ID          SystemID, 
                             long            DeviceNum, 
                             char MPTYPE    *DataFormat, 
                             long            InitFlag, 
                             MIL_ID MPTYPE  *IdVarPtr,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 void MFTYPE 
OdigFree(                    MIL_ID          DigitizerID,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OdigGrabContinuous(          MIL_ID          DigitizerID, 
                             MIL_ID          BufferID,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OdigGrab(                    MIL_ID          DigitizerID, 
                             MIL_ID          BufferID,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 void MFTYPE 
OdigHalt(                    MIL_ID          DigitizerID,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OdigGrabWait(                MIL_ID          DigitizerID,
                             long            Flag,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 void MFTYPE 
OdigControl(                 MIL_ID          DigitizerID,
                             long            ControlType, 
                             double          ControlValue,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 long MFTYPE 
OdigInquire(                 MIL_ID          DigitizerID,
                             long            InquireType, 
                             void MPTYPE    *UserVarPtr,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 void MFTYPE 
OdigReference(               MIL_ID          DigitizerID,
                             long            ReferenceType, 
                             long            ReferenceLevel,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 void MFTYPE 
OdigChannel(                 MIL_ID          DigitizerID,
                             long            Channel,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OdigSetLut(                  MIL_ID          DigitizerID, 
                             long            LutType, 
                             long            Start, 
                             long            NbElem, 
                             void MPTYPE    *LutData, 
                             long            SizeBit, 
                             long            LutResetType,
                             void MPTYPE    *ThreadContextPtr);


MFTYPE32 MDIGHOOKFCTPTR MFTYPE 
OdigHookFunction(            MIL_ID          DigitizerID,
                             long            HookType,
                             MDIGHOOKFCTPTR  HookHandlerPtr,
                             void MPTYPE    *UserDataPtr,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 OAPPHOOKFCTPTR MFTYPE 
OappHookFunction(            long            HookType,
                             OAPPHOOKFCTPTR  HookHandlerPtr,
                             void MPTYPE    *UserDataPtr,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 long MFTYPE 
OdispAlloc(                  MIL_ID          SystemID, 
                             long            DeviceNum, 
                             char MPTYPE    *DispFormat, 
                             long            InitFlag, 
                             MIL_ID MPTYPE  *IdVarPtr,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OdispSetLut(                 MIL_ID          DisplayID, 
                             long            LutType, 
                             long            Start,
                             long            NbElem, 
                             void           *LutData,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OdispOverlayKey(             MIL_ID          DisplayID, 
                             long            KeyMode, 
                             long            KeyCond, 
                             long            KeyMask,
                             long            KeyColor,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 void MFTYPE 
OdispControl(                MIL_ID          DisplayID,
                             long            ControlType, 
                             long            Value,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 long MFTYPE 
OdispInquire(                MIL_ID          DisplayID, 
                             long            InquireType, 
                             void MPTYPE    *UserVarPtr,
                             void MPTYPE    *ThreadContextPtr);
                                        
MFTYPE32 void MFTYPE 
OdispFree(                   MIL_ID          DisplayID,
                             void MPTYPE    *ThreadContextPtr);
                                                 
#ifndef MDISPHOOKFCTPTR

typedef MFTYPE32 long (MFTYPE MPTYPE *MDISPHOOKFCTPTR)(long         HookType,
                                                       MIL_ID       EventId,
                                                       void MPTYPE *UserPtr);
#endif


MFTYPE32 MDISPHOOKFCTPTR MFTYPE 
OdispHookFunction(           MIL_ID          DisplayID,
                             long            HookType,
                             MDISPHOOKFCTPTR HookHandlerPtr,
                             void MPTYPE    *UserDataPtr,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OdispZoom(                   MIL_ID          DisplayID, 
                             long            XFactor, 
                             long            YFactor,
                             void MPTYPE    *ThreadContextPtr);

MFTYPE32 void MFTYPE 
OdispPan(                    MIL_ID          DisplayID, 
                             long            XOffset, 
                             long            YOffset,
                             void MPTYPE    *ThreadContextPtr);

long MINTFTYPE 
CExtGrabHandler(             long            FHookType, 
                             MIL_ID          EventId, 
                             void MPTYPE    *FExpansionFlagPtr);

//==============================================================================
// ORION LowLevel prototypes
//==============================================================================

MFTYPE32 long MFTYPE 
OLLinit(                     MIL_ID          SystemId,
                             long            InitMode,
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLsysAlloc(                 long            SystemType,
                             long            SystemNum,
                             long            InitFlag,
                             MIL_ID         *SystemIdPtr, 
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLsysFree(                  MIL_ID          SystemId, 
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLsysLutSelectPalette(      MIL_ID          SystemId,
                             unsigned long   palette, 
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLsysLutSelectMode(         MIL_ID          SystemId,
                             unsigned long   mode, 
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLsysLutWrite(              MIL_ID                SystemId,
                             unsigned long         lut,
                             unsigned char MPTYPE *pBuffer,
                             unsigned long         length,
                             unsigned long         offset, 
                             void                 *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLsysLutRead(               MIL_ID                SystemId,
                             unsigned long         lut,
                             unsigned char MPTYPE *pBuffer,
                             unsigned long         length,
                             unsigned long         offset, 
                             void                 *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdigSetLut(                MIL_ID                SystemId,
                             long                  LutType,     
                             long                  Start,       
                             long                  NbElem,      
                             void                 *LutData,    
                             long                  SizeBits,    
                             long                  LutResetType,
                             void                 *OverlappedStruct);
   
MFTYPE32 long MFTYPE 
OLLdispOverlayKey(           MIL_ID          SystemId, 
                             long            lKeyMode, 
                             long            lKeyCond,
                             long            nKeyColorHigh, 
                             long            nKeyColorLow, 
                             void           *OverlappedStruct);
   
MFTYPE32 long MFTYPE 
OLLsysControl(               MIL_ID          SystemId,  
                             long            ControlType, 
                             long            ControlValue, 
                             void           *OverlappedStruct);

MFTYPE32 unsigned long MFTYPE 
OLLsysConfigAccess(          MIL_ID          SystemId, 
                             long            VendorId, 
                             long            DeviceId,
                             long            DeviceNumber,
                             long            OpperationFlag,
                             long            OpperationType,
                             long            Offset,
                             long            Size,
                             void           *UserDataPtr,
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdigAlloc(                 MIL_ID          SystemId, 
                             char MPTYPE    *DataFormat, 
                             long            InitFlag, 
                             void           *OverlappedStruct);
 
MFTYPE32 long MFTYPE 
OLLGetDcfFromDisk(           MIL_ID          SystemId, 
                             char MPTYPE    *DataFormat, 
                             long            InitFlag, 
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLSetDcfToHardWare(         MIL_ID          SystemId, 
                             long            InitFlag, 
                             void           *OverlappedStruct);


MFTYPE32 void MFTYPE 
OLLdigFree(                  MIL_ID          SystemId,
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdigControl(               MIL_ID          SystemId, 
                             long            ControlType, 
                             double          ControlValue,  
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdigChannel(               MIL_ID          SystemId, 
                             long            Channel, 
                             void           *OverlappedStruct);    

MFTYPE32 long MFTYPE 
OLLdigReference(             MIL_ID          SystemId, 
                             long            ReferenceType, 
                             long            ReferenceLevel, 
                             void           *OverlappedStruct);

MFTYPE32 void MFTYPE 
OLLdigGrabContinuous(        MIL_ID          SystemId, 
                             void           *OverlappedStruct);   

MFTYPE32 void MFTYPE 
OLLdigChangeIntStat(	        MIL_ID          SystemId, 
                             long			   Enable, 
                             void		     *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdigHookInterruptFromKernel(MIL_ID         SystemId, 
                              long           Enable, 
                              void          *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdigHookFunction(          MIL_ID          SystemId,
                             long            HookType,
                             MDIGHOOKFCTPTR  HookFunctionPtr,
                             void MPTYPE    *UserDataPtr, 
                             void           *OverlappedStruct);
                                                
MFTYPE32 long MFTYPE 
OLLdigGrabPosition(          MIL_ID          SystemId,   
                             long            SrcOffX,   
                             long            SrcOffY,   
                             long            SrcSizeX,   
                             long            SrcSizeY,  
                             long            DestSizeX,   
                             long            DestSizeY,   
                             long            DestPhysAddr, 
                             long            DestPitchX,  
                             long            DestPhysAddrU,   
                             long            DestPitchXU,   
                             long            DestPhysAddrV,  
                             long            DestPitchXV, 
                             void           *OverlappedStruct);
                                                
MFTYPE32 void MFTYPE 
OLLdigGrabSingle(            MIL_ID          SystemId, 
                             void           *OverlappedStruct);       

MFTYPE32 long MFTYPE 
OLLdigGrabStop(              MIL_ID          SystemId,
                             long            ImmediateStop,
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdigGrabWait(              MIL_ID          SystemId,
                             long            WaitFlag,
                             long            WaitMode,
                             long            WaitTimeout,
                             void           *OverlappedStruct);


MFTYPE32 long MFTYPE 
OLLsysInquire(               MIL_ID          SystemId,
                             long            InquireType, 
                             long           *UserDataPtr,
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdigInquire(               MIL_ID          SystemId,
                             long            InquireType, 
                             void           *pInquireValue,
                             void           *OverlappedStruct);


MFTYPE32 long MFTYPE 
OLLdigGrabStatus(            MIL_ID          SystemId, 
                             long            StatusFlag, 
                             void           *OverlappedStruct); 
MFTYPE32 long MFTYPE 
OLLdigGrab(                  MIL_ID         SystemId,       
                             void           *DestGrabBuffer,
                             void           *GrabControl,
                             void           *OverlappedStruct);

MFTYPE32 short MFTYPE 
OLLDmaInit(                  void);

MFTYPE32 short MFTYPE 
OLLDmaClose(                 void);

MFTYPE32 long MFTYPE 
OLLDmaBufferAlloc(           MIL_ID          SystemId,
                             unsigned long   xsize, 
                             unsigned long   ysize,
                             unsigned char   pixelsize, 
                             unsigned long   Alignment,
                             unsigned long   Prefetch,
                             unsigned long   Frontier,
                             unsigned long   AllocFlags,
                             unsigned long   *AllocFlagsOk,
                             void           *OverlappedStruct);
                                        
MFTYPE32 long MFTYPE 
OLLDmaBufferCreateId(        MIL_ID          SystemId, 
                             unsigned long   physaddr, 
                             unsigned long   xsize,
                             unsigned long   ysize, 
                             unsigned char   pixelsize, 
                             unsigned long   Alignment,
                             unsigned long   Prefetch,
                             unsigned long   Frontier,
                             unsigned long   AllocFlags,
                             unsigned long   *AllocFlagsOk,
                             void           *OverlappedStruct);
                                            
MFTYPE32 void  MPTYPE * MFTYPE 
OLLDmaBufferGetPtr(          MIL_ID          SystemId, 
                             long            bufid, 
                             void           *OverlappedStruct);
                                                  
MFTYPE32 short MFTYPE 
OLLDmaBufferFree(            MIL_ID          SystemId, 
                             long            bufid, 
                             void           *OverlappedStruct);

MFTYPE32 short MFTYPE  
OLLDmaBufferLock(            short           bufid);

MFTYPE32 short MFTYPE  
OLLDmaBufferUnlock(          short           bufid);

MFTYPE32 unsigned long MFTYPE 
OLLDmaBufferGetPhysAdr(      MIL_ID          SystemId, 
                             long            bufid, 
                             void           *OverlappedStruct);
                                                     
MFTYPE32 short MFTYPE  
OLLDmaBufferFreeId(          MIL_ID          SystemId, 
                             long            bufid, 
                             void           *OverlappedStruct);
                                             
MFTYPE32 long MFTYPE  
OLLDmaBufferAssignId(        MIL_ID                SystemId,
                             unsigned char MPTYPE *virtptr,
                             unsigned long         physaddr, 
                             unsigned long         xsize,
                             unsigned long         ysize, 
                             unsigned char         pixelsize, 
                             unsigned long         Alignment,
                             unsigned long         Prefetch,
                             unsigned long         Frontier,
                             unsigned long         AllocFlags,
                             unsigned long         *AllocFlagsOk,
                             void                 *OverlappedStruct);


MFTYPE32 long MFTYPE 
OLLdispAlloc (               MIL_ID          SystemId,
                             char MPTYPE    *DataFormat,
                             long            InitFlag, 
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdispControl(              MIL_ID          SystemId, 
                             long            ControlType, 
                             long            Value, 
                             void           *OverlappedStruct);

MFTYPE32 void MFTYPE 
OLLdispFree(                  MIL_ID          SystemId,
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLencoderInit(              MIL_ID          SystemId, 
                             long            ControlType,
                             long            Value, 
                             long            Format, 
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdispInquire(              MIL_ID          SystemId, 
                             long            InquireType,
                             void           *InquiredValue,
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdispSetLut(               MIL_ID                SystemId,
                             unsigned long         LutType, 
                             unsigned long MPTYPE *LutData, 
                             void                 *OverlappedStruct);
                                                
MFTYPE32 long MFTYPE 
OLLdispZoom(                 MIL_ID          SystemId, 
                             unsigned long   XFactor, 
                             unsigned long   YFactor, 
                             void           *OverlappedStruct);
                                              
MFTYPE32 long MFTYPE 
OLLdispPan(                  MIL_ID          SystemId, 
                             unsigned long   XOffset, 
                             unsigned long   YOffset, 
                             void           *OverlappedStruct);

MFTYPE32 long MFTYPE 
OLLdispKeyLut(               MIL_ID                SystemId,
                             unsigned long         LutType, 
                             void           *OverlappedStruct);

//==============================================================================
// internal prototype
//==============================================================================

MFTYPE32 unsigned long MFTYPE 
OLLPciBridge(                MIL_ID          SystemId,
                             void           *OverlappedStruct);

MFTYPE32 unsigned long MFTYPE 
OLLFindPCIDevice(            MIL_ID                SystemId,
                             unsigned long         DeviceID, 
                             unsigned long         VendorID, 
                             unsigned long         Index, 
                             unsigned long MPTYPE *PciId, 
                             void                 *OverlappedStruct);

MFTYPE32 void * MFTYPE 
OLLFindOrionStruc(          MIL_ID          SystemId, 
                             void           *OverlappedStruct);
   
MFTYPE32 long MFTYPE 
OLLFindOrionId(             long            SystemNum);

MFTYPE32 long MFTYPE 
OLLMapPhysicalMemory(        unsigned long                PhysAddr,
                             unsigned long                Limit,
                             void ORIONFARTYPE * MPTYPE *PointerToMemory,
                             void MPTYPE                 *Handle);
   
MFTYPE32 long MFTYPE 
OLLGeneralUnmapPhysicalMemory(void ORIONFARTYPE *PointerToMemory,
                              void MPTYPE        *Handle);

MFTYPE32 long MFTYPE 
OLLUnmapPhysicalMemory(      void MPTYPE    *Handle);

MFTYPE32 void MPTYPE * MFTYPE 
OLLMemGetPhysAddrFromLinearAddr(MIL_ID                SystemId, 
                                void MPTYPE          *LinearAddr,
                                unsigned long         LinearSize,
                                unsigned long MPTYPE *LinearStatus,
                                void                 *OverlappedStruct);

MFTYPE32 unsigned long MFTYPE 
OLLPciReadConfiguration(        MIL_ID                SystemID,
                                unsigned long         BusDevFunNum,
                                unsigned long         RegNum,
                                unsigned long        *UserPtr,
                                void                 *OverlappedStruct);

MFTYPE32 unsigned long MFTYPE 
OLLPciWriteConfiguration(       MIL_ID                SystemID,
                                unsigned long         BusDevFunNum, 
                                unsigned long         RegNum,
                                unsigned long         Data,
                                void                 *OverlappedStruct);
                                                        
MFTYPE32 unsigned long MFTYPE 
OLLsysWait(                     unsigned long         value);

MFTYPE32 unsigned long MFTYPE 
OLLregReadMga_32(               MIL_ID                SystemId,
                                unsigned long         offset,
                                short                 flag, 
                                void                 *OverlappedStruct);

MFTYPE32 void MFTYPE 
OLLregWriteMga_32(              MIL_ID                SystemId,
                                unsigned long         offset, 
                                unsigned long         data, 
                                void                 *OverlappedStruct);
   
MFTYPE32 unsigned short MFTYPE 
OLLregReadMga_16(               MIL_ID                SystemId,
                                unsigned long         offset,
                                short                 flag, 
                                void                 *OverlappedStruct);

MFTYPE32 void MFTYPE 
OLLregWriteMga_16(              MIL_ID                SystemId,
                                unsigned long         offset, 
                                unsigned short        data, 
                                void                 *OverlappedStruct);
   
MFTYPE32 unsigned char MFTYPE 
OLLregReadMga_8(                MIL_ID                SystemId,
                                unsigned long         offset,
                                short                 flag, 
                                void                 *OverlappedStruct);

MFTYPE32 void MFTYPE
OLLregWriteMga_8(               MIL_ID                SystemId,
                                unsigned long         offset, 
                                unsigned char         data, 
                                void                 *OverlappedStruct);
   
MFTYPE32 unsigned long MFTYPE 
OLLLutEmulate(                  unsigned long         Register, 
                                unsigned long        *ShadowPtr); 

MFTYPE32 unsigned long MFTYPE 
OLLMGAWriteRegister(            MIL_ID                SystemId, 
                                unsigned long         Register, 
                                unsigned long         Field, 
                                unsigned long         Data, 
                                void                 *OverlappedStruct);

MFTYPE32 unsigned long MFTYPE 
OLLMGAReadRegister(             MIL_ID                SystemId, 
                                unsigned long         Register, 
                                unsigned long         Field, 
                                unsigned long        *Data, 
                                unsigned long         Mode,
                                void                 *OverlappedStruct);
                        

MFTYPE32 unsigned long MFTYPE    
OLLrestoreShadowRegister(       MIL_ID                SystemId, 
                                unsigned long         Register, 
                                unsigned long         Field, 
                                void                 *OverlappedStruct);

MFTYPE32 unsigned long MFTYPE 
OLLrestoreShadowChip(           MIL_ID                SystemId, 
                                unsigned long         Chip, 
                                void                 *OverlappedStruct);
MFTYPE32 unsigned long MFTYPE 
OLLReadAllRegister(             MIL_ID                SystemId, 
                                unsigned long         Chip, 
                                void                 *OverlappedStruct);
MFTYPE32 unsigned long MFTYPE 
OLLReadAllRegister(             MIL_ID                SystemId, 
                                unsigned long         Chip, 
                                void                 *OverlappedStruct);


#ifdef __cplusplus
}
#endif


#endif

