/********************************************************************
 *
 * Grab an image, and optionally save it.
 *
 * (Note that if you are running in single screen mode under Windows,
 * you will not see anything in the Genesis image buffer until you
 * enable keying with a separate program).
 *
 ********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "imapi.h"

void main(int argc, char **argv)
{
    long Device;            /* Genesis device */
    long Thread;            /* Thread to execute all functions */
    long DispBuf;           /* Buffer allocated in display memory */
    long ScreenBuf;         /* Display buffer full size of screen */
    long Camera;            /* Camera */
    long SizeX, SizeY;      /* Image Size */
    char Error[IM_ERR_SIZE];/* String to hold error message */
    int Save = 0, i;        /* Miscellaneous variables */

/* Check arguments */
    if (argc > 1 && *argv[1] == '?')
    {
        printf("Usage: GRAB [file.tif] [-x] [-y]\n");
        printf("       -x size\t Image X size\n");
        printf("       -y size\t Image Y size\n");
        exit(1);
    }

    if (argc > 1 && *argv[1] != '-')
        Save = 1;

/* Allocate the board, a thread and a camera */
    imDevAlloc(0, 0, NULL, IM_DEFAULT, &Device);
    imThrAlloc(Device, 0, &Thread);
    imCamAlloc(Thread, NULL, IM_DEFAULT, &Camera);

/* Determine the image size */
    imCamInquire(Thread, Camera, IM_DIG_SIZE_X, &SizeX);
    imCamInquire(Thread, Camera, IM_DIG_SIZE_Y, &SizeY);

/* Check if the user requested a different size */
    for (i = 1; i < argc; i++)
    {
        if (!strcmp(argv[i], "-x"))
            sscanf(argv[i+1], "%li", &SizeX);
        else if (!strcmp(argv[i], "-y"))
            sscanf(argv[i+1], "%li", &SizeY);
    }

    printf("Image size is %lix%li\n", SizeX, SizeY);

/* Allocate a full screen display buffer and clear it */
    imBufChild(Thread, IM_DISP, 0, 0, IM_ALL, IM_ALL, &ScreenBuf);
    imBufClear(Thread, ScreenBuf, 0, 0);

/* Allocate a buffer at a specific location on the display */
    imBufChild(Thread, IM_DISP, 0, 0, SizeX, SizeY, &DispBuf);

/* Start a continuous grab into the display buffer */
    imDigGrab(Thread, 0, Camera, DispBuf, IM_CONTINUOUS, 0, 0);

/* Halt when the user hits Enter */
    printf("Press <Enter> to stop");
    getchar();
    imThrHalt(Thread, IM_FRAME);

/* Optionally save the image */
    if (Save)
    {
        if (imCamInquire(Thread, Camera, IM_DIG_NUM_BANDS, NULL) == 3)
        {
        /* Save all bands if colour */
            imBufSave(Thread, argv[1], IM_TIFF, DispBuf);
        }
        else
        {
        /* Save just the first band of the display if not colour */
            long MonoBuf;

            imBufChildBand(Thread, DispBuf, 0, &MonoBuf);
            imBufSave(Thread, argv[1], IM_TIFF, MonoBuf);
            imBufFree(Thread, MonoBuf);
        }
    }

/* Wait for everything to finish, then check for errors */
    imSyncHost(Thread, 0, IM_COMPLETED);
    if (imAppGetError(IM_ERR_MSG_FUNC, Error))
        printf("%s\n", Error);

/* Clean up */
    imCamFree(Thread, Camera);
    imBufFree(Thread, DispBuf);
    imBufFree(Thread, ScreenBuf);
    imThrFree(Thread);
    imDevFree(Device);
}

