; *******************************************************************
; *
; * COPYRIGHT (c) 1995 Matrox Electronic Systems Ltd.
; * All Rights Reserved
; *
; *******************************************************************
;
; *******************************************************************
; *
; * Synopsis:        This script will execute convolution operation
; *                  on integer images
; *
; * Used Functions:  imIntConvolve 
; *
; * Designed:        AJONG 
; *
; * Date:            17/05/95
; *
; * Notes:           This Example will demonstrate 
; *                  1) Apply convolution with a
; *                     predefined kernel, 
; *                  2) Create a user defined kernel and perform  
; *                     convolution.
; *
; *******************************************************************


   long    dev
   long    thread
   long    SrcBuf
   long    DispBuf1
   long    DispBuf2
   long    Buf1
   long    Buf2
   long    ConvBuf
   long    ShortBuf1
   long    ShortBuf2
   long    errorNum
   setstr  errorMsg "                                                             "
   setstr  errorFunc "                                                            "
   

   
   ; Allocate the device and thread
   imDevAlloc 0 0 null IM_DEFAULT &dev
   imThrAlloc dev 0 &thread

   
   ; Alloc the ubyte buffers
   imBufAlloc thread 512 480 1 IM_UBYTE IM_PROC &SrcBuf
   imBufAlloc thread 512 480 1 IM_UBYTE IM_PROC &Buf1
   imBufAlloc thread 512 480 1 IM_UBYTE IM_PROC &Buf2

   ; Alloc the Short buffers
   imBufAlloc thread 512 480 1 IM_SHORT IM_PROC &ShortBuf1
   imBufAlloc thread 512 480 1 IM_SHORT IM_PROC &ShortBuf2
   

   ; Alloc the display child
   imBufChild thread IM_DISP_RED 0   0 512 480 &DispBuf1
   imBufChild thread IM_DISP_RED 512 0 512 480 &DispBuf2
   

   ; Check and Display any errors which might have occured
   execcl "test_err.cl" thread
   
   
   ; load the image "hwteam.tif"
   imBufLoad thread "hwteam.tif" IM_TIFF SrcBuf
   imBufCopy thread SrcBuf DispBuf1 0 0
   pause "Original image... Press any key to continue" 


   
   ;;-------------------------------------
   ;; Convolve with a pre-defined kernel |
   ;;-------------------------------------
   
   ; use source buffer as a control buffer
   imBufPutField thread SrcBuf IM_CTL_OVERSCAN IM_REPLACE
   imBufPutField thread SrcBuf IM_CTL_OVERSCAN_VAL 0x0

   
   ; perform a convolution with a pre-defined kernel
   imIntConvolve thread SrcBuf Buf2 IM_HORIZ_EDGE SrcBuf 0      
   imBufCopy thread Buf2 DispBuf2 0 0      
   pause "IM_HORIZ_EDGE detection...  Press any key to continue."

   ; perform a convolution with a pre-defined kernel
   imIntConvolve thread SrcBuf Buf2 IM_SOBEL_EDGE SrcBuf 0      
   imBufCopy thread Buf2 DispBuf2 0 0      
   pause "IM_SOBEL_EDGE detection...  Press any key to continue."

   ; perform a convolution with a pre-defined kernel
   imIntConvolve thread SrcBuf Buf2 IM_SHARPEN SrcBuf 0      
   imBufCopy thread Buf2 DispBuf2 0 0      
   pause "IM_SHARPEN sharpenning...  Press any key to continue."



   
   ;;--------------------------------------
   ;; Convolve with a user-defined kernel |
   ;;--------------------------------------

   ;
   ; Let's now create an arbitary kernel 7 by 7 with type SHORT (2bytes)...
   ;

   allocarray con_ker_array (7*7*2)
   execcl "con7_7.cl"
   imBufAlloc thread 7 7 1 IM_SHORT IM_PROC &ConvBuf
   imBufPut thread ConvBuf [getarray con_ker_array]
   
   
   ; put kernel fields...
   imBufPutField thread ConvBuf IM_KER_SHIFT     6
   imBufPutField thread ConvBuf IM_KER_ABSOLUTE  IM_ENABLE


   ; 8->16 conversion
   imIntConvert thread SrcBuf ShortBuf1 IM_DEFAULT 0
   
   ;
   ; perform a convolution with the user-defined buffer
   ; Multiply each pixel with "-4" and then shift right 2 bit
   ; and the absolute value.
   ;
   imIntConvolve thread ShortBuf1 ShortBuf2 ConvBuf SrcBuf 0 
   
   ; 16-> 8 conversion
   imIntConvert thread ShortBuf2 Buf2 IM_DEFAULT 0
        
   imBufCopy thread Buf2 DispBuf2 0 0      
   pause "Convolution with a user defined kernel (Blurring).  Press any key to continue."

     
   imIntMac2 thread ShortBuf1 ShortBuf2 ShortBuf2 2 -1 0 0  
     
   ; 16-> 8 conversion
   imIntConvert thread ShortBuf2 Buf2 IM_CLIP 0
        
   imBufCopy thread Buf2 DispBuf2 0 0      
   pause "Sharpenning using output of (Blurring).  Press any key to continue."
     
     
   
   ;
   ; Free up all buffers threads and deallocate the device
   ;
   imBufFree thread ShortBuf1
   imBufFree thread ShortBuf2
   imBufFree thread ConvBuf
   imBufFree thread DispBuf2
   imBufFree thread DispBuf1
   imBufFree thread SrcBuf 
   imBufFree thread Buf1
   imBufFree thread Buf2
   imThrFree thread
   imDevFree dev
   
   
   ; free all used variables
   freearray con_ker_array
   freedev
   freethread
   freeSrcBuf
   freeDispBuf1
   freeDispBuf2
   freeConvBuf
   freeBuf1
   freeBuf2
   freeShortBuf1
   freeShortBuf2
   freeerrorNum
   clearstr errorMsg 
   clearstr errorFunc
