; *******************************************************************
; *
; * COPYRIGHT (c) 1995 Matrox Electronic Systems Ltd.
; * All Rights Reserved
; *
; *******************************************************************
;
; *******************************************************************
; *
; * Synopsis:        This script will generate the Power Spectrum of an image
; *                  The Power Spectrum which is 10Log(Power) will be  displayed
; *                  as 40log(Power) so that a dynamic range of 63 db will
; *                  translate to a dynamic range of 256 . This example will also
; *                  use the imIntFindExtreme function to locate the Peak
; *                  of the power Spectrum and then set the peak at 255 
; *                  
; *
; * Used Functions:  imIntFFT, imFloatConvert, imFloatDyadic, imFloatUnary
; *                  imFloatMonadic, imIntMonadic, imIntFindExtreme
; *
; * Designed:        VTARANTI 
; *
; * Date:            17/05/95
; *
; * Notes:           
; *
; *******************************************************************

   long    dev
   long    thread
   long    SrcBuf
   long    OutBuf
   long    RealBuf
   long    ImagBuf
   long    fRealBuf
   long    fImagBuf
   
   long    maxPix
   long    Disp1Buf
   long    Disp2Buf
   long    errorNum
   setstr  errorMsg "                                                             "
   setstr  errorFunc "                                                            "
   

   
   ; Allocate the device and thread
   imdevalloc 0 0 null im_default &dev
   imthralloc dev 0 &thread

   ; Alloc the buffers
   imbufalloc thread 512 512 1 im_ubyte im_proc &SrcBuf
   imbufalloc thread 512 512 1 im_ubyte im_proc &OutBuf
   imbufalloc thread 512 512 1 im_long im_proc &RealBuf
   imbufalloc thread 512 512 1 im_long im_proc &ImagBuf
   imbufalloc thread 512 512 1 im_float im_proc &fRealBuf
   imbufalloc thread 512 512 1 im_float im_proc &fImagBuf
   

   ; Alloc the display child
   imbufchild thread im_disp_red 0 0 512 512 &Disp1Buf
   imbufchild thread im_disp_red 512 0 512 512 &Disp2Buf
   

   ; load the image 
   imbufload thread "cirtstv2.tif" im_tiff SrcBuf
   imbufcopy thread SrcBuf Disp1Buf 0 0
   
   
   
   ; Check and Display any errors which have occured
   if ( [imthrgeterror thread  im_err_msg [getstrptr errorMsg]] != 0)
   {
      seterrorNum [imthrgeterror thread  im_err_func [getstrptr errorFunc]]
      echoln " The Error Code is : "
      echo [nbrtostr errorNum 10]
      echoln " The Logged Error is : "
      echo [getstr errorMsg]
      echoln " The offended function is : "
      echo [getstr errorFunc]
      return
   }
   
   ; use source buffer as a control buffer
   imbufputfield thread SrcBuf IM_CTL_DIRECTION IM_FORWARD
   imbufputfield thread SrcBuf IM_CTL_NORMALIZE IM_ENABLE


   ;
   ; Restart From initial image 4 times
   ;
   for index 0 4 1
   {
      ; Convert the 8 bit image to a 32 bit image
      imintconvert thread SrcBuf RealBuf im_default  0
      
      ; zero the imaginary buffer
      imbufclear thread ImagBuf 0x0 0
      
      ; add 15 fractional bits  to the Real Buffer by shifting the input 8 bits to 
      ; the right by 15 bits
      imintmonadic thread RealBuf 15 RealBuf im_shift 0
      
      ;perform fft use Src Buffer as Control Buffer
      imintfft thread RealBuf ImagBuf RealBuf ImagBuf SrcBuf 0
      
      ;Convert the Real and imaginary buffers to floats
      imfloatconvert thread RealBuf fRealBuf im_truncate 0
      imfloatconvert thread ImagBuf fImagBuf im_truncate 0
      
      ; Calculate the Power Specturm = real*real + imag*imag
      imfloatdyadic thread fRealBuf fImagBuf fRealBuf  im_square_add 0 
      
      ; get the log of the power spectrum  (note: the log is in base e=2.71828)
      imfloatunary thread fRealBuf fRealBuf im_log  0
      
      ; multiply the power spectrum by 40 (and by 0.434294482 to convert log base e to base 10)
      ; ie. multiply by 17.0 . This will give the power spectrum a good dynamic range for
      ; displaying in 8 bit buffers
      imfloatmonadic thread fRealBuf 17.0 fRealBuf im_mult 0 
      
      ; convert the float buffers to long 32 bit buffer
      imfloatconvert thread fRealBuf RealBuf im_round 0
      
      ; find the peak value in the long buffer
      imintfindextreme thread RealBuf SrcBuf im_max_pixel 0
      seterrorNum [imbufgetfield thread SrcBuf im_res_max_pixel &maxPix]
      
      ;Set the MaxPix to 255
      imintmonadic thread RealBuf (255-maxPix) RealBuf im_add 0
      
      ;convert to 8 bit display buffer and clipping to range of the display buffer
      imintconvert thread  RealBuf OutBuf im_clip 0
      
      ; display the output
      imbufcopy thread OutBuf Disp2Buf 0 0
      
      echoln " Loop Count: "
      echo [nbrtostr _index 10]
      
      
      ; Check and Display any errors which have occured
      if ( [imthrgeterror thread  im_err_msg [getstrptr errorMsg]] != 0)
      {
         seterrorNum [imthrgeterror thread  im_err_func [getstrptr errorFunc]]
         echoln " The Error Code is : "
         echo [nbrtostr errorNum 10]
         echoln " The Logged Error is : "
         echo [getstr errorMsg]
         echoln " The offended function is : "
         echo [getstr errorFunc]
         return
      }
      
   }
   
   ; Free up all buffers threads and deallocate the device
   imbuffree thread Disp1Buf
   imbuffree thread Disp2Buf
   imbuffree thread SrcBuf 
   imbuffree thread OutBuf 
   imbuffree thread RealBuf
   imbuffree thread ImagBuf
   imbuffree thread fRealBuf
   imbuffree thread fImagBuf
   imthrfree thread
   imdevfree dev
   
   
   ; free all used variables
   freedev
   freethread
   freeSrcBuf
   freeOutBuf
   freeRealBuf
   freeImagBuf
   freefRealBuf
   freefImagBuf
   freemaxPix
   freeDisp1Buf
   freeDisp2Buf
   freeerrorNum
   clearstr errorMsg 
   clearstr errorFunc
