; *******************************************************************
; *
; * COPYRIGHT (c) 1995 Matrox Electronic Systems Ltd.
; * All Rights Reserved
; *
; *******************************************************************
;
; *******************************************************************
; *
; * Synopsis:        This script will perform the game of life operation
; *
; * Used Functions:  imIntConnectMap
; *
; * Designed:        VTARANTI 
; *
; * Date:            17/05/95
; *
; * Notes:           This Example will demonstrate how to load a Lut
; *                  buffer with the interpreter.
; *
; *******************************************************************

   long    dev
   long    thread
   long    SrcBuf
   long    DispBuf
   long    Buf1
   long    Buf2
   long    lut
   long    errorNum
   setstr  errorMsg "                                                             "
   setstr  errorFunc "                                                            "
   

   ;; load the lut
   execcl "lut_life"
   
   ; Allocate the device and thread
   imdevalloc 0 0 null im_default &dev
   imthralloc dev 0 &thread

   ; Alloc the buffers
   imbufalloc thread 512 480 1 im_ubyte im_proc &SrcBuf
   imbufalloc thread 512 480 1 im_ubyte im_proc &Buf1
   imbufalloc thread 512 480 1 im_ubyte im_proc &Buf2
   imbufalloc thread 512 1 1 im_short im_proc &lut

   ; Alloc the display child
   imbufchild thread im_disp_red 0 0 512 480 &DispBuf
   

   ; load the image 
   imbufload thread "man&dog.tif" im_tiff SrcBuf
   
   
   ; Check and Display any errors which have occured
   if ( [imthrgeterror thread  im_err_msg [getstrptr errorMsg]] != 0)
   {
      seterrorNum [imthrgeterror thread  im_err_func [getstrptr errorFunc]]
      echoln " The Error Code is : "
      echo [nbrtostr errorNum 10]
      echoln " The Logged Error is : "
      echo [getstr errorMsg]
      echoln " The offended function is : "
      echo [getstr errorFunc]
      return
   }
   
   ; use source buffer as a control buffer
   imbufputfield thread SrcBuf IM_CTL_OVERSCAN IM_REPLACE
   imbufputfield thread SrcBuf IM_CTL_OVERSCAN_VAL 0x0

   ;write the lut to memory
   imbufput thread lut [getarray lut_life]

   ;
   ; Restart From initial image 4 times
   ;
   for index2 0 4 1
   {
      ; perform the first Connect Map operation
      imintconnectmap thread SrcBuf Buf1 lut SrcBuf  0
      imbufcopy thread Buf1 DispBuf 0 0

      ;
      ; Iterate two hundred iterations
      ;
      for index 0 20 1
      {
         ; now flip back and fourth
         imintconnectmap thread Buf1 Buf2 lut SrcBuf 0
         imbufcopy thread Buf2 DispBuf 0 0

         imintconnectmap thread Buf2 Buf1 lut SrcBuf 0
         imbufcopy thread Buf1 DispBuf 0 0

         ; Sync with thread (Not Really needed)
         seterrorNum [imsynchost thread 0 IM_COMPLETED ]
         
         ; Display Counters
         echoln " Inner Loop Count: "
         echoln [nbrtostr _index 10]
         echoln " Outer Loop Count: "
         echo [nbrtostr _index2 10]
      }
   }
   
   ;
   ; Free up all buffers threads and deallocate the device
   ;
   imbuffree thread DispBuf
   imbuffree thread SrcBuf 
   imbuffree thread Buf1
   imbuffree thread Buf2
   imbuffree thread lut
   imthrfree thread
   imdevfree dev
   
   ; free the interpreters lut table
   freearray lut_life 
   
   ; free all used variables
   freedev
   freethread
   freeSrcBuf
   freeDispBuf
   freeBuf1
   freeBuf2
   freelut
   freeerrorNum
   clearstr errorMsg 
   clearstr errorFunc
