# --------------------------------------------------------------------------
function GetEnviron(var)
{
   if (var in ENVIRON)
      return(ENVIRON[var])

   _var_ = tolower(var)
   for (varndx in ENVIRON)
      {
      if (tolower(varndx) == _var_)
         return(ENVIRON[varndx])
      }

   return("")
}

# Create response file for UNIX C80 Compliler

BEGIN {

   if (ARGC < 3)
      {
      printf "gawk -f ti_tools.awk option_file out_file"
      exit 1
      }

   optionFile = ARGV[1]
   ARGV[1] = ""

   outFile = ARGV[2]
   ARGV[2] = ""

   srcDir   = ""
   cDir     = ""
   aDir     = ""
   _options = ""
   objFile  = ""
   srcFile  = ""

   CurrentWorkingDir = tolower(getcwd())

   split(CurrentWorkingDir, CWD, ":")

   if ((pathProject = tolower(GetEnviron("PATH_PROJECT"))) != "")
      {
      # Remove DOS drive letter in CurrentWorkingDir
      srcDir = sprintf("set SRCDIR=\"${PROJECT}%s\\\"\n", substr(CurrentWorkingDir,3))
      }
   else
      {
      printf "ERROR: Environment variable PATH_PROJECT undefined!\n"
      exit 1
      }


   if (srcDir != "")
      {
      gsub(/\\/, "/", srcDir)
      printf "%s", srcDir >outFile
      }

   if (getline<optionFile > 0)
      {
      # Get type
      tiType = tolower($0)
      gsub(/ /, "", tiType)

      if (tiType != "mvpar" && tiType != "mvplnk")
         {
         if (GetEnviron("C_DIR") != "")
            {
            nbf = split(tolower(GetEnviron("C_DIR")), c_dir, ";")

            cDir = sprintf("setenv C_DIR \"")

            for (i=1; i<nbf; i++)
               cDir = sprintf("%s${PROJECT}%s;", cDir, c_dir[i])

            cDir = sprintf("%s.;${C_DIR}\"\n", cDir)

            aDir = sprintf("setenv A_DIR \"${C_DIR}\"\n")
            }

         if (cDir != "")
            {
            # Remove DOS drive letter in cDir
            gsub(substr(pathProject,1,2), "", cDir)
            gsub(/\\/, "/", cDir)
            printf "%s", cDir >outFile
            }

         if (aDir != "")
            {
            # Remove DOS drive letter in aDir
            gsub(substr(pathProject,1,2), "", aDir)
            gsub(/\\/, "/", aDir)
            printf "%s", aDir >outFile
            }

         addVersion = ""

         while (getline<optionFile > 0)
            {
            dOptionStr = $1
            gsub(/ /, "")
            srcFile = $0

            if (match($0, /\/r/) == 1)
               {
               gsub(/\/r/, "/r ")
               split($0, rspFile, " ")
               while (getline<rspFile[2] > 0)
                  {
                  gsub(/\\/, "/")

                  if (((tiType == "mpcl") || (tiType == "mpasm") || (tiType == "ppcl")) && (index($0, "-v") != 0))
                     {
#                     if (tiType == "ppcl")
#                        addVersion = "-v3"

                     # Unsupported switch for mpcl and mpasm, remove it from options
                     gsub(/-v2 /, "")
                     gsub(/-v3 /, "")
                     gsub(/-v2/, "")
                     gsub(/-v3/, "")
                     }

                  if ((tiType == "ppcl") && (index($0, "-mv") != 0))
                     {
                     # Unsupported switch for ppcl, remove it from options
                     gsub(/-mv /, "")
                     gsub(/-mv/, "")
                     }

                  printf "%s\n", $0 >outFile
                  }

               close(rspFile[2])
               }

            if (match($0, /-g/) == 1)
               {
               _options = sprintf("%s %s", _options, "-g")
               }

            if (match($0, /-s/) == 1)
               {
               _options = sprintf("%s %s", _options, "-s")
               }

            if (match($0, /-d/) == 1 || match($0, /-o/) == 1)
               {
               _options = sprintf("%s %s", _options, dOptionStr)
               }


            if (match($0, /-fo/) == 1 && (tiType == "mpasm" || tiType == "ppasm"))
               {
               # ASM only
               compiler = "ASM"
               gsub(/-fo/, "-fo ")
               gsub(/\\/, "/")
               split($0, objdir, " ")
               printf "set OBJDIR=%s\n", objdir[2] >outFile
               }

            if (match($0, /-fr/) == 1 && (tiType == "mpcl" || tiType == "ppcl"))
               {
               # C only
               compiler = "C"
               gsub(/-fr/, "-fr ")
               gsub(/\\/, "/")
               _options = sprintf("%s %s", _options, $0)
               }

            if (match($0, /-eo/) == 1 && (tiType == "mpcl" || tiType == "ppcl"))
               {
               # C only
               compiler = "C"
               gsub(/-eo/, "-eo ")
               gsub(/\\/, "/")
               _options = sprintf("%s %s", _options, $0)
               }

            if (match($0, /-mv/) == 1)
               {
               # C only
               compiler = "C"

               if (tiType == "ppcl")
                  volatileSwitch = ""     # Unsupported switch for ppcl
               else
                  volatileSwitch = "-mv"

               gsub(/\\/, "/")
               _options = sprintf("%s %s", _options, volatileSwitch)
               }
            }

         if (_options != "")
            {
            if (compiler == "ASM")
               gsub(/-g/, "-s", _options)

            printf "setenv %s_OPTION \"${%s_OPTION}%s\"\n", compiler, compiler, _options >outFile
            }

         if (length(pathProject) == 2)
            tmpDir = "${PROJECT}/sw/usr/${USER}"
         else
            {
            tmpDir = "${PROJECT}/"pathProject"/usr/${USER}"
            gsub(substr(pathProject,1,2), "", tmpDir)
            gsub(/\\/, "/", tmpDir)
            gsub(/\/\//, "/", tmpDir)
            }

         if (compiler == "C")
            printf "setenv %s_OPTION \"${%s_OPTION} -ft %s %s\"\n", compiler, compiler, tmpDir, addVersion >outFile

         printf "setenv TMP \"%s\"\n", tmpDir >outFile

         if (srcFile != "")
            {
            gsub(/\\/, "/", srcFile)
            printf "set SRCFILE=\"%s\"\n", srcFile >outFile
            }
         }
      else if (tiType == "mvpar")
         {
         printf "set AR_OPTION=\"" >outFile
         while (getline<optionFile > 0)
            {
            gsub(/\\/, "/")
            gsub(/\-\+/, "")     # Remove -+, which is used by most PC librarian
            _options = objFile
            objFile  = srcFile
            srcFile  = $0
            }

         if (_options != "")
            printf "%s", _options >outFile

         printf "\"\n" >outFile

         if (objFile != "")
            {
            # If absolute specified, translate it to a UNIX path
            if (index(objFile, ":") != 0)
               {
               if (index(objFile, "/") == index(objFile, ":")+1)
                  {
                  split(objFile, tmpstr, ":")
                  objFile = "${PROJECT}"tmpstr[2]
                  }
               else
                  {
                  split(objFile, tmpstr, ":")
                  objFile = sprintf("${PROJECT}%s\\%s", tolower(CWD[2]), tmpstr[2])
                  gsub(/\\/, "/", objFile)
                  }
               }
            else if (index(objFile, "/") == 1)
               {
               objFile = "${PROJECT}"objFile
               }

            printf "set OBJFILE=\"%s\"\n", objFile >outFile
            }

         if (srcFile != "")
            printf "set SRCFILE=\"%s\"\n", srcFile >outFile
         }
      else if (tiType == "mvplnk")
         {
         if (GetEnviron("C_DIR") != "")
            {
            nbf = split(tolower(GetEnviron("C_DIR")), c_dir, ";")

            cDir = sprintf("setenv C_DIR \"${C_DIR}")

            for (i=1; i<nbf; i++)
               cDir = sprintf("%s;${PROJECT}%s", cDir, c_dir[i])

            cDir = sprintf("%s;.\"\n", cDir)
            }

         if (cDir != "")
            {
            # Remove DOS drive letter in cDir
            gsub(substr(pathProject,1,2), "", cDir)
            gsub(/\\/, "/", cDir)
            printf "%s", cDir >outFile
            }

         if (getline<optionFile > 0)
            {
            srcFile = $0
            gsub(/ /, "", srcFile)
            gsub(/\\/, "/", srcFile)
            printf "set SRCFILE=\"%s\"\n", srcFile >outFile
            }
         }

      close(optionFile)
      close(outFile)
      }
}
